﻿/*****************************************************************************
 * Copyright (C) 2014 Microchip Technology Inc. and its subsidiaries
 * (Microchip).  All rights reserved.
 * 
 * You are permitted to use the software and its derivatives with Microchip
 * products. See the license agreement accompanying this software, if any,
 * for additional info regarding your rights and obligations.
 * 
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED “AS IS” WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A
 * PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE
 * LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY,
 * CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY FOR ANY
 * DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY
 * INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, OR OTHER SIMILAR
 * COSTS. To the fullest extend allowed by law, Microchip and its licensors
 * liability will not exceed the amount of fees that you paid directly to
 * Microchip to use this software or $1,000, whichever is greater.
 * 
 *****************************************************************************
 * Revision history:
 * 22.10.2013   Mihai Cuciuc    0.1     Communication with USB device functional,
 *                                      all DALI tests implemented.
 * 22.01.2014   Oana Barahtian  1.0     Changed the application to a multi-
 *                                      threaded model, restructured user
 *                                      interface and added functionality.
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;   // DllImport

namespace DALITestApplication.Classes
{
    // Types of USB data packets to be transmitted
    enum USB_PACKETS
    {
        TYPE_NOTHING,                       // Nothing to do
        TYPE_PING,                          // Ask for a reply in order to test the communication
        TYPE_DALI_COMMAND,                  // Send command on the DALI bus
        TYPE_STATE_QUERY,                   // [DEBUG] Ask the DALI state machine what state it's in
        TYPE_DALI_COMMAND_SEQUENCE,         // Send group of commands on the DALI bus
        TYPE_PING_DELAY,                    // Ask the device for a delayed reply (for accurate PC application timing)
        TYPE_SEND_DATA_TIMING_LIST,         // Send arbitrary waveform on the DALI bus
        TYPE_RESET_TIMING_LIST,             // Prepare to load arbitrary waveform data
        TYPE_LOAD_DATA_TIMING_LIST,         // Load arbitrary waveform data
        TYPE_SET_RECEIVE_TIMING_LIST,       // Enable/Disable waveform capture vs. decoded protocol
        TYPE_READ_FLAGS,                    // [DEBUG] Read DALI state machine flags
        TYPE_ENABLE_DEBUG_PACKETS,          // Enable/Disable the reception of debug data (state transitions)
        TYPE_GET_FW_VERSION                 // Request firmware version
    };

    // Types of USB data packets that can be received
    enum USB_REPLIES
    {
        REPLY_NOTHING,                      // Nothing done
        REPLY_PING,                         // Reply to ping request
        REPLY_DALI_ANSWER,                  // Confirmation of sent DALI command and DALI reply (if available)
        REPLY_STATE_QUERY,                  // [DEBUG] State that the DALI state machine is in
        REPLY_DALI_COMMAND_SEQUENCE,        // Confirmation of sent DALI group of commands and DALI replies (if available)
        REPLY_PING_DELAY,                   // Delayed reply
        REPLY_SEND_DATA_TIMING_LIST,        // Confirmation that arbitrary waveform is being sent on the DALI bus
        REPLY_RESET_TIMING_LIST,            // Confirmation of preparation to load arbitrary waveform data
        REPLY_LOAD_DATA_TIMING_LIST,        // Confirmation of loading arbitrary waveform data
        REPLY_SET_RECEIVE_TIMING_LIST,      // Confirmation of selection of waveform capture vs decoded protocol
        UNSOL_RECEIVE_TIMING_LIST,          // Captured waveform data from the DALI bus
        UNSOL_RECEIVE_DATA,                 // Captured decoded protocol data from the DALI bus
        REPLY_READ_FLAGS,                   // [DEBUG] DALI state machine flags
        UNSOL_STATE_CHANGE,                 // [DEBUG] State machine transition or flag being toggled
        REPLY_ENABLE_DEBUG_PACKETS,         // Confirmation of selection of debug data
        REPLY_GET_FW_VERSION                // Reply with firmware version
    };

    class Utils : Messages
    {
        // These functions are being used to communicate to the USB to DALI Interface hardware.
        // They are defined in hidapi.dll which this project links to.
        [DllImport("hidapi.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int DaliOpen();

        [DllImport("hidapi.dll", CallingConvention = CallingConvention.Cdecl)]
        unsafe public static extern int DaliWrite(byte* buf);

        [DllImport("hidapi.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int DaliGetManufacturerString(StringBuilder buf);

        [DllImport("hidapi.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int DaliGetProductString(StringBuilder buf);

        [DllImport("hidapi.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int DaliGetSerialNumberString(StringBuilder buf);

        [DllImport("hidapi.dll", CallingConvention = CallingConvention.Cdecl)]
        unsafe public static extern int DaliRead(byte* buf, int timeout);

        [DllImport("hidapi.dll", CallingConvention = CallingConvention.Cdecl)]

        public static extern int DaliClose();

        // This variable is 1 when the USB to DALI Interface board has been properly detected
        // and initialised, and 0 otherwise. It is used as a flag to not allow the application
        // to try to communicate with non-existent hardware.
        public int initialised = 0;

        // This function parses the numeric command received as the parameters and returns the
        // human-readable text of the command. It also does address decoding and parameter
        // parsing.
        public string decode(Byte addr, Byte data)
        {
            string s;
            s = "";
            if (addr == 0xA1)
            {
                s += "TERMINATE ";
                if (data != 0x00)
                {
                    s += "(malformed)";
                }
            }
            else if (addr == 0xA3)
            {
                s += "DATA TRANSFER REGISTER (DTR) = " + data.ToString();
            }
            else if (addr == 0xA5)
            {
                s += "INITIALISE " + data.ToString();
            }
            else if (addr == 0xA7)
            {
                s += "RANDOMISE ";
                if (data != 0x00)
                {
                    s += "(malformed)";
                }
            }
            else if (addr == 0xA9)
            {
                s += "COMPARE ";
                if (data != 0x00)
                {
                    s += "(malformed)";
                }
            }
            else if (addr == 0xAB)
            {
                s += "WITHDRAW ";
                if (data != 0x00)
                {
                    s += "(malformed)";
                }
            }
            else if (addr == 0xB1)
            {
                s += "SEARCHADDRH = " + data.ToString();
            }
            else if (addr == 0xB3)
            {
                s += "SEARCHADDRM = " + data.ToString();
            }
            else if (addr == 0xB5)
            {
                s += "SEARCHADDRL = " + data.ToString();
            }
            else if (addr == 0xB7)
            {
                s += "PROGRAM SHORT ADDRESS = " + (data >> 1).ToString();
            }
            else if (addr == 0xB9)
            {
                s += "VERIFY SHORT ADDRESS = " + (data >> 1).ToString();
            }
            else if (addr == 0xBB)
            {
                s += "QUERY SHORT ADDRESS ";
                if (data != 0x00)
                {
                    s += "(malformed)";
                }
            }
            else if (addr == 0xBD)
            {
                s += "PHYSICAL SELECTION ";
                if (data != 0x00)
                {
                    s += "(malformed)";
                }
            }
            else if (addr == 0xC1)
            {
                s += "ENABLE DEVICE TYPE X = " + data.ToString();
            }
            else if (addr == 0xC3)
            {
                s += "DATA TRANSFER REGISTER 1 (DTR1) = " + data.ToString();
            }
            else if (addr == 0xC5)
            {
                s += "DATA TRANSFER REGISTER 2 (DTR2) = " + data.ToString();
            }
            else if (addr == 0xC7)
            {
                s += "WRITE MEMORY LOCATION = " + data.ToString();
            }
            else if (addr >= 0xA0 && addr <= 0xFD)
            {
                s += "UNKNOWN COMMAND";
            }
            else
            {
                // address
                if ((addr & 0xFE) == 0xFE)
                {
                    s += "BROADCAST ";
                }
                else if ((addr & 0xE0) == 0x80) // 100x xxxx
                {
                    s += "GROUP " + ((addr & 0x1E) >> 1).ToString() + " ";
                }
                else if ((addr & 0x80) == 0x00)
                {
                    s += "SHORT ADDRESS " + ((addr & 0x7E) >> 1).ToString() + " ";
                }
                else
                {
                    s += "UNKNOWN ADDRESS 0x" + addr.ToString("X2") + " ";
                }

                // command / direct arc power control
                if ((addr & 0x01) == 0x01)
                {
                    s += "COMMAND ";
                    if (data == 0x00)
                    {
                        s += "OFF";
                    }
                    else if (data == 0x01)
                    {
                        s += "UP";
                    }
                    else if (data == 0x02)
                    {
                        s += "DOWN";
                    }
                    else if (data == 0x03)
                    {
                        s += "STEP UP";
                    }
                    else if (data == 0x04)
                    {
                        s += "STEP DOWN";
                    }
                    else if (data == 0x05)
                    {
                        s += "RECALL MAX LEVEL";
                    }
                    else if (data == 0x06)
                    {
                        s += "RECALL MIN LEVEL";
                    }
                    else if (data == 0x07)
                    {
                        s += "STEP DOWN AND OFF";
                    }
                    else if (data == 0x08)
                    {
                        s += "ON AND STEP UP";
                    }
                    else if (data == 0x09)
                    {
                        s += "ENABLE DAPC SEQUENCE";
                    }
                    else if ((data & 0xF0) == 0x10)
                    {
                        s += "GO TO SCENE " + (data & 0x0F).ToString();
                    }
                    else if (data == 0x20)
                    {
                        s += "RESET";
                    }
                    else if (data == 0x21)
                    {
                        s += "STORE ACTUAL LEVEL IN THE DTR";
                    }
                    else if (data == 0x2A)
                    {
                        s += "STORE THE DTR AS MAX LEVEL";
                    }
                    else if (data == 0x2B)
                    {
                        s += "STORE THE DTR AS MIN LEVEL";
                    }
                    else if (data == 0x2C)
                    {
                        s += "STORE THE DTR AS SYSTEM FAILURE LEVEL";
                    }
                    else if (data == 0x2D)
                    {
                        s += "STORE THE DTR AS POWER ON LEVEL";
                    }
                    else if (data == 0x2E)
                    {
                        s += "STORE THE DTR AS FADE TIME";
                    }
                    else if (data == 0x2F)
                    {
                        s += "STORE THE DTR AS FADE RATE";
                    }
                    else if ((data & 0xF0) == 0x40)
                    {
                        s += "STORE THE DTR AS SCENE " + (data & 0x0F).ToString();
                    }
                    else if ((data & 0xF0) == 0x50)
                    {
                        s += "REMOVE FROM SCENE " + (data & 0x0F).ToString();
                    }
                    else if ((data & 0xF0) == 0x60)
                    {
                        s += "ADD TO GROUP " + (data & 0x0F).ToString();
                    }
                    else if ((data & 0xF0) == 0x70)
                    {
                        s += "REMOVE FROM GROUP " + (data & 0x0F).ToString();
                    }
                    else if (data == 0x80)
                    {
                        s += "STORE DTR AS SHORT ADDRESS";
                    }
                    else if (data == 0x81)
                    {
                        s += "ENABLE WRITE MEMORY";
                    }
                    else if (data == 0x90)
                    {
                        s += "QUERY STATUS";
                    }
                    else if (data == 0x91)
                    {
                        s += "QUERY CONTROL GEAR";
                    }
                    else if (data == 0x92)
                    {
                        s += "QUERY LAMP FAILURE";
                    }
                    else if (data == 0x93)
                    {
                        s += "QUERY LAMP POWER ON";
                    }
                    else if (data == 0x94)
                    {
                        s += "QUERY LIMIT ERROR";
                    }
                    else if (data == 0x95)
                    {
                        s += "QUERY RESET STATE";
                    }
                    else if (data == 0x96)
                    {
                        s += "QUERY MISSING SHORT ADDRESS";
                    }
                    else if (data == 0x97)
                    {
                        s += "QUERY VERSION NUMBER";
                    }
                    else if (data == 0x98)
                    {
                        s += "QUERY CONTENT DTR";
                    }
                    else if (data == 0x99)
                    {
                        s += "QUERY DEVICE TYPE";
                    }
                    else if (data == 0x9A)
                    {
                        s += "QUERY PHYSICAL MINIMUM LEVEL";
                    }
                    else if (data == 0x9B)
                    {
                        s += "QUERY POWER FAILURE";
                    }
                    else if (data == 0x9C)
                    {
                        s += "QUERY CONTENT DTR1";
                    }
                    else if (data == 0x9D)
                    {
                        s += "QUERY CONTENT DTR2";
                    }
                    else if (data == 0xA0)
                    {
                        s += "QUERY ACTUAL LEVEL";
                    }
                    else if (data == 0xA1)
                    {
                        s += "QUERY MAX LEVEL";
                    }
                    else if (data == 0xA2)
                    {
                        s += "QUERY MIN LEVEL";
                    }
                    else if (data == 0xA3)
                    {
                        s += "QUERY POWER ON LEVEL";
                    }
                    else if (data == 0xA4)
                    {
                        s += "QUERY SYSTEM FAILURE LEVEL";
                    }
                    else if (data == 0xA5)
                    {
                        s += "QUERY FADE TIME/FADE RATE";
                    }
                    else if ((data & 0xF0) == 0xB0)
                    {
                        s += "QUERY SCENE LEVEL, SCENE " + (data & 0x0F).ToString();
                    }
                    else if (data == 0xC0)
                    {
                        s += "QUERY GROUPS 0-7";
                    }
                    else if (data == 0xC1)
                    {
                        s += "QUERY GROUPS 8-15";
                    }
                    else if (data == 0xC2)
                    {
                        s += "QUERY RANDOM ADDRESS (H)";
                    }
                    else if (data == 0xC3)
                    {
                        s += "QUERY RANDOM ADDRESS (M)";
                    }
                    else if (data == 0xC4)
                    {
                        s += "QUERY RANDOM ADDRESS (L)";
                    }
                    else if (data == 0xC5)
                    {
                        s += "QUERY MEMORY LOCATION";
                    }
                    else if (data == 0xE0)
                    {
                        s += "REFERENCE SYSTEM POWER";
                    }
                    else if (data == 0xE1)
                    {
                        s += "ENABLE CURRENT PROTECTOR";
                    }
                    else if (data == 0xE2)
                    {
                        s += "DISABLE CURRENT PROTECTOR";
                    }
                    else if (data == 0xE3)
                    {
                        s += "SELECT DIMMING CURVE";
                    }
                    else if (data == 0xE4)
                    {
                        s += "STORE DTR AS FAST FADE TIME";
                    }
                    else if (data == 0xED)
                    {
                        s += "QUERY GEAR TYPE";
                    }
                    else if (data == 0xEE)
                    {
                        s += "QUERY DIMMING CURVE";
                    }
                    else if (data == 0xEF)
                    {
                        s += "QUERY POSSIBLE OPERATING MODES";
                    }
                    else if (data == 0xF0)
                    {
                        s += "QUERY FEATURES";
                    }
                    else if (data == 0xF1)
                    {
                        s += "QUERY FAILURE STATUS";
                    }
                    else if (data == 0xF2)
                    {
                        s += "QUERY SHORT CIRCUIT";
                    }
                    else if (data == 0xF3)
                    {
                        s += "QUERY OPEN CIRCUIT";
                    }
                    else if (data == 0xF4)
                    {
                        s += "QUERY LOAD DECREASE";
                    }
                    else if (data == 0xF5)
                    {
                        s += "QUERY LOAD INCREASE";
                    }
                    else if (data == 0xF6)
                    {
                        s += "QUERY CURRENT PROTECTOR ACTIVE";
                    }
                    else if (data == 0xF7)
                    {
                        s += "QUERY THERMAL SHUT DOWN";
                    }
                    else if (data == 0xF8)
                    {
                        s += "QUERY THERMAL OVERLOAD";
                    }
                    else if (data == 0xF9)
                    {
                        s += "QUERY REFERENCE RUNNING";
                    }
                    else if (data == 0xFA)
                    {
                        s += "QUERY REFERENCE MEASUREMENT FAILED";
                    }
                    else if (data == 0xFB)
                    {
                        s += "QUERY CURRENT PROTECTOR ENABLED";
                    }
                    else if (data == 0xFC)
                    {
                        s += "QUERY OPERATING MODE";
                    }
                    else if (data == 0xFD)
                    {
                        s += "QUERY FAST FADE TIME";
                    }
                    else if (data == 0xFE)
                    {
                        s += "QUERY MIN FAST FADE TIME";
                    }
                    else if (data == 0xFF)
                    {
                        s += "QUERY EXTENDED VERSION NUMBER";
                    }
                    else
                    {
                        s += "UNKNOWN COMMAND";
                    }

                }
                else
                {
                    s += "DAPC ";
                    if (data == 0x00)
                    {
                        s += "0 (OFF)";
                    }
                    else if (data == 0xFF)
                    {
                        s += "255 (MASK)";
                    }
                    else
                    {
                        s += data.ToString();
                    }
                }
            }

            return s;
        }

        // Transmit a buffer of data to the USB to DALI Interface via USB. It needs the parameter
        // to be a Byte[] buffer of length 65.
        public void sendData(Byte[] outgoing)
        {
            if (initialised == 1)
            {
                if (outgoing[1] == (Byte)USB_PACKETS.TYPE_DALI_COMMAND)
                {
                    Info("TX: [" + outgoing[2].ToString("X2") + outgoing[3].ToString("X2") + "]: " + decode(outgoing[2], outgoing[3]), Color.Blue);
                }
                if (outgoing[1] == (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE)
                {
                    Info("TX: Command sequence", Color.Blue);
                    Info("{", Color.Blue);
                    for (int i = 0; i < outgoing[2]; i++)
                    {
                        Info("\t[" + outgoing[4 + 3 * i].ToString("X2") + outgoing[5 + 3 * i].ToString("X2") + "]: " + decode(outgoing[4 + 3 * i], outgoing[5 + 3 * i]), Color.Blue);
                    }
                    Info("}", Color.Blue);
                }
                unsafe
                {
                    fixed (byte* outBuf = outgoing)
                    {
                        if (DaliWrite(outBuf) != 65)
                        {
                            Error("USB Communication error (write)");
                            initialised = 0;
                        }
                    }
                }
            }
        }

        // Receives data from the USB to DALI Interface via USB. Its parameters are:
        //      incoming: Byte[] buffer of length 65, which will contain the received data
        //      timeout: number of milliseconds to wait for a packet, before timing out
        //      noDataError: flag which tells the function whether the case where no data is
        //                   received should be considered an error. This defaults to 1 (true)
        // The function returns -1 if the board is not initialised by the application and if
        // it is, it returns whatever DaliRead() returned.
        public int getData(Byte[] incoming, int timeout, int noDataError = 1)
        {
            if (initialised == 1)
            {
                int i, r;
                for (i = 0; i < 65; i++)
                    incoming[i] = 0;
                r = 0;
                unsafe
                {
                    fixed (byte* inBuf = incoming)
                    {
                        r = DaliRead(&inBuf[1], timeout);
                    }
                }

                if (r < 0)
                {
                    Error("USB Communication error (read)");
                    initialised = 0;
                }
                else if ((r == 0) && (noDataError == 1))
                {
                    Error("USB Communication error (no data received)");
                    initialised = 0;
                }
                else
                {
                    if (incoming[1] == (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        if (incoming[2] != 0x00)
                        {
                            Info("RX: [" + incoming[3].ToString("X2") + "]: " + incoming[3].ToString(), Color.Yellow);
                        }
                    }
                    else if (incoming[1] == (Byte)USB_REPLIES.REPLY_STATE_QUERY)
                    {
                        Info("Dali State: " + incoming[2].ToString());
                    }
                    else if (incoming[1] == (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        int show;
                        show = 0;
                        for (i = 0; i < incoming[2]; i++)
                        {
                            if (incoming[3 + 3 * i] != 0x00)
                            {
                                show = 1;
                                break;
                            }
                        }

                        if (show != 0)
                        {
                            Info("RX: Command sequence answers", Color.Yellow);
                            Info("{", Color.Yellow);
                            for (i = 0; i < incoming[2]; i++)
                            {
                                Info("\tRX: [" + incoming[3 + 3 * i].ToString("X2") + incoming[5 + 3 * i].ToString("X2") + "]: " + incoming[3 + 3 * i].ToString() + ", " + incoming[5 + 3 * i].ToString(), Color.Yellow);
                            }
                            Info("}", Color.Yellow);
                        }
                    }
                }
                return r;
            }
            else
            {
                return -1;
            }
        }

        // The DALI State machine internal flags can be queried. This function returns a string
        // containing the textual representation of these flags, for debugging purposes.
        public string decodeFlags(UInt32 flags)
        {
            string str = "";
            if ((flags & 0x00001000) != 0) str += "txThisDevice ";
            if ((flags & 0x00000800) != 0) str += "rxFromWaitForBackframe ";
            if ((flags & 0x00000400) != 0) str += "rxFromIdle ";
            if ((flags & 0x00000200) != 0) str += "cableConnected ";
            if ((flags & 0x00000100) != 0) str += "rxForward ";
            if ((flags & 0x00000080) != 0) str += "rxBackward ";
            if ((flags & 0x00000040) != 0) str += "rxTimingError ";
            if ((flags & 0x00000020) != 0) str += "configRxTimingList ";
            if ((flags & 0x00000010) != 0) str += "rxTimingListDone ";
            if ((flags & 0x00000008) != 0) str += "rxDone ";
            if ((flags & 0x00000004) != 0) str += "rxError ";
            if ((flags & 0x00000002) != 0) str += "txDone ";
            if ((flags & 0x00000001) != 0) str += "txError ";

            return str;
        }

        // When receiving data from the USB to DALI Interface about a bus transaction, a set of
        // flags is also available which indicates which of the fields in the transaction are
        // valid. This function returns a textual representation of these fields, for debugging
        // purposes.
        public string decodePacketFlags(UInt16 flags)
        {
            string str = "";
            if ((flags & 0x00000001) != 0) str += "forwardFrameValid ";
            if ((flags & 0x00000002) != 0) str += "backwardFrameValid ";
            if ((flags & 0x00000004) != 0) str += "backwardFrameDelayValid ";
            if ((flags & 0x00000008) != 0) str += "txThisDevice ";
            if ((flags & 0x00000010) != 0) str += "txError ";
            if ((flags & 0x00000020) != 0) str += "rxTimingError ";
            if ((flags & 0x00000040) != 0) str += "rxError ";
            return str;
        }

        // Wait a specific delay, using timing from the microcontroller
        public void listSendDelay(UInt16 delay)
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            byte[] result = new byte[2];
            result[0] = (byte)(delay / 256);
            result[1] = (byte)(delay % 256);

            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_PING_DELAY;
            outgoing[2] = result[0];
            outgoing[3] = result[1];

            sendData(outgoing);
            getData(incoming, 2 * delay + 100);

        }

        // send a command twice
        public void listSendCommTwice(string comm)
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            byte[] resultComm = new byte[2];

            // convert the commands from string to hex
            resultComm[0] = (byte)Convert.ToInt32(comm.Substring(0, 2), 16);
            resultComm[1] = (byte)Convert.ToInt32(comm.Substring(2, 2), 16);

            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = resultComm[0]; // Command 0 addr
            outgoing[5] = resultComm[1]; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = resultComm[0]; // Command 1 addr
            outgoing[8] = resultComm[1]; // Command 1 data

            sendData(outgoing);
            getData(incoming, 300);
        }

        public void listSendComm(string comm)
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            byte[] resultComm = new byte[2];

            resultComm[0] = (byte)Convert.ToInt32(comm.Substring(0, 2), 16);
            resultComm[1] = (byte)Convert.ToInt32(comm.Substring(2, 2), 16);

            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = resultComm[0]; //addr
            outgoing[3] = resultComm[1]; // data

            sendData(outgoing);
            getData(incoming, 200);
        }
    }
}
