﻿/*****************************************************************************
 * Copyright (C) 2014 Microchip Technology Inc. and its subsidiaries
 * (Microchip).  All rights reserved.
 * 
 * You are permitted to use the software and its derivatives with Microchip
 * products. See the license agreement accompanying this software, if any,
 * for additional info regarding your rights and obligations.
 * 
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED “AS IS” WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A
 * PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE
 * LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY,
 * CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY FOR ANY
 * DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY
 * INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, OR OTHER SIMILAR
 * COSTS. To the fullest extend allowed by law, Microchip and its licensors
 * liability will not exceed the amount of fees that you paid directly to
 * Microchip to use this software or $1,000, whichever is greater.
 * 
 *****************************************************************************
 * Revision history:
 * 22.10.2013   Mihai Cuciuc    0.1     Communication with USB device functional,
 *                                      all DALI tests implemented.
 * 22.01.2014   Oana Barahtian  1.0     Changed the application to a multi-
 *                                      threaded model, restructured user
 *                                      interface and added functionality.
 */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Diagnostics;               // Accurate timing measurements
using DALITestApplication.Classes;

namespace DALITestApplication
{
    public partial class MainForm : Form
    {
        public static TestsForm testForm = new TestsForm();
        DaliTest daliTest = new DaliTest();
        Commands commdList = new Commands();

        // counters
        int finishedTests = 0;
        int maxNoTests;
        int totalErrors = 0;
  
        // flags
        bool stopExtraTest = false;  
        bool run = true;
        bool validateAdvanced = false;
        bool validateBasicParam = false;
        bool validateBasicAddr = false;
        bool flagInfo = true;
        bool flagMessages = true;

        // Get input from test tree 
        static bool GetStatus(string s) { return testForm.treeDict[s]; }

        private Thread DALIThread = null;
        private Thread UserCommandsThread = null;

        delegate void WriteProgressCallback(int i);
        delegate void WriteErrorsCallback(int i);
        delegate void WriteCommValueCallback(string str);
        delegate void WriteCommListCallback(string str);
        delegate void WriteCommValueBasicValueCallBack(string str);

        public MainForm()
        {
            InitializeComponent();

            daliTest.preaheatTimeControl = this.preaheatTimeControl;
            daliTest.responseTimeControl = this.responseTimeControl;

            // Messages.cs
            daliTest.debugMenu = this.showDEBUGToolStripMenuItem;
            daliTest.infoMenu = this.showINFOToolStripMenuItem;
            daliTest.messageMenu = this.showMESSAGESToolStripMenuItem;
            daliTest.errorMenu = this.ackERRORsToolStripMenuItem;

            daliTest.log = this.richTextBox1;
            daliTest.form = this;

            commdList.listCommGrid = commDataGrid;

            try
            {
                commdList.LoadDefaultCommands();
            }
            catch (Exception e)
            {
                return;
            }
            try
            {
                commdList.LoadDaliCommands();
            }
            catch(Exception e)
            {
                return;
            }

            SetToolTips();
            SetDropDownListItems();
            InitObjectsforBasic();
        }

        private void SetDropDownListItems()
        {
            if (commdList.listDaliCommand.Count != 0)
            {
                for (int i = 0; i < commdList.listDaliCommand.Count; i++)
                {
                    commdsNameDropDown.Items.Insert(i, commdList.listDaliCommand[i].GetName());
                }
                // disable editing
                commdsNameDropDown.DropDownStyle = ComboBoxStyle.DropDownList;
                // set a default item
                commdsNameDropDown.SelectedIndex = 0;
                UpdateObjectsforCurrentCommd();
            }
        }

        private void SetToolTips()
        {
            ToolTip toolTipbit = new ToolTip();

            toolTipbit.SetToolTip(this.bit15, "Bit 15\nY: short or group address/broadcast\nY = '0': short address\nY = '1': group address or broadcast");
            toolTipbit.SetToolTip(this.bit14, "Bit 14\nA: significant address bit");
            toolTipbit.SetToolTip(this.bit13, "Bit 13\nA: significant address bit");
            toolTipbit.SetToolTip(this.bit12, "Bit 12\nA: significant address bit");
            toolTipbit.SetToolTip(this.bit11, "Bit 11\nA: significant address bit");
            toolTipbit.SetToolTip(this.bit10, "Bit 10\nA: significant address bit");
            toolTipbit.SetToolTip(this.bit9, "Bit 9\nA: significant address bit");
            toolTipbit.SetToolTip(this.bit8, "Bit 8\nS: selector bit\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit7, "Bit 7\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit6, "Bit 6\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit5, "Bit 5\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit4, "Bit 4\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit3, "Bit 3\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit2, "Bit 2\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit1, "Bit 1\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
            toolTipbit.SetToolTip(this.bit0, "Bit 0\nX: In case of a short address, group address or broadcast shall indicate\n whether the data byte contains a direct arc powerlevel or a command:\nS = '0': data byte = direct arc power level\nS = '1': data byte = command");
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            bit15.Checked = true;
            bit14.Checked = true;
            bit13.Checked = true;
            bit12.Checked = true;
            bit11.Checked = true;
            bit10.Checked = true;
            bit9.Checked = true;
            bit8.Checked = true;

            maxProgress.Text = "/ " + "0";
            finishedTest.Text = "0";

            InitUSB();

            if (daliTest.initialised == 0)
            {
                daliTest.Message("Could not initialise USB to DALI Interface", Color.Red);
            } 
        }
         
        // Close the form
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Utils.DaliClose();
        }

         // Displays how many tests are finished 
        private void WriteProgress(int i)
        {
            if (this.finishedTest.InvokeRequired)
            {
                WriteProgressCallback d = new WriteProgressCallback(WriteProgress);
                this.Invoke(d, new object[] { i });
            }
            else
            {
                finishedTest.Text = i.ToString();
            }
        }

        //Displays the number of errors
        private void WriteErrors(int i)
        {
            if (this.errorsLabel.InvokeRequired)
            {
                WriteErrorsCallback d = new WriteErrorsCallback(WriteErrors);
                this.Invoke(d, new object[] { i });
            }
            else 
            {
                errorsLabel.Text = i.ToString();
            }
        }

        private void InitUSB()
        {
            if (daliTest.initialised == 0)
            {
                StringBuilder myString = new StringBuilder(255);
                Byte[] incoming = new Byte[65];
                Byte[] outgoing = new Byte[65];
                int r;

                if (Utils.DaliOpen() != 0)
                {
                    return;
                }

                daliTest.Message("Successfully opened USB to DALI Interface");

                if (Utils.DaliGetManufacturerString(myString) != 0)
                {
                    daliTest.Error("Failed to read manufacturer string");
                    return;
                }

                daliTest.Message("USB Manufacturer String: " + myString.Remove(myString.Length - 1, 1));

                if (Utils.DaliGetProductString(myString) != 0)
                {
                    daliTest.Error("Failed to read product string");
                    return;
                }

                daliTest.Message("USB Product String: " + myString.Remove(myString.Length - 1, 1));
                daliTest.Message("Flushing input buffer..");

                while (daliTest.getData(incoming, 100, 0) > 0);

                daliTest.Message("Done");
                daliTest.initialised = 1;

                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_GET_FW_VERSION;
                daliTest.sendData(outgoing);
                r = daliTest.getData(incoming, 200);

                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_GET_FW_VERSION)
                    {
                        daliTest.Error("Wrong USB packet type received trying to get firmware version");
                    }
                    else
                    {
                        daliTest.Message("USB to DALI Interface Firmware version: " + incoming[2].ToString() + "." + incoming[3].ToString());
                    }
                }
                else
                {
                    daliTest.Error("Failed to read firmware version");
                    return;
                }
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (daliTest.initialised == 1)
            {
                Byte[] outgoing = new Byte[65];
                Byte[] incoming = new Byte[65];
                int r;
  
                r = daliTest.getData(incoming, 100, 0);
                if (r > 0)
                {
                    if (incoming[1] == (Byte)USB_REPLIES.UNSOL_RECEIVE_DATA)
                    {
                        Byte flagsData, backwardData;
                        UInt16 forwardData;
                        Boolean forwardFrameValid, backwardFrameValid, txThisDevice, txError, rxError;

                        forwardData = (UInt16)((incoming[2] << 8) | incoming[3]);
                        backwardData = incoming[4];
                        flagsData = incoming[5];

                        forwardFrameValid = (flagsData & 0x01) == 0x01;
                        backwardFrameValid = (flagsData & 0x02) == 0x02;
                        txThisDevice = (flagsData & 0x08) == 0x08;
                        txError = (flagsData & 0x10) == 0x10;
                        rxError = (flagsData & 0x20) == 0x20;
                        rxError = (flagsData & 0x40) == 0x40;

                        if (txError || rxError || rxError)
                        {
                            daliTest.Error("Received packet with errors. Flags = " + daliTest.decodePacketFlags(flagsData));
                        }
                        else if (txThisDevice)
                        {
                            daliTest.Error("Caught message transmitted from this device. Flags = " + daliTest.decodePacketFlags(flagsData));
                        }
                        else if (forwardFrameValid && (!backwardFrameValid))
                        {
                            daliTest.Message("Other TX: [" + forwardData.ToString("X4") + "]: " + daliTest.decode(incoming[2], incoming[3]));
                        }
                        else if (forwardFrameValid && backwardFrameValid)
                        {
                            daliTest.Message("Other TX: [" + forwardData.ToString("X4") + "]: " + daliTest.decode(incoming[2], incoming[3]));
                            daliTest.Message("      RX: [" + backwardData.ToString("X2") + "]: " + backwardData);
                        }
                    }
                    else
                    {
                        daliTest.Message("Got packet type: " + incoming[1].ToString());
                    }
                }
            }
            InitUSB();
        }
            
        // While running tests some objects will be disabled
        private void userTestConfigDisable()
        {
            buttonTests.Enabled = false;
            testNo.Enabled = false;
            preaheatTimeControl.Enabled = false;
            responseTimeControl.Enabled = false;

            extraTestToolStripMenuItem.Enabled= false;

            queryLevel.Enabled = false;
            powerController.Enabled = false;

            sendUserCommand.Enabled = false;
            sendUserCmmdBasic.Enabled = false;
            sendSelectedComm.Enabled = false;
            sendListButton.Enabled = false;
        }
        
        // Reenable what has been disabled
        private void userTestConfigEnable()
        {
            buttonTests.Enabled = true;
            testNo.Enabled = true;
            preaheatTimeControl.Enabled = true;
            responseTimeControl.Enabled = true;

            extraTestToolStripMenuItem.Enabled = true;

            queryLevel.Enabled = true;
            powerController.Enabled = true;

            sendUserCommand.Enabled = true;
            sendUserCmmdBasic.Enabled = true;
            sendSelectedComm.Enabled = true;
            sendListButton.Enabled = true;
        }

        private void userCommandsConfigEnable()
        {
            buttonTests.Enabled = true;
            testNo.Enabled = true;
            preaheatTimeControl.Enabled = true;
            responseTimeControl.Enabled = true;

            extraTestToolStripMenuItem.Enabled = true;

            queryLevel.Enabled = true;
            powerController.Enabled = true;

            sendUserCommand.Enabled = true;
            sendUserCmmdBasic.Enabled = true;
            sendSelectedComm.Enabled = true;
            sendListButton.Enabled = true;
            moveDownLineDataGrid.Enabled = true;
            moveUpLineDataGrid.Enabled = true;
            deleteRowDataGrid.Enabled = true;
            cleanDataGrid.Enabled = true;
            commDataGrid.ReadOnly = false;

            buttonRunTests.Enabled = true;
        }

        private void userCommandsConfigDisable()
        {
            buttonTests.Enabled = false;
            testNo.Enabled = false;
            preaheatTimeControl.Enabled = false;
            responseTimeControl.Enabled = false;

            extraTestToolStripMenuItem.Enabled = false;

            queryLevel.Enabled = false;
            powerController.Enabled = false;

            sendUserCommand.Enabled = false;
            sendUserCmmdBasic.Enabled = false;
            sendSelectedComm.Enabled = false;
            sendListButton.Enabled = false;
            moveDownLineDataGrid.Enabled = false;
            moveUpLineDataGrid.Enabled = false;
            deleteRowDataGrid.Enabled = false;
            cleanDataGrid.Enabled = false;
            commDataGrid.ReadOnly = true;

            buttonRunTests.Enabled = false;
        }

        private void buttonRunTests_Click(object sender, EventArgs e)
        {
            maxNoTests = noTest() * (int)testNo.Value;
            if (run == true)
            {
                if (daliTest.initialised == 0)
                {
                    daliTest.Message("============================================", Color.Red);
                    daliTest.Message("==== DALI To USB Interface not detected ====", Color.Red);
                    daliTest.Message("============================================", Color.Red);

                    return;
                }

                if (maxNoTests != 0)
                {
                    timer1.Enabled = false;
                    // Creates a new thread
                    this.DALIThread = new Thread(() => RUN_ALL((int)testNo.Value));
                    this.DALIThread.Start();
                    DALIThread.IsBackground = true;

                    userTestConfigDisable();
                    buttonRunTests.Text = "Stop";
                    run = false;
                }
                else
                {
                    daliTest.Message("============ No test to run =============", Color.Red);
                }
            }
            else
            {
                int i = 0;
                FormCollection fc = Application.OpenForms;

                if (DALIThread != null)
                {
                    DALIThread.Abort();
                }

                daliTest.Message("============== Aborted by USER ==================", Color.Red);

                if (fc.Count > 1)
                {
                    foreach (Form frm in fc)
                    {
                        i++;
                        // Close all windows, but not the main one
                        if (i > 1)
                        {
                            Invoke(new Action(() => frm.Dispose()));
                        }
                    }
                }

                totalErrors = 0;
                daliTest.numErrors = 0;

                finishedTests = 0;
                userTestConfigEnable();
                buttonRunTests.Text = "Run";
                timer1.Enabled = true;
                run = true;
                Thread.Sleep(100);
            }
        }

        // Add summary report to the log after running a test
        private void WriteConclusions(int i, string testName)
        {
            daliTest.Message("Run#\tErrors\tTestID");
            daliTest.Summary(i.ToString("D6") + "\t" + daliTest.numErrors.ToString("D6") + "\t" + testName);
            daliTest.Message("--------------------------------------------");
            finishedTests++;
            WriteProgress(finishedTests);
            totalErrors += daliTest.numErrors;
            WriteErrors(totalErrors);
        }

        // Verifies which of the tests are selected and runs them
        private void RUN_ALL(int maxNoTests)
        {
            Func<string, bool> method = GetStatus;
            IAsyncResult cookie;
            bool result;
            int i;
            long t0;

            t0 = DateTime.Now.Ticks;

           
            // Check bus connected state
            daliTest.CheckBus();

            // [IMPORTANT] if you change the tests name in treeview [test.design], please change it here as well

            for (i = 0; i < testNo.Value; i++)
            {
                daliTest.Message("============================================");
                daliTest.Message("========= Run # " + i.ToString("D6") + " =====================");
                daliTest.Message("============================================");

                // run 12.1.1.1
                cookie = method.BeginInvoke("12.1.1.1: Current rating", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_1_1();
                    WriteConclusions(i, "12.1.1.1: Current rating");
                }

                // run 12.1.1.2
                cookie = method.BeginInvoke("12.1.1.2: Voltage rating", null, null);
                result = method.EndInvoke(cookie);

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_1_2();
                    WriteConclusions(i, "12.1.1.2: Voltage rating");
                }

                // run 12.1.1.3
                cookie = method.BeginInvoke("12.1.1.3: Back channel rise time / fall time", null, null);
                result = method.EndInvoke(cookie);

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_1_3();
                    WriteConclusions(i, "12.1.1.3: Back channel rise time / fall time");
                }

                // run 12.1.1.4
                cookie = method.BeginInvoke("12.1.1.4: Transmission rate", null, null);
                result = method.EndInvoke(cookie);

               // if (testForm.treeDict["12.1.1.4"] == true)
                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_1_4();
                    WriteConclusions(i, "12.1.1.4: Transmission rate");
                }

                // run 12.1.1.5
                cookie = method.BeginInvoke("12.1.1.5: Pulse width", null, null);
                result = method.EndInvoke(cookie);

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_1_5();
                    WriteConclusions(i, "12.1.1.5: Pulse width");
                }

                // run 12.1.1.6
                cookie = method.BeginInvoke("12.1.1.6: Code violation", null, null);
                result = method.EndInvoke(cookie);

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_1_6();
                    WriteConclusions(i, "12.1.1.6: Code violation");
                }

                // run 12.1.2.1
                cookie = method.BeginInvoke("12.1.2.1: Frame structure timing", null, null);
                result = method.EndInvoke(cookie); 

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_2();
                    WriteConclusions(i, "12.1.2.1: Frame structure timing");
                }

                // run 12.1.3.1
                cookie = method.BeginInvoke("12.1.3.1: Insulation test", null, null);
                result = method.EndInvoke(cookie); 

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_3();
                    WriteConclusions(i, "12.1.3.1: Insulation test");
                }

                // run 12.1.4.1
                cookie = method.BeginInvoke("12.1.4.1: Polarity", null, null);
                result = method.EndInvoke(cookie);

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_4_1();
                    WriteConclusions(i, "12.1.4.1: Polarity");
                }
                
                // run 12.1.4.2
                cookie = method.BeginInvoke("12.1.4.2: Overvoltage protection", null, null);
                result = method.EndInvoke(cookie);

               //if (testForm.treeDict["12.1.4.2"] == true)
                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_4_2();
                    WriteConclusions(i, "12.1.4.2: Overvoltage protection");
                }

                // run 12.1.5.1
                cookie = method.BeginInvoke("12.1.5.1: Response time", null, null);
                result = method.EndInvoke(cookie);

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_1_5();
                    WriteConclusions(i, "12.1.5.1: Response time");
                }
                
                // run 12.2.1.1
                cookie = method.BeginInvoke("12.2.1.1: RESET", null, null);
                result = method.EndInvoke(cookie);

                //if (testForm.treeDict["12.2.1.1"] == true)
                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_1();
                    WriteConclusions(i, "12.2.1.1: RESET");
                }

                // run 12.2.1.2
                cookie = method.BeginInvoke("12.2.1.2: RESET: timeout / command in-between", null, null);
                result = method.EndInvoke(cookie);

                if(result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_2();
                    WriteConclusions(i, "12.2.1.2: RESET: timeout / command in-between");
                }

                // run 12.2.1.3
                cookie = method.BeginInvoke("12.2.1.3: 100 ms - timeout", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_3();
                    WriteConclusions(i, "12.2.1.3: 100 ms - timeout");
                    totalErrors += daliTest.numErrors;
                }

                // run 12.2.1.4
                cookie = method.BeginInvoke("12.2.1.4: Commands in-between", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_4();
                    WriteConclusions(i, "12.2.1.4: Commands in-between");
                }

                // run 12.2.1.5
                cookie = method.BeginInvoke("12.2.1.5: VERSION NUMBER", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_5();
                    WriteConclusions(i, "12.2.1.5: VERSION NUMBER");
                }

                // run 12.2.1.6
                cookie = method.BeginInvoke("12.2.1.6: STORE ACTUAL LEVEL IN THE DTR", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_6();
                    WriteConclusions(i, "12.2.1.6: STORE ACTUAL LEVEL IN THE DTR");
                }

                // run 12.2.1.7
                cookie = method.BeginInvoke("12.2.1.7: PERSISTENT MEMORY", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_7();
                    WriteConclusions(i, "12.2.1.7: PERSISTENT MEMORY");
                }

                // run 12.2.1.8
                cookie = method.BeginInvoke("12.2.1.8: DTR1", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_8();
                    WriteConclusions(i, "12.2.1.8: DTR1");
                }

                // run 12.2.1.9
                cookie = method.BeginInvoke("12.2.1.9: DTR2", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_1_9();
                    WriteConclusions(i, "12.2.1.9: DTR2");
                }

                // run 12.2.2.1
                cookie = method.BeginInvoke("12.2.2.1: STORE THE DTR AS MAX LEVEL", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_2_1();
                    WriteConclusions(i, "12.2.2.1: STORE THE DTR AS MAX LEVEL");
                }

                // run 12.2.2.2
                cookie = method.BeginInvoke("12.2.2.2: STORE THE DTR AS MIN LEVEL", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_2_2();
                    WriteConclusions(i, "12.2.2.2: STORE THE DTR AS MIN LEVEL");
                }

                // run 12.2.2.3
                cookie = method.BeginInvoke("12.2.2.3: STORE THE DTR AS SYSTEM FAILURE LEVEL", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_2_3();
                    WriteConclusions(i, "12.2.2.3: STORE THE DTR AS SYSTEM FAILURE LEVEL");
                }

                // run 12.2.2.4
                cookie = method.BeginInvoke("12.2.2.4: STORE THE DTR AS POWER ON LEVEL", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_2_4();
                    WriteConclusions(i, "12.2.2.4: STORE THE DTR AS POWER ON LEVEL4");
                }

                // run 12.2.2.5
                cookie = method.BeginInvoke("12.2.2.5: STORE THE DTR AS FADE TIME", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_2_5();
                    WriteConclusions(i, "12.2.2.5: STORE THE DTR AS FADE TIME");
                }

                // run 12.2.2.6
                cookie = method.BeginInvoke("12.2.2.6: STORE THE DTR AS FADE RATE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_2_6();
                    WriteConclusions(i, "12.2.2.6: STORE THE DTR AS FADE RATE");
                }

                // run 12.2.2.7
                cookie = method.BeginInvoke("12.2.2.7: STORE THE DTR AS SCENE / GO TO SCENE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_2_7();
                    WriteConclusions(i, "12.2.2.7: STORE THE DTR AS SCENE / GO TO SCENE");
                }

                // run 12.2.3.1
                cookie = method.BeginInvoke("12.2.3.1: REMOVE FROM SCENE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_3_1();
                    WriteConclusions(i, "12.2.3.1: REMOVE FROM SCENE");
                }

                // run 12.2.3.2
                cookie = method.BeginInvoke("12.2.3.2: ADD TO GROUP / REMOVE FROM GROUP", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_3_2();
                    WriteConclusions(i, "12.2.3.2: ADD TO GROUP / REMOVE FROM GROUP");
                }

                // run 12.2.3.3
                cookie = method.BeginInvoke("12.2.3.3: STORE THE DTR AS SHORT ADDRESS", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_3_3();
                    WriteConclusions(i, "12.2.3.3: STORE THE DTR AS SHORT ADDRESS");
                }

                // run 12.2.4.1
                cookie = method.BeginInvoke("12.2.4.1: Memory Bank 0", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_4_1();
                    WriteConclusions(i, "12.2.4.1: Memory Bank 0");
                }

                // run 12.1.4.2
                cookie = method.BeginInvoke("12.2.4.2: Memory Bank 1", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_4_2();
                    WriteConclusions(i, "12.2.4.2: Memory Bank 1");
                }

                // run 12.2.4.3
                cookie = method.BeginInvoke("12.2.4.3: Other Memory Banks", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_4_3();
                    WriteConclusions(i, "12.2.4.3: Other Memory Banks");
                }

                // run 12.2.4.4
                cookie = method.BeginInvoke("12.2.4.4: ENABLE WRITE MEMORY", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_2_4_4();
                    WriteConclusions(i, "12.2.4.4: ENABLE WRITE MEMORY");
                }

                // run 12.3.1.1
                cookie = method.BeginInvoke("12.3.1.1: FADE TIME", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_1_1();
                    WriteConclusions(i, "12.3.1.1: FADE TIME");
                }

                // run 12.3.1.2
                cookie = method.BeginInvoke("12.3.1.2: FADE RATE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_1_2();
                    WriteConclusions(i, "12.3.1.2: FADE RATE");
                }

                // run 12.3.2.1
                cookie = method.BeginInvoke("12.3.2.1: LOGARITHMIC DIMMING CURVE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_2_1();
                    WriteConclusions(i, "12.3.2.1: LOGARITHMIC DIMMING CURVE");
                }

                // run 12.3.2.2
                cookie = method.BeginInvoke("12.3.2.2: CONTINUOUS DIMMING: DAPC", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_2_2();
                    WriteConclusions(i, "12.3.2.2: CONTINUOUS DIMMING: DAPC");
                }

                // run 12.3.2.3
                cookie = method.BeginInvoke("12.3.2.3: CONTINUOUS DIMMING: UP / DOWN", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_2_3();
                    WriteConclusions(i, "12.3.2.3: CONTINUOUS DIMMING: UP / DOWN");
                }

                // run 12.3.2.4
                cookie = method.BeginInvoke("12.3.2.4: CONTINUOUS DIMMING: STEP UP / STEP DOWN", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_2_4();
                    WriteConclusions(i, "12.3.2.4: CONTINUOUS DIMMING: STEP UP / STEP DOWN");
                }

                // run 12.3.2.5
                cookie = method.BeginInvoke("12.3.2.5: Dimming curve: DAPC SEQUENCE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_2_5();
                    WriteConclusions(i, "12.3.2.5: Dimming curve: DAPC SEQUENCE");
                }

                // run 12.3.3.1
                cookie = method.BeginInvoke("12.3.3.1: OFF", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_1();
                    WriteConclusions(i, "12.3.3.1: OFF");
                }

                // run 12.3.3.2
                cookie = method.BeginInvoke("12.3.3.2: DIRECT ARC POWER CONTROL", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_2();
                    WriteConclusions(i, "12.3.3.2: DIRECT ARC POWER CONTROL");
                }

                // run 12.3.3.3
                cookie = method.BeginInvoke("12.3.3.3: UP", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_3();
                    WriteConclusions(i, "12.3.3.3: UP");
                }

                // run 12.3.3.4
                cookie = method.BeginInvoke("12.3.3.4: DOWN", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_4();
                    WriteConclusions(i, "12.3.3.4: DOWN");   
                }

                // run 12.3.3.5
                cookie = method.BeginInvoke("12.3.3.5: STEP UP", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_5();
                    WriteConclusions(i, "12.3.3.5: STEP UP");
                }

                // run 12.3.3.6
                cookie = method.BeginInvoke("12.3.3.6: STEP DOWN", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_6();
                    WriteConclusions(i, "12.3.3.6: STEP DOWN");
                }

                // run 12.3.3.7
                cookie = method.BeginInvoke("12.3.3.7: RECALL MAX LEVEL", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_7();
                    WriteConclusions(i, "12.3.3.7: RECALL MAX LEVEL");
                }

                // run 12.3.3.8
                cookie = method.BeginInvoke("12.3.3.8: RECALL MIN LEVEL", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_8();
                    WriteConclusions(i, "12.3.3.8: RECALL MIN LEVEL");
                }

                // run 12.3.3.9
                cookie = method.BeginInvoke("12.3.3.9: ON AND STEP UP", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_9();
                    WriteConclusions(i, "12.3.3.9: ON AND STEP UP"); ;
                }

                // run 12.3.3.10
                cookie = method.BeginInvoke("12.3.3.10: STEP DOWN AND OFF", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_3_3_10();
                    WriteConclusions(i, "12.3.3.10: STEP DOWN AND OFF");
                }

                // run 12.4.1
                cookie = method.BeginInvoke("12.4.1: Physical address allocation", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_4();
                    WriteConclusions(i, "12.4.1: Physical address allocation");
                }

                // run 12.5.1.1
                cookie = method.BeginInvoke("12.5.1.1: INITIALISE: 15 minutes timer", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_1_1();
                    WriteConclusions(i, "12.5.1.1: INITIALISE: 15 minutes timer");
                }

                // run 12.5.1.2
                cookie = method.BeginInvoke("12.5.1.2: TERMINATE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_1_2();
                    WriteConclusions(i, "12.5.1.2: TERMINATE");
                }

                // run 12.5.1.3
                cookie = method.BeginInvoke("12.5.1.3: INITIALISE: short address", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_1_3();
                    WriteConclusions(i, "12.5.1.3: INITIALISE: short address");
                }

                // run 12.5.1.4
                cookie = method.BeginInvoke("12.5.1.4: INITIALISE: no short address", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_1_4();
                    WriteConclusions(i, "12.5.1.4: INITIALISE: no short address");
                }

                // run 12.5.1.5
                cookie = method.BeginInvoke("12.5.1.5: INITIALISE: 100 ms timeout", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_1_5();
                    WriteConclusions(i, "12.5.1.5: INITIALISE: 100 ms timeout");
                }

                // run 12.5.1.6
                cookie = method.BeginInvoke("12.5.1.6: INITIALISE: command in-between", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_1_6();
                    WriteConclusions(i, "12.5.1.6: INITIALISE: command in-between");
                }

                // run 12.5.2.1
                cookie = method.BeginInvoke("12.5.2.1: RANDOMISE: reset values", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_2_1();
                    WriteConclusions(i, "12.5.2.1: RANDOMISE: reset values");
                }

                // run 12.5.2.2
                cookie = method.BeginInvoke("12.5.2.2: RANDOMISE: 100 ms timeout", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_2_2();
                    WriteConclusions(i, "12.5.2.2: RANDOMISE: 100 ms timeout");
                }

                // run 12.5.2.3
                cookie = method.BeginInvoke("12.5.2.3: RANDOMISE: command in-between", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_2_3();
                    WriteConclusions(i, "12.5.2.3: RANDOMISE: command in-between");
                }

                // run 12.5.3.1
                cookie = method.BeginInvoke("12.5.3.1: COMPARE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_3_1();
                    WriteConclusions(i, "12.5.3.1: COMPARE");
                }

                // run 12.5.3.2
                cookie = method.BeginInvoke("12.5.3.2: WITHDRAW", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_3_2();
                    WriteConclusions(i, "12.5.3.2: WITHDRAW");
                }

                // run 12.5.4.1
                cookie = method.BeginInvoke("12.5.4.1: PROGRAM SHORT ADDRESS", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_4_1();
                    WriteConclusions(i, "12.5.4.1: PROGRAM SHORT ADDRESS");
                }

                // run 12.5.4.2
                cookie = method.BeginInvoke("12.5.4.2: VERIFY SHORT ADDRESS", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_4_2();
                    WriteConclusions(i, "12.5.4.2: VERIFY SHORT ADDRESS");
                }

                // run 12.5.4.3
                cookie = method.BeginInvoke("12.5.4.3: QUERY SHORT ADDRESS", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_4_3();
                    WriteConclusions(i, "12.5.4.3: QUERY SHORT ADDRESS");
                }

                // run 12.5.4.4
                cookie = method.BeginInvoke("12.5.4.4: SEARCH ADDRESS: reset value", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_5_4_4();
                    WriteConclusions(i, "12.5.4.4: SEARCH ADDRESS: reset value");
                }

                // run 12.6.1.1
                cookie = method.BeginInvoke("12.6.1.1: QUERY DEVICE TYPE", null, null);
                result = method.EndInvoke(cookie);
                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_1_1();
                    WriteConclusions(i, "12.6.1.1: QUERY DEVICE TYPE");
                }

                // run 12.6.1.2
                cookie = method.BeginInvoke("12.6.1.2: QUERY LAMP FAILURE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_1_2();
                    WriteConclusions(i, "12.6.1.2: QUERY LAMP FAILURE");
                }

                // run 12.6.1.3
                cookie = method.BeginInvoke("12.6.1.3: QUERY LAMP POWER ON", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_1_3();
                    WriteConclusions(i, "12.6.1.3: QUERY LAMP POWER ON");
                }

                // run 12.6.1.4
                cookie = method.BeginInvoke("12.6.1.4: QUERY LIMIT ERROR", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_1_4();
                    WriteConclusions(i, "12.6.1.4: QUERY LIMIT ERROR");
                }

                // run 12.6.1.5
                cookie = method.BeginInvoke("12.6.1.5: QUERY POWER FAILURE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_1_5();
                    WriteConclusions(i, "12.6.1.5: QUERY POWER FAILURE");
                }

                // run 12.6.1.6
                cookie = method.BeginInvoke("12.6.1.6: QUERY STATUS: control gear ok", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_1_6();
                    WriteConclusions(i, "12.6.1.6: QUERY STATUS: control gear ok");
                }

                // run 12.6.1.7
                cookie = method.BeginInvoke("12.6.1.7: QUERY STATUS: fade running", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_1_7();
                    WriteConclusions(i, "12.6.1.7: QUERY STATUS: fade running");
                }

                // run 12.6.2.1
                cookie = method.BeginInvoke("12.6.2.1: RESERVED: standard commands", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_2_1();
                    WriteConclusions(i, "12.6.2.1: RESERVED: standard commands");
                }

                // run 12.6.2.2
                cookie = method.BeginInvoke("12.6.2.2: Application extended commands", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_2_2();
                    WriteConclusions(i, "12.6.2.2: Application extended commands");
                }

                // run 12.6.2.3
                cookie = method.BeginInvoke("12.6.2.3: RESERVED: special commands 1", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_2_3();
                    WriteConclusions(i, "12.6.2.3: RESERVED: special commands 1");
                    totalErrors += daliTest.numErrors;
                }

                // run 12.6.2.3
                cookie = method.BeginInvoke("12.6.2.4: RESERVED: special commands 2", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_2_4();
                    WriteConclusions(i, "12.6.2.4: RESERVED: special commands 2");
                }

                // run 12.6.2.5
                cookie = method.BeginInvoke("12.6.2.5: Not supported device types", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_6_2_5();
                    WriteConclusions(i, "12.6.2.5: Not supported device types");
                }
 
                // run 12.7.1.1
                cookie = method.BeginInvoke("12.7.1.1: QUERY FEATURES", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_1();
                    WriteConclusions(i, "12.7.1.1: QUERY FEATURES");
                }

                // run 12.7.1.2
                cookie = method.BeginInvoke("12.7.1.2: QUERY SHORT CIRCUIT", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_2();
                    WriteConclusions(i, "12.7.1.2: QUERY SHORT CIRCUIT");
                }

                // run 12.7.1.3
                cookie = method.BeginInvoke("12.7.1.3: QUERY OPEN CIRCUIT", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_3();
                    WriteConclusions(i, "12.7.1.3: QUERY OPEN CIRCUIT");
                }

                // run 12.7.1.4
                cookie = method.BeginInvoke("12.7.1.4: QUERY LOAD DECREASE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_4();
                    WriteConclusions(i, "12.7.1.4: QUERY LOAD DECREASE");
                }

                // run 12.7.1.5
                cookie = method.BeginInvoke("12.7.1.5: QUERY LOAD INCREASE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_5();
                    WriteConclusions(i, "12.7.1.5: QUERY LOAD INCREASE");
                }

                // run 12.7.1.6
                cookie = method.BeginInvoke("12.7.1.6: QUERY CURRENT PROTECTOR ACTIVE: Underload", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_6();
                    WriteConclusions(i, "12.7.1.6: QUERY CURRENT PROTECTOR ACTIVE: Underload");
                }

                // run 12.7.1.7
                cookie = method.BeginInvoke("12.7.1.7: QUERY CURRENT PROTECTOR ACTIVE: Overload", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_7();
                    WriteConclusions(i, "12.7.1.7: QUERY CURRENT PROTECTOR ACTIVE: Overload");
                }

                // run 12.7.1.8
                cookie = method.BeginInvoke("12.7.1.8: QUERY THERMAL SHUT DOWN", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_8();
                    WriteConclusions(i, "12.7.1.8: QUERY THERMAL SHUT DOWN");
                }

                // run 12.7.1.9
                cookie = method.BeginInvoke("12.7.1.9: QUERY THERMAL OVERLOAD", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_9();
                    WriteConclusions(i, "12.7.1.9: QUERY THERMAL OVERLOAD");
                }

                // run 12.7.1.10
                cookie = method.BeginInvoke("12.7.1.10: Query control gear information", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_1_10();
                    WriteConclusions(i, "12.7.1.10: Query control gear information");
                }

                // run 12.7.2.1
                cookie = method.BeginInvoke("12.7.2.1: REFERENCE SYSTEM POWER", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_1();
                    WriteConclusions(i, "12.7.2.1: REFERENCE SYSTEM POWER");
                }

                // run 12.7.2.2
                cookie = method.BeginInvoke("12.7.2.2: REFERENCE SYSTEM POWER: 100 ms-timeout", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_2();
                    WriteConclusions(i, "12.7.2.2: REFERENCE SYSTEM POWER: 100 ms-timeout");
                }

                // run 12.7.2.3
                cookie = method.BeginInvoke("12.7.2.3: REFERENCE SYSTEM POWER: Command in-between", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_3();
                    WriteConclusions(i, "12.7.2.3: REFERENCE SYSTEM POWER: Command in-between");
                }

                // run 12.7.2.4
                cookie = method.BeginInvoke("12.7.2.4: REFERENCE SYSTEM POWER: 15 minutes timer", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_4();
                    WriteConclusions(i, "12.7.2.4: REFERENCE SYSTEM POWER: 15 minutes timer");
                }

                // run 12.7.2.5
                cookie = method.BeginInvoke("12.7.2.5: REFERENCE SYSTEM POWER: failed", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_5();
                    WriteConclusions(i, "12.7.2.5: REFERENCE SYSTEM POWER: failed");
                }

                // run 12.7.2.6
                cookie = method.BeginInvoke("12.7.2.6: ENABLE / DISABLE CURRENT PROTECTOR", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_6();
                    WriteConclusions(i, "12.7.2.6: ENABLE / DISABLE CURRENT PROTECTOR");
                }

                // run 12.7.2.7
                cookie = method.BeginInvoke("12.7.2.7: SELECT DIMMING CURVE", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_7();
                    WriteConclusions(i, "12.7.2.7: SELECT DIMMING CURVE");
                }

                // run 12.7.2.8
                cookie = method.BeginInvoke("12.7.2.8: FAST FADE TIME", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_8();
                    WriteConclusions(i, "12.7.2.8: FAST FADE TIME");
                }

                // run 12.7.2.9
                cookie = method.BeginInvoke("12.7.2.9: Reset State / Persistent Memory", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_2_9();
                    WriteConclusions(i, "12.7.2.9: Reset State / Persistent Memory");
                }

                // run 12.7.3.1
                cookie = method.BeginInvoke("12.7.3.1: ENABLE DEVICE TYPE: Application extended commands", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_3_1();
                    WriteConclusions(i, "12.7.3.1: ENABLE DEVICE TYPE: Application extended commands");
                }

                // run 12.7.3.2
                cookie = method.BeginInvoke("12.7.3.2: ENABLE DEVICE TYPE: Application extended configuration commands 1", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_3_2();
                    WriteConclusions(i, "12.7.3.2: ENABLE DEVICE TYPE: Application extended configuration commands 1");
                }

                // run 12.7.3.3
                cookie = method.BeginInvoke("12.7.3.3: ENABLE DEVICE TYPE: Application extended configuration commands 2", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_3_3();
                    WriteConclusions(i, "12.7.3.3: ENABLE DEVICE TYPE: Application extended configuration commands 2");
                }

                // run 12.7.4.1
                cookie = method.BeginInvoke("12.7.4.1: QUERY EXTENDED VERSION NUMBER", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_4_1();
                    WriteConclusions(i, "12.7.4.1: QUERY EXTENDED VERSION NUMBER");
                }

                // run 12.7.4.2
                cookie = method.BeginInvoke("12.7.4.2: RESERVED APPLICATION EXTENDED COMMANDS", null, null);
                result = method.EndInvoke(cookie);

                if (result == true)
                {
                    daliTest.numErrors = 0;
                    daliTest.Message("--------------------------------------------");
                    daliTest.RunTest_12_7_4_2();
                    WriteConclusions(i, "12.7.4.2: RESERVED APPLICATION EXTENDED COMMANDS");
                }
            }

            daliTest.Message("Time elapsed:     " + ((double)(DateTime.Now.Ticks - t0) / 1e7).ToString() + "s", Color.Green);
            daliTest.Message("Number of errors: " +totalErrors.ToString(), Color.Green);

            daliTest.Message("--------------------------------------------");
            daliTest.Message("------------------- DONE -------------------");
            daliTest.Message("--------------------------------------------");
            
            Invoke(new Action(() => timer1.Enabled = true));
            run = true;
            Invoke(new Action(() => buttonRunTests.Text = "Run"));
            Invoke(new Action(() => userTestConfigEnable()));

            totalErrors = 0;
            daliTest.numErrors = 0;
            finishedTests = 0;
        }

        //  Monifies the intesity of the light
        private void powerController_ValueChanged(object sender, EventArgs e)
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            Byte address, value;

            address = 0xFE;
            value = (Byte)powerController.Value;
            if (numericUpDown.Value > -1) address = (Byte)((Byte)numericUpDown.Value << 1);

            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = address;
            outgoing[3] = value;
            daliTest.sendData(outgoing);
            daliTest.getData(incoming, 100);
        }

        // Exit
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Really Quit?", "Exit", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {

                // Aborts the thread if the form is closed while the thread is running
                if (DALIThread != null)
                {
                    DALIThread.Abort();
                }

                Utils.DaliClose();
                Application.Exit();
            }
        }

        // Clear log
        private void clearStripMenuItem_Click(object sender, EventArgs e)
        {
            richTextBox1.Clear();
            daliTest.textBoxLen = 0;
        }

        // Save the content of the log
        private void saveStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = "log file.rtf";
            saveFileDialog1.Title = "Save text file";

            if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK && saveFileDialog1.FileName.Length > 0)
            {
                richTextBox1.SaveFile(saveFileDialog1.FileName);
            }
        }

        // Close the application
        private void buttonClose_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Really Quit?", "Exit", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                Utils.DaliClose();
                Application.Exit();
            }
        }

        // Choose the tests
        private void buttonTests_Click(object sender, EventArgs e)
        {
            int maxNoTests;

            testForm.ShowDialog();

            maxNoTests = noTest() * (int)testNo.Value;
            maxProgress.Text = "/ " + maxNoTests.ToString();
        }

        private void WriteCommValue(string str)
        {
            if (this.commValueLabel.InvokeRequired)
            {
                WriteCommValueCallback d = new WriteCommValueCallback(WriteCommValue);
                this.Invoke(d, new object[] { str });
            }
            else
            {
                commValueLabel.Text = str;
            }
        }

        private void WriteCommValueBasic(string str)
        {
            if (this.commValueLabel.InvokeRequired)
            {
                WriteCommValueBasicValueCallBack d = new WriteCommValueBasicValueCallBack(WriteCommValueBasic);
                this.Invoke(d, new object[] { str });
            }
            else
            {
                labelHexInterpretation.Text = str;
            }
        }

        private void changeHexComm()
        {
            Byte addr = 0, data = 0;
            if (bit15.Checked) addr |= 0x80;
            if (bit14.Checked) addr |= 0x40;
            if (bit13.Checked) addr |= 0x20;
            if (bit12.Checked) addr |= 0x10;
            if (bit11.Checked) addr |= 0x08;
            if (bit10.Checked) addr |= 0x04;
            if (bit9.Checked) addr |= 0x02;
            if (bit8.Checked) addr |= 0x01;
            if (bit7.Checked) data |= 0x80;
            if (bit6.Checked) data |= 0x40;
            if (bit5.Checked) data |= 0x20;
            if (bit4.Checked) data |= 0x10;
            if (bit3.Checked) data |= 0x08;
            if (bit2.Checked) data |= 0x04;
            if (bit1.Checked) data |= 0x02;
            if (bit0.Checked) data |= 0x01;
            WriteCommValue(daliTest.decode(addr, data));
            string aux = addr.ToString("X2") + data.ToString("X2");
            commndTextBox.Text = aux;
            commndErrorLabel.Visible = false;
        }

        // This method is called whenever the user types a command in the text field
        private void UpdateCommand(string str)
        {
            byte[] result = new byte[2];

            // convert the commands from string to hex
            result[0] = (byte)Convert.ToInt32(str.Substring(0, 2), 16);
            result[1] = (byte)Convert.ToInt32(str.Substring(2, 2), 16);

            // update the label for command description 
            WriteCommValue(daliTest.decode(result[0], result[1]));

            // update the bits selection
            if ((result[0] & 0x80) != 0)
            {
                bit15.Checked = true;
            }
            else
            {
                bit15.Checked = false;
            }
            if ((result[0] & 0x40) != 0)
            {
                bit14.Checked = true;
            }
            else
            {
                bit14.Checked = false;
            }
            if ((result[0] & 0x20) != 0)
            {
                bit13.Checked = true;
            }
            else
            {
                bit13.Checked = false;
            }
            if ((result[0] & 0x10) != 0)
            {
                bit12.Checked = true;
            }
            else
            {
                bit12.Checked = false;
            }
            if ((result[0] & 0x08) != 0)
            {
                bit11.Checked = true;
            }
            else
            {
                bit11.Checked = false;
            }
            if ((result[0] & 0x04) != 0)
            {
                bit10.Checked = true;
            }
            else
            {
                bit10.Checked = false;
            }
            if ((result[0] & 0x02) != 0)
            {
                bit9.Checked = true;
            }
            else
            {
                bit9.Checked = false;
            }
            if ((result[0] & 0x01) != 0)
            {
                bit8.Checked = true;
            }
            else
            {
                bit8.Checked = false;
            }

            if ((result[1] & 0x80) != 0)
            {
                bit7.Checked = true;
            }
            else
            {
                bit7.Checked = false;
            }
            if ((result[1] & 0x40) != 0)
            {
                bit6.Checked = true;
            }
            else
            {
                bit6.Checked = false;
            }
            if ((result[1] & 0x20) != 0)
            {
                bit5.Checked = true;
            }
            else
            {
                bit5.Checked = false;
            }
            if ((result[1] & 0x10) != 0)
            {
                bit4.Checked = true;
            }
            else
            {
                bit4.Checked = false;
            }
            if ((result[1] & 0x08) != 0)
            {
                bit3.Checked = true;
            }
            else
            {
                bit3.Checked = false;
            }
            if ((result[1] & 0x04) != 0)
            {
                bit2.Checked = true;
            }
            else
            {
                bit2.Checked = false;
            }
            if ((result[1] & 0x02) != 0)
            {
                bit1.Checked = true;
            }
            else
            {
                bit1.Checked = false;
            }
            if ((result[1] & 0x01) != 0)
            {
                bit0.Checked = true;
            }
            else
            {
                bit0.Checked = false;
            }
        }

        private void bit15_CheckedChanged_1(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit14_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit13_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit12_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit11_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit10_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit9_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit8_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit7_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit6_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit5_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit4_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit3_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit2_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit1_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        private void bit0_CheckedChanged(object sender, EventArgs e)
        {
            changeHexComm();
        }

        // clear the log content
        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            richTextBox1.Clear();
            daliTest.textBoxLen = 0;
        }

        // save the log content
        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = "log file.rtf";
            saveFileDialog1.Title = "Save text file";

            if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK && saveFileDialog1.FileName.Length > 0)
            {
                richTextBox1.SaveFile(saveFileDialog1.FileName);
            }
        }

        // copy the selected part of the log to clipboard
        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(richTextBox1.SelectedText);
        }

        // select 
        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            richTextBox1.SelectAll();
        }

        private int noTest()
        {
            int no = 0;
            for (int i = 0; i < testForm.treeDict.Values.Count; i++ )
            {
                if (testForm.treeDict.ElementAt(i).Value.Equals(true))
                {
                    no++;
                }
            }
            return no;
        }

        private void sendUserCommand_Click(object sender, EventArgs e)
        {
            string hexValue = commndTextBox.Text;
            commndErrorLabel.Visible = false;
            validateAdvanced = false;

            if (IsHex(CompleteString(hexValue, 4)))
            {
                validateAdvanced = true;
                UpdateCommand(CompleteString(hexValue, 4));
            }
            else
            {
                validateAdvanced = false;
                commndErrorLabel.Visible = true;
            }
            if (validateAdvanced == true)
            {
                daliTest.listSendComm(CompleteString(commndTextBox.Text, 4).ToUpper());
            }
            else 
            {
                MessageBox.Show("Insert a valid data!");
            }
        }

        private void extraTestLoop()
        {
            Byte[] incoming = new Byte[65];

            while (true)
            {
                if (daliTest.initialised == 1)
                {
                    extraTest();
                    daliTest.Message("---------- Next board -----------");

                    while (daliTest.getData(incoming, 100, 0) >= 0);
                }
                else
                {
                    InitUSB();
                }
            }
        }

        private void extraTest()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            Stopwatch stopWatch = new Stopwatch();
            Byte flagsData;
            Boolean txThisDevice, txError;
            long t0, time;
            int r, state;

            daliTest.Message("Starting test: USB to DALI Interface test");
            daliTest.Message("Checking communication");

            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_PING;

            daliTest.sendData(outgoing);
            r = daliTest.getData(incoming, 100);

            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_PING)
                {
                    daliTest.Error("Error M0: Wrong USB packet type received for PING request");
                }
                else
                {
                    daliTest.Message("\tCommunication OK.");
                }
            }
            else
            {
                daliTest.Error("Error M1: Cannot communicate with USB to DALI Interface");
                daliTest.Message("Test done");
                //timer1.Enabled = true;
                return;
            }

            daliTest.Message("Checking delay generator");
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_PING_DELAY;
            outgoing[2] = 0x00;
            outgoing[3] = 100;
            stopWatch.Start();

            daliTest.sendData(outgoing);
            r = daliTest.getData(incoming, 500);
            stopWatch.Stop();

            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_PING_DELAY)
                {
                    daliTest.Error("Error M2: Wrong USB packet type received for PING_DELAY request");
                }
                else
                {
                    daliTest.Debug("Delay = " + stopWatch.ElapsedMilliseconds.ToString() + " ms");
                    if (stopWatch.ElapsedMilliseconds < 95 || stopWatch.ElapsedMilliseconds > 105)
                    {
                        daliTest.Error("Error M3: Poor delay generator accuracy");
                    }
                }
            }
            daliTest.Message("Checking DALI bus state at the beginning of test");

            // Now, check if the cable is connected at the beginning of the test
            state = 0; // assume cable unplugged

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;

            daliTest.sendData(outgoing);
            r = daliTest.getData(incoming, 100);

            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    daliTest.Error("Wrong USB packet type received");
                }
                else
                {
                    flagsData = incoming[4];

                    txThisDevice = (flagsData & 0x08) == 0x08;
                    txError = (flagsData & 0x10) == 0x10;

                    if (txThisDevice && (!txError))
                    {
                        daliTest.Message("\tDALI bus connected at the beginning of test");
                        state = 1; // cable plugged
                    }
                    else
                    {
                        daliTest.Message("\tDALI bus not connected at the beginning of test");
                    }
                }
            }

            if (state == 1)
            {
                daliTest.Message("Sending messages on the DALI bus. USB and DALI LEDs should both indicate activity. Disconnect cable to proceed..");
            }
            else
            {
                daliTest.Message("Trying to send messages on the DALI bus. USB and DALI LEDs should both indicate activity. Connect cable to proceed..");
            }

            t0 = DateTime.Now.Ticks;
            time = t0;

            do
            {
                time = (DateTime.Now.Ticks - t0) / 10000;
                // broadcast query control gear
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x91;

                daliTest.sendData(outgoing);
                r = daliTest.getData(incoming, 100, 0);

                if (r > 0)
                {
                    if (incoming[1] == (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        flagsData = incoming[4];

                        txThisDevice = (flagsData & 0x08) == 0x08;
                        txError = (flagsData & 0x10) == 0x10;

                        if ((state == 0) && txThisDevice && (!txError))
                        {
                            daliTest.Message("\tDALI bus connected");
                            state = 1; // cable plugged
                            break;
                        }
                        else if ((state == 1) && txThisDevice && txError)
                        {
                            daliTest.Message("\tDALI bus disconnected");
                            state = 0;
                            break;
                        }
                    }
                }
                else
                {
                    daliTest.Message("\tUSB to DALI Interface unplugged.");
                    break;
                }
                Thread.Sleep(50);

            } while (time <= 2 * 60 * 1000);

            if (time > 2 * 60 * 1000)
            {
                daliTest.Error("Error M4: Time-out waiting for bus state change");
            }
            daliTest.Message("Test done");
            //timer1.Enabled = true; 
        }

        void setActive(Button obj)
        {
            obj.Enabled = true;
            obj.Visible = true;
        }

        void setInactive(Button obj)
        {
            obj.Enabled = false;
            obj.Visible = false;
        }

        private void extraTestToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (stopExtraTest == false)
            {
                timer1.Enabled = false;
                tabDebug.Enabled = false;
                tabDebug.Visible = false;

                flagInfo = showINFOToolStripMenuItem.Checked;
                flagMessages = showMESSAGESToolStripMenuItem.Checked;

                showINFOToolStripMenuItem.Checked = false;
                showMESSAGESToolStripMenuItem.Checked = true;

                setActive(buttonCloseExtraTest);

                stopExtraTest = true;

                this.DALIThread = new Thread(() => extraTestLoop());
                this.DALIThread.Start();
                DALIThread.IsBackground = true;

                extraTestToolStripMenuItem.Enabled = false;
            }
        }

        private void queryLevel_Click_1(object sender, EventArgs e)
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            Byte address;

            address = 0xFF;
            if (numericUpDown.Value > -1) address = (Byte)((Byte)numericUpDown.Value << 1 | 1);

            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = address;
            outgoing[3] = 0xA0;

            daliTest.sendData(outgoing);
            daliTest.getData(incoming, 100);
        }

        private void buttonCloseExtraTest_Click(object sender, EventArgs e)
        {
                extraTestToolStripMenuItem.Enabled = true;
                this.DALIThread.Abort();
                Thread.Sleep(100);

                setInactive(buttonCloseExtraTest);

                tabDebug.Enabled = true;
                tabDebug.Visible = true;

                daliTest.Message("======== Testing procedure was closed =======", Color.Red);

                showINFOToolStripMenuItem.Checked = flagInfo;
                showMESSAGESToolStripMenuItem.Checked = flagMessages;

                timer1.Enabled = true;
                stopExtraTest = false;
        }

        private void buttonRunExtraTest_Click(object sender, EventArgs e)
        {
            if (stopExtraTest == false)
            {

                tabDebug.Enabled = false;
                tabDebug.Visible = false;

                setActive(buttonRunExtraTest);
                setActive(buttonCloseExtraTest);

                stopExtraTest = true;

                this.DALIThread = new Thread(() => extraTest());
                this.DALIThread.Start();
                DALIThread.IsBackground = true;
            }
        }

        private void toolStripMenuItem_Click(object sender, EventArgs e)
        {
            string m  = "Copyright (C) 2014 Microchip Technology Inc. and its subsidiaries (Microchip). " + 
                        "All rights reserved.\n\nYou are permitted to use the software and its derivatives " + 
                        "with Microchip products. See the license agreement accompanying this software, if " +
                        "any, for additional info regarding your rights and obligations.\n\nSOFTWARE AND " + 
                        "DOCUMENTATION ARE PROVIDED “AS IS” WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR " +
                        "IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE, " + 
                        "NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP " +
                        "OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, " +
                        "CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY FOR ANY DIRECT OR " +
                        "INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, " + 
                        "INDIRECT OR CONSEQUENTIAL DAMAGES, OR OTHER SIMILAR COSTS. To the fullest extend " +
                        "allowed by law, Microchip and its licensors liability will not exceed the amount of " +
                        "fees that you paid directly to Microchip to use this software or $1,000, whichever is " + 
                        "greater.";
            MessageBox.Show(m);
        }

        private void testNo_ValueChanged(object sender, EventArgs e)
        {
            maxNoTests = noTest() * (int)testNo.Value;
            maxProgress.Text = "/ " + maxNoTests.ToString(); 
        }

        // save a specific command 
        private void saveToListUserComm_Click(object sender, EventArgs e)
        {
            string hexValue = commndTextBox.Text;
            commndErrorLabel.Visible = false;
            validateAdvanced = false;

            if (IsHex(CompleteString(hexValue, 4)))
            {
                validateAdvanced = true;
                UpdateCommand(CompleteString(hexValue, 4));
            }
            else
            {
                validateAdvanced = false;
                commndErrorLabel.Visible = true;
            }

            if (validateAdvanced == true)
            {
                // put the command in DataGrid
                commdList.AddRows(CompleteString(commndTextBox.Text, 4).ToUpper(), false, "150");
                // save the specific command in memory with a default structure
                commdList.listValues.Add(new ListValue(CompleteString(commndTextBox.Text, 4).ToUpper(), false, "150"));
            }
            else
            {
                MessageBox.Show("Insert a valid data!");
            }
        }

        // send a custom list
        private void sendListButton_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            this.UserCommandsThread = new Thread(() => SendCommandsList());
            this.UserCommandsThread.Start();
            this.UserCommandsThread.IsBackground = true;
            userCommandsConfigDisable();

        }

        private void SendCommandsList()
        {
            if (commdList.listCommGrid.Rows.Count != 0)
            {
                commdList.SaveLocalListConfiguration();
                for (int i = 0; i < commdList.listValues.Count; i++)
                {
                    // if the command must send twice
                    if (commdList.listValues[i].GetSend() == true)
                    {
                        daliTest.listSendCommTwice(commdList.listValues[i].GetComm());
                        daliTest.listSendDelay(Convert.ToUInt16(commdList.listValues[i].GetTime()));
                    }
                    else
                    {
                        daliTest.listSendComm(commdList.listValues[i].GetComm());
                        daliTest.listSendDelay(Convert.ToUInt16(commdList.listValues[i].GetTime()));
                    }
                    commDataGrid.Rows[i].Selected = true;
                }
            }
            Invoke(new Action(() => timer1.Enabled = true));
            Invoke(new Action(() => userCommandsConfigEnable()));
        }

        // Clean the DataGrid
        private void cleanDataGrid_Click(object sender, EventArgs e)
        {
            string message = "Do you wish to remove all the list entries?";
            MessageBoxButtons buttons = MessageBoxButtons.YesNo;
            DialogResult result;

            if (commdList.rows != 0)
            {
                result = MessageBox.Show(message, "", buttons, MessageBoxIcon.Question,
                    MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign);

                if (result == DialogResult.Yes)
                {
                    commdList.rows = 0;
                    commdList.listValues.Clear();
                    commdList.listCommGrid.Rows.Clear();
                    commdList.listCommGrid.Refresh();
                    commdList.rows = 0;
                }
            }
            else
            {
                MessageBox.Show("This list is empty.");
            }
        }

        // Delete selected row
        private void deleteRowDataGrid_Click(object sender, EventArgs e)
        {
            if (commdList.rows != 0)
            {
                commdList.rows--;
                foreach (DataGridViewRow item in commdList.listCommGrid.SelectedRows)
                {
                    commdList.listCommGrid.Rows.RemoveAt(item.Index);
                }
            }
            else
            {
                MessageBox.Show("The list is empty.");
            }
        }

        // Move selected row up
        private void moveUpLineDataGrid_Click(object sender, EventArgs e)
        {
            if (commdList.listCommGrid.RowCount > 0)
            {
                if (commdList.listCommGrid.SelectedRows.Count > 0)
                {
                    int rowCount = commdList.listCommGrid.Rows.Count;
                    int index = commdList.listCommGrid.SelectedCells[0].OwningRow.Index;

                    if (index == 0)
                    {
                        return;
                    }
                    DataGridViewRowCollection rows = commdList.listCommGrid.Rows;

                    // remove the previous row and add it behind the selected row.
                    DataGridViewRow prevRow = rows[index - 1];
                    rows.Remove(prevRow);
                    prevRow.Frozen = false;
                    rows.Insert(index, prevRow);
                    commdList.listCommGrid.ClearSelection();
                    commdList.listCommGrid.Rows[index - 1].Selected = true;
                }
            }
        }

        //Move selected row down
        private void moveDownLineDataGrid_Click(object sender, EventArgs e)
        {
            if (commdList.listCommGrid.RowCount > 0)
            {
                if (commdList.listCommGrid.SelectedRows.Count > 0)
                {
                    int rowCount = commdList.listCommGrid.Rows.Count;
                    int index = commdList.listCommGrid.SelectedCells[0].OwningRow.Index;

                    // include the header row
                    if (index == (rowCount - 1))
                    {
                        return;
                    }
                    DataGridViewRowCollection rows = commdList.listCommGrid.Rows;

                    // remove the next row and add it in front of the selected row.
                    DataGridViewRow nextRow = rows[index + 1];
                    rows.Remove(nextRow);
                    nextRow.Frozen = false;
                    rows.Insert(index, nextRow);
                    commdList.listCommGrid.ClearSelection();
                    commdList.listCommGrid.Rows[index + 1].Selected = true;
                }
            }
        }

        private void sendSelectedComm_Click(object sender, EventArgs e)
        {
            commdList.SaveLocalListConfiguration();

            if (commdList.listCommGrid.RowCount > 0)
            {
                if (commdList.listCommGrid.SelectedRows.Count > 0)
                {
                    int index = commdList.listCommGrid.SelectedCells[0].OwningRow.Index;

                    if (commdList.listValues[index].GetSend() == true)
                    {
                        daliTest.listSendCommTwice(commdList.listValues[index].GetComm());
                        daliTest.listSendDelay(Convert.ToUInt16(commdList.listValues[index].GetTime()));
                    }
                    else
                    {
                        daliTest.listSendComm(commdList.listValues[index].GetComm());
                        daliTest.listSendDelay(Convert.ToUInt16(commdList.listValues[index].GetTime()));
                    }
                }
            }
        }

        private void suportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string m =  " ------------------------------------------------------ \n" +
                        " This program was built to work with: \n" +
                        " - USB to DALI interface firmware version: 1.0 \n" +
                        " - USB to DALI interface hardware version: 219-04-2101 Rev A \n" +
                        " ------------------------------------------------------\n\n" +
                        "For more details, please visit \n" +
                        "\t\twww.microchip.com/lighting \n" +
                        "or contact the email address\n" + 
                        "\t\tlighting@microchip.com";
            MessageBox.Show(m);


        }

        private void saveCommadsListToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (commdList.rows != 0)
            {
                // save the current configuration in memory
               commdList.SaveLocalListConfiguration();

               saveCurrentConfig.FileName = "commands_onfiguration.txt";
               saveCurrentConfig.Title = "Save text file";

                if (saveCurrentConfig.FileName != "" && saveCurrentConfig.ShowDialog() == DialogResult.OK)
                {
                    // add a header whith the version file
                    File.WriteAllText(saveCurrentConfig.FileName, "version=1.0" + Environment.NewLine);
                    // write the configuration line by line
                    File.AppendAllLines(saveCurrentConfig.FileName, commdList.listValues.Select(x => x.GetComm() + "-" + x.GetSend() + "-" + x.GetTime()).ToArray());
                }
            }
            else
            {
                MessageBox.Show("List is empty.");
            }
        }

        private void loadCommadsListToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openCommConfig.ShowDialog() == DialogResult.OK)
            {
                //clear list
                commdList.listCommGrid.Rows.Clear();
                commdList.listCommGrid.Refresh();
                commdList.rows = 0;

                // read the file, line by line
                string[] line = File.ReadAllLines(openCommConfig.FileName);
                // check the file version
                string[] header = line[0].Split('=');
                string[] row;
                bool checkBox;

                if (header[1] != "1.0")
                {
                    string message = "The file is incompatible with this program version!";
                    string caption = "";
                    MessageBox.Show(message, caption, MessageBoxButtons.OK);
                    return;            
                }

                for (int i = 1; i < line.Length; i++)
                {
                    row = line[i].Split('-');
                    
                    // row[0] -> the command
                    // row[1] -> if the command will be send twice
                    // row[2] -> the time beetween two succesive commands

                    if (row[1] == "True")
                    {
                        checkBox = true;
                    }
                    else 
                    {
                        checkBox = false;
                    }
                    // add the entry
                    commdList.listCommGrid.Rows.Add(row[0], checkBox, row[2]);
                }

                commdList.rows = line.Length - 1;
                commdList.SaveLocalListConfiguration();
            }
        }

        private string CompleteString(string input, int l)
        {
            int len = input.Length;

            if (len == l)
            {
                return input;
            }
            else 
            {
                // 4 is max input lenght
                for (int i = 0; i < l - len; i++ )
                {
                    input = "0" + input;
                }
                return input;
            }
        }

        private bool IsHex(string str)
        {
            return System.Text.RegularExpressions.Regex.IsMatch(str, @"^[0-9A-Fa-f\r\n]+$");
        }

        private void tabControl1_Leave(object sender, EventArgs e)
        {
            string hexValue = commndTextBox.Text;
            commndErrorLabel.Visible = false;
            validateAdvanced = false;

            if (IsHex(CompleteString(hexValue, 4)))
            {
                validateAdvanced = true;
                UpdateCommand(CompleteString(hexValue, 4));
            }
            else
            {
                validateAdvanced = false;
                commndErrorLabel.Visible = true;
            }
        }

        private void input_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                string hexValue = commndTextBox.Text;
                commndErrorLabel.Visible = false;
                validateAdvanced = false;

                if (IsHex(CompleteString(hexValue, 4)))
                {
                    validateAdvanced = true;
                    UpdateCommand(CompleteString(hexValue, 4));
                }
                else
                {
                    validateAdvanced = false;
                    commndErrorLabel.Visible = true;
                }
            }
        }

        private void InitObjectsforBasic()
        {
            int index = commdsNameDropDown.SelectedIndex;

            WriteCommValueBasic("");
            labelHexValue.Text = "0000";

            radioButtonBroadcast.Checked = true;
            radioButtonGroupAddr.Checked = false;
            radioButtonShortAddr.Checked = false;

            // in the begining, the Addressing text field is readOnly (aka Disabled)
            textBoxAddr.ReadOnly = true;

            // if param is 0
            if (commdList.listDaliCommand[index].GetParameter() == 0)
            {
                textBoxParam.ReadOnly = true;
            }
            else
            {
                textBoxParam.ReadOnly = false;
            }

            checkBoxTwice.Checked = commdList.listDaliCommand[index].GetDefaultTwice();
            textBoxAddr.Text = "0";
            textBoxParam.Text = "0";
        }

        private void UpdateObjectsforCurrentCommd()
        {
            int index = commdsNameDropDown.SelectedIndex;

            radioButtonBroadcast.Enabled = commdList.listDaliCommand[index].GetAddressing();
            radioButtonGroupAddr.Enabled = commdList.listDaliCommand[index].GetAddressing();
            radioButtonShortAddr.Enabled = commdList.listDaliCommand[index].GetAddressing();

            textBoxAddr.ReadOnly = true;

            if (radioButtonShortAddr.Enabled || radioButtonGroupAddr.Enabled)
            {
                if (radioButtonGroupAddr.Checked || radioButtonShortAddr.Checked)
                {
                    // in the begining, the Addressing text field is readOnly (aka Disabled)
                    textBoxAddr.ReadOnly = false;
                }
            }
            // if param is 0
            if (commdList.listDaliCommand[index].GetParameter() == 0)
            {
                textBoxParam.ReadOnly = true;
            }
            else
            {
                textBoxParam.ReadOnly = false;
            }

            checkBoxTwice.Checked = commdList.listDaliCommand[index].GetDefaultTwice();
            textBoxParam.Text = "0";
        }

        private void commdsNameDropDown_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateObjectsforCurrentCommd();
        }

        private void radioButtonBroadcast_CheckedChanged(object sender, EventArgs e)
        {

            if (radioButtonBroadcast.Checked)
            {
                textBoxAddr.ReadOnly = true;
            }
        }

        private void radioButtonGroupAddr_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonGroupAddr.Checked)
            {
               textBoxAddr.ReadOnly = false;
            }
        }

        private void radioButtonShortAddr_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonShortAddr.Checked)
            {
                textBoxAddr.ReadOnly = false;
            }
        }

        private void textBoxAddr_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                ProcessCommand(commdsNameDropDown.SelectedIndex);
            }
        }

        private void textBoxAddr_TextChanged(object sender, EventArgs e)
        {
            ProcessCommand(commdsNameDropDown.SelectedIndex);
        }

        private void textBoxParam_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                ProcessCommand(commdsNameDropDown.SelectedIndex);
            }
        }

        private void textBoxParam_TextChanged(object sender, EventArgs e)
        {
            ProcessCommand(commdsNameDropDown.SelectedIndex);
        }

        // Get selected command from dropDownList, the values 
        // from text fields and return the command in hex
        // index = index of selected command
        private void ProcessCommand(int index)
        {
            // get the command map
            string map = commdList.listDaliCommand[index].GetMap();
            string address = "";
            string binaryValue;

            // make error labeld not visible
            valueLabeltext.Visible = false;
            valueLabel.Visible = false;
            labelValueParam.Visible = false;
            labelParamValue.Visible = false;
            labelHexInterpretation.Visible = true;

            // if parameters number != 0
            if (commdList.listDaliCommand[index].GetParameter() != 0)
            {
                try
                {
                    int value = Convert.ToInt32(textBoxParam.Text);
                    // data validation 
                    if (commdList.listDaliCommand[index].GetParameter() == 4 && (value > 15 || value < 0))
                    {
                        labelHexInterpretation.Visible = false;
                        labelValueParam.Visible = true;
                        labelParamValue.Visible = true;
                        labelParamValue.Text = "15";

                        validateBasicParam = false;

                        return;
                    }
                    if (commdList.listDaliCommand[index].GetParameter() == 8 && (value > 255 || value < 0))
                    {
                        labelHexInterpretation.Visible = false;
                        labelValueParam.Visible = true;
                        labelParamValue.Visible = true;
                        labelParamValue.Text = "255";

                        validateBasicParam = false;

                        return;
                    }

                    // data processing
                    if (commdList.listDaliCommand[index].GetParameter() == 4)
                    {
                        binaryValue = "";
                        binaryValue = Convert.ToString(value, 2);
                        binaryValue = CompleteString(binaryValue, 4);

                        map = map.Replace(map.Substring(12, 4), binaryValue);
                        validateBasicParam = true;
                    }
                    if (commdList.listDaliCommand[index].GetParameter() == 8)
                    {
                        binaryValue = "";
                        binaryValue = Convert.ToString(value, 2);
                        binaryValue = CompleteString(binaryValue, 8);

                        map = map.Replace(map.Substring(8, 8), binaryValue);
                        validateBasicParam = true;
                    }
                }
                catch (FormatException ex)
                {
                    validateBasicParam = false;
                    return;
                }
            }

            if (commdList.listDaliCommand[index].GetParameter() == 0)
            {
                validateBasicParam = true;
            }
     
            // if Addressing is Y
            if (commdList.listDaliCommand[index].GetAddressing() == true)
            {
                if (radioButtonBroadcast.Checked == true)
                {
                    address = "1111111"; // YAAAAAA
                    map = map.Replace(map.Substring(0, 7), address);
                    validateBasicAddr = true;
                }
               // if GroupAddress or ShortAddress are checked
                else 
                {
                    // firt of all we convert the string input into integer value and verify it 
                    try
                    {
                       int value = Convert.ToInt32(textBoxAddr.Text);
                        // data validation
                        if (radioButtonGroupAddr.Checked && (value > 15 || value < 0))
                        {
                            labelHexInterpretation.Visible = false;
                            valueLabeltext.Visible = true;
                            valueLabel.Visible = true;
                            valueLabel.Text = "15";
                            validateBasicAddr = false;
                            return;

                        }
                        if (radioButtonShortAddr.Checked && (value > 63 || value < 0))
                        {
                            labelHexInterpretation.Visible = false;
                            valueLabeltext.Visible = true;
                            valueLabel.Visible = true;
                            valueLabel.Text = "63";
                            validateBasicAddr = false;
                            return;

                        }
                        // data processing
                        if (radioButtonGroupAddr.Checked == true)
                        {
                            binaryValue = "";
                            // convert string value to binary
                            binaryValue = Convert.ToString(value, 2);
                            binaryValue = CompleteString(binaryValue, 4); // because length address is 4 bits

                            address = "100" + binaryValue;    // YAA + address value
                            map = map.Replace(map.Substring(0, 7), address);
                            validateBasicAddr = true;
                        }
                        if (radioButtonShortAddr.Checked == true)
                        {
                            binaryValue = "";

                            binaryValue = Convert.ToString(value, 2);
                            binaryValue = CompleteString(binaryValue, 6); // because length address is 6 bits

                            address = "1" + binaryValue;   // Y + address value
                            map = map.Replace(map.Substring(0, 7), address);
                            validateBasicAddr = true;
                        }
                    }
                    catch (FormatException ex)
                    {
                        validateBasicAddr = false;
                        return;
                    }
                } // else end
            } // if end
            if (commdList.listDaliCommand[index].GetAddressing() == false)
            {
                validateBasicAddr = true;
            }

            if (validateBasicAddr && validateBasicParam)
            {
                string hexValue = Convert.ToInt32(map, 2).ToString("X");
                //daliTest.Message("final map: " + map);
                //daliTest.Message("hx: " + hexValue);
                labelHexValue.Text = hexValue;
                WriteCommValueBasic(daliTest.decode((byte)Convert.ToInt32(hexValue.Substring(0, 2), 16), (byte)Convert.ToInt32(hexValue.Substring(2, 2), 16)));
            }
        }

        private void sendUserCmmdBasic_Click(object sender, EventArgs e)
        {
            ProcessCommand(commdsNameDropDown.SelectedIndex);
            if (!validateBasicAddr || !validateBasicParam)
            {
                MessageBox.Show("Insert correct input!");
                return;
            }
            else
            {
                if (checkBoxTwice.Checked == false)
                {
                    daliTest.listSendComm(labelHexValue.Text);
                }
                else 
                {
                    daliTest.listSendCommTwice(labelHexValue.Text);
                }
            }
        }

        private void saveToListBasic_Click(object sender, EventArgs e)
        {
            ProcessCommand(commdsNameDropDown.SelectedIndex);
            if (!validateBasicAddr || !validateBasicParam)
            {
                MessageBox.Show("Insert correct input!");
                return;
            }
            else
            {
                // put the command in DataGrid
                commdList.AddRows(labelHexValue.Text, checkBoxTwice.Checked, "150");
                // save the specific command in memory with a default structure
                commdList.listValues.Add(new ListValue(labelHexValue.Text, checkBoxTwice.Checked, "150"));
            }
        }
    }
}
  