﻿/*****************************************************************************
 * Copyright (C) 2014 Microchip Technology Inc. and its subsidiaries
 * (Microchip).  All rights reserved.
 * 
 * You are permitted to use the software and its derivatives with Microchip
 * products. See the license agreement accompanying this software, if any,
 * for additional info regarding your rights and obligations.
 * 
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED “AS IS” WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A
 * PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE
 * LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY,
 * CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY FOR ANY
 * DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY
 * INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, OR OTHER SIMILAR
 * COSTS. To the fullest extend allowed by law, Microchip and its licensors
 * liability will not exceed the amount of fees that you paid directly to
 * Microchip to use this software or $1,000, whichever is greater.
 * 
 *****************************************************************************
 * Revision history:
 * 22.10.2013   Mihai Cuciuc    0.1     Communication with USB device functional,
 *                                      all DALI tests implemented.
 * 22.01.2014   Oana Barahtian  1.0     Changed the application to a multi-
 *                                      threaded model, restructured user
 *                                      interface and added functionality.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;

namespace DALITestApplication
{

    public partial class TestsForm : Form
    {
        public TestsForm()
        {
            InitializeComponent();
            initDictionary(this.treeView1);
        }

        /**
         * Dictionary for treeView config: string: name of node and bool: if is checked or not
         */
        public Dictionary<string, bool> treeDict = new Dictionary<string, bool>();

        /**
         * save the treeView configuration into Dictionary
         */
        public Dictionary<string, bool> saveTreeState(TreeView tree)
        {
            for (int i = 0; i < tree.Nodes.Count; i++)
            {
                //treeDict[tree.Nodes[i].Text] = tree.Nodes[i].Checked;
                for (int j = 0; j < tree.Nodes[i].Nodes.Count; j++)
                {
                    treeDict[tree.Nodes[i].Nodes[j].Text] = tree.Nodes[i].Nodes[j].Checked;
                }
            }

            return treeDict;
        }

        /**
         * after reading the input file, modify curret treeview configuration 
         */
        public void loadTreeState(Dictionary<string, bool> dict)
        {
            int count = 0;
            for (int i = 0; i < treeView1.Nodes.Count; i++)
            {
                for (int j = 0; j < treeView1.Nodes[i].Nodes.Count; j++)
                {
                    this.treeView1.Nodes[i].Nodes[j].Checked = dict.ElementAt(count).Value;
                    count++;
                }
            }
        }

        /**
         * Initialisation of the dictionary: put all the nodes with default value - false (unchecked)
         */
        public Dictionary<string, bool> initDictionary(TreeView tree)
        {
            for (int i = 0; i < tree.Nodes.Count; i++)
            {
                // treeDict.Add(tree.Nodes[i].Text, tree.Nodes[i].Checked);
                for (int j = 0; j < tree.Nodes[i].Nodes.Count; j++)
                {
                    treeDict.Add(tree.Nodes[i].Nodes[j].Text, tree.Nodes[i].Nodes[j].Checked);
                }
            }
            return treeDict;
        }

        /**
         * Handel the "Select All" button action
         */
        private void selectAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < this.treeView1.Nodes.Count; i++)
            {
                this.treeView1.Nodes[i].Checked = true;
                for (int j = 0; j < this.treeView1.Nodes[i].Nodes.Count; j++)
                {
                    this.treeView1.Nodes[i].Nodes[j].Checked = true;
                }
            }
        }

        /**
        * Handel the "Deselect All" button action
        */
        private void deselectAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < this.treeView1.Nodes.Count; i++)
            {
                this.treeView1.Nodes[i].Checked = false;
                for (int j = 0; j < this.treeView1.Nodes[i].Nodes.Count; j++)
                {
                    this.treeView1.Nodes[i].Nodes[j].Checked = false;
                }
            }
        }

        /**
         * Load a test configuration from file
         */
        private void buttonLoad_Click(object sender, EventArgs e)
        {

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                string[] line = File.ReadAllLines(openFileDialog1.FileName);

                try
                {
                    var dict = line.Select(l => l.Split('=')).ToDictionary(a => a[0], a => a[1]);
                    if (!dict.ElementAt(0).Value.Equals(" 1.0"))
                    {
                        string message = "The file is incompatible with this program version!";
                        string caption = "";
                        MessageBox.Show(message, caption, MessageBoxButtons.OK);
                        return;
                    }

                    for (int i = 1; i < dict.Count; i++)
                    {
                        /**
                         *  if test name, from treeview, is different by test name, from input file
                         */
                        if (!this.treeDict.ElementAt(i - 1).Key.Equals(dict.ElementAt(i).Key))
                        {
                            string message = "There are issues with the configuration file, some tests are unknown!";
                            string caption = "Error Detected in Input";
                            MessageBox.Show(message, caption, MessageBoxButtons.YesNo);
                        }
                        /**
                         *  test if readed value for a specific key is true or false and write this in 
                         *  treeDict - local dictionary.
                         */
                        if (dict.ElementAt(i).Value == "True")
                        {
                            this.treeDict[dict.ElementAt(i).Key] = true;
                        }
                        if (dict.ElementAt(i).Value == "False")
                        {
                            this.treeDict[dict.ElementAt(i).Key] = false;
                        }
                    }
                    loadTreeState(this.treeDict);
                }
                catch (System.IndexOutOfRangeException ex)
                {
                    string message = "This file is incompatible!";
                    string caption = "";
                    MessageBox.Show(message, caption, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                
            }
        }

        /**
         * Save curent tree configuration 
         */
        private void buttonSave_Click(object sender, EventArgs e)
        {
            //StringBuilder sb = new StringBuilder();

            saveTreeState(this.treeView1);

            saveFileDialog1.FileName = "test_configuration.txt";
            saveFileDialog1.Title = "Save text file";

            if (saveFileDialog1.FileName != "" && saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                File.WriteAllText(saveFileDialog1.FileName, "version = 1.0" + Environment.NewLine);
                File.AppendAllLines(saveFileDialog1.FileName, treeDict.Select(x => x.Key + "=" + x.Value).ToArray());
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            saveTreeState(this.treeView1);
            bool flag = false;

            for (int i = 0; i < treeDict.Count; i++)
            {
                if (treeDict.ElementAt(i).Value.Equals(true))
                {
                    flag = true;
                }
            }

            if (flag == false)
            {
                MessageBox.Show("No test was selected!", "", MessageBoxButtons.OK);
            }

            this.Hide();
        }

        private void treeView1_AfterCheck(object sender, TreeViewEventArgs e)
        {
            if (e.Action != TreeViewAction.Unknown)
            {
                foreach (TreeNode child in e.Node.Nodes)
                {
                    //child.Checked = e.Node.Checked;
                    child.Checked = child.Parent.Checked;
                }
            }
        }
        
        private void Tests_Load_1(object sender, EventArgs e)
        {
            /* workaround to fix the bug related to bold fonts */
            treeView1.BackColor = treeView1.BackColor;
            treeView1.Update();
        }
    }
}
