﻿/*****************************************************************************
 * Copyright (C) 2014 Microchip Technology Inc. and its subsidiaries
 * (Microchip).  All rights reserved.
 * 
 * You are permitted to use the software and its derivatives with Microchip
 * products. See the license agreement accompanying this software, if any,
 * for additional info regarding your rights and obligations.
 * 
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED “AS IS” WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A
 * PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE
 * LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY,
 * CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY FOR ANY
 * DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY
 * INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, OR OTHER SIMILAR
 * COSTS. To the fullest extend allowed by law, Microchip and its licensors
 * liability will not exceed the amount of fees that you paid directly to
 * Microchip to use this software or $1,000, whichever is greater.
 * 
 *****************************************************************************
 * Revision history:
 * 22.10.2013   Mihai Cuciuc    0.1     Communication with USB device functional,
 *                                      all DALI tests implemented.
 * 22.01.2014   Oana Barahtian  1.0     Changed the application to a multi-
 *                                      threaded model, restructured user
 *                                      interface and added functionality.
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DALITestApplication.Classes;
using System.Threading;

namespace DALITestApplication
{
    class DaliTest : Utils
    {
        public NumericUpDown preaheatTimeControl;
        public NumericUpDown responseTimeControl;

        // This value is also defined in the USB to DALI Interface firmware. It is the starting
        // value for the timer when waiting for a backward frame, and it is used to time how
        // long it took the Control Gear to answer.
        public UInt16 TE22_VALUE = 65536 - 22 * 1250;

        /* Get input from popup */
        static double GetData(string s)
        {
            double readValue = 0;
            DataInputForm Measurement_InputForm = new DataInputForm();

            Measurement_InputForm.InputValue = "0";
            Measurement_InputForm.Message = s;
            Measurement_InputForm.Text = "Measurement";

            while (true)
            {
                DialogResult dialogResult = Measurement_InputForm.ShowDialog();

                if (dialogResult == DialogResult.OK)
                {
                    if (Double.TryParse(Measurement_InputForm.InputValue, out readValue))
                    {
                        break;
                    }
                }
            }

            Measurement_InputForm.Dispose();

            return readValue;
        }

        public void CheckBus()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            int r;

            // Read flags
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_READ_FLAGS;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_READ_FLAGS)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[4] & 0x02) != 0x02)
                {
                    // Bit 2 of the high byte is set if the DALI bus is connected to the board
                    Error("The USB to DALI Interface is not connected to a DALI bus. Test results might be inaccurate.");
                }
            }
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_1_1()
        {
            double readValue;
            Func<string, double> method = GetData;
            IAsyncResult cookie;

            Message("Starting test 12.1.1.1: Current rating");

            MessageBox.Show("Waveform V_HIGH := 22.5V", "Required action");
            Message("V_HIGH set to 22.5V");

            cookie = method.BeginInvoke("Measure I_DATA_WIRE and enter value (in mA) in the input field", null, null);
            readValue = method.EndInvoke(cookie);

            Message("Measured current on data wire " + readValue.ToString() + " mA");

            if (readValue > 2)
            {
                Error("Error 1111 \"Current > 2 mA\"");
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_1_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            int r;

            Message("Starting test 12.1.1.2: Voltage rating");

            MessageBox.Show("Waveform\nV_HIGH := 22.5V\nV_LOW := -6.5V\nt_Rise := t_Fall := 10μs", "Required action");
            Message("Set parameters:");
            Message("\tV_HIGH := 22.5V");
            Message("\tV_LOW := -6.5V");
            Message("\tt_Rise := t_Fall := 10μs");

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Error("Error 1121 \"No communication at VHIGH = 22.5 V, VLOW = -6.5 V\"");
                }
            }

            MessageBox.Show("Waveform\nV_HIGH := 9.5V\nV_LOW := 6.5V\nt_Rise := t_Fall := 10μs", "Required action");
            Message("Set parameters:");
            Message("\tV_HIGH := 9.5V");
            Message("\tV_LOW := 6.5V");
            Message("\tt_Rise := t_Fall := 10μs");

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Error("Error 1122 \"No communication at VHIGH = 9.5 V, VLOW = 6.5 V\"");
                }
            }


            MessageBox.Show("Waveform\nV_HIGH := 9.5V\nV_LOW := 6.5V\nt_Rise := t_Fall := 100μs", "Required action");
            Message("Set parameters:");
            Message("\tV_HIGH := 9.5V");
            Message("\tV_LOW := 6.5V");
            Message("\tt_Rise := t_Fall := 100μs");

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Error("Error 1123 \"No communication at tRISE = tFALL= 100 μs\"");
                }
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_1_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            int r;

            Message("Starting test 12.1.1.3: Back channel rise time / fall time");

            MessageBox.Show("Waveform\nV_HIGH := 17.0V\nV_LOW := 0.0V\nt_Rise := t_Fall := 50μs\nI_IPS := 250mA", "Required action");
            Message("Set parameters:");
            Message("\tV_HIGH := 17.0V");
            Message("\tV_LOW := 0.0V");
            Message("\tt_Rise := t_Fall := 50μs");
            Message("\tI_IPS := 250mA");

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);

            if (MessageBox.Show("Measure that signal from DUT has:\nV_HIGH > 11.5V\nV_LOW < 4.5V\n10μs < t_Rise < 100μs\n10μs < t_Fall < 100μs\nAre all values within the ranges?", "Required action", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 1131 \"Rise time / fall time out of range at I_IPS = 250 mA\"");
            }

            MessageBox.Show("Waveform\nV_HIGH := 15.0V\nV_LOW := 0.0V\nt_Rise := t_Fall := 50μs\nI_IPS := 8mA", "Required action");
            Message("Set parameters:");
            Message("\tV_HIGH := 15.0V");
            Message("\tV_LOW := 0.0V");
            Message("\tt_Rise := t_Fall := 50μs");
            Message("\tI_IPS := 8mA");

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);

            if (MessageBox.Show("Measure that signal from DUT has:\nV_HIGH > 11.5V\nV_LOW < 4.5V\n10μs < t_Rise < 100μs\n10μs < t_Fall < 100μs\nAre all values within the ranges?", "Required action", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 1132 \"Rise time / fall time out of range at I_IPS = 8 mA\"");
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_1_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            UInt16[] timings = new UInt16[40];
            int r, i, l;
            Boolean backwardFrameValid;

            Message("Starting test 12.1.1.4: Transmission rate");

            MessageBox.Show("Waveform\nV_HIGH := 16.0V\nV_LOW := 0.0V\nt_Rise := t_Fall := 50μs\nI_IPS := 250mA", "Required action");
            Message("Set parameters:");
            Message("\tV_HIGH := 16.0V");
            Message("\tV_LOW := 0.0V");
            Message("\tt_Rise := t_Fall := 50μs");
            Message("\tI_IPS := 250mA");

            Message("Sending BROADCAST QUERY CONTROL GEAR using data rate 1080Hz");

            i = 0;
            // 9 '1' bits
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            timings[i++] = 1386;
            // add sequence '10'
            timings[i++] = 1386;
            timings[i++] = 2 * 1386;
            timings[i++] = 1386;
            // add sequence '010'
            timings[i++] = 1386;
            timings[i++] = 2 * 1386;
            timings[i++] = 2 * 1386;
            timings[i++] = 1386;
            // add '0'
            timings[i++] = 1386;
            timings[i++] = 1386;
            // add sequence '01'
            timings[i++] = 1386;
            timings[i++] = 2 * 1386;
            l = i;

            // resetting timing list
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_RESET_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_RESET_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            // loading timing values
            i = 0;
            for (i = 0; i < l; i++)
            {
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_LOAD_DATA_TIMING_LIST;
                outgoing[2] = 0x00;
                outgoing[3] = (Byte)(((UInt16)(65536 - timings[i])) >> 8);
                outgoing[4] = (Byte)(((UInt16)(65536 - timings[i])) & 0xFF);
                outgoing[5] = 0xFF;
                outgoing[6] = 0xFF;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_LOAD_DATA_TIMING_LIST)
                    {
                        Debug(incoming[1].ToString());
                        Error("Wrong USB packet type received");
                    }
                }
            }

            // sending command
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_SEND_DATA_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_SEND_DATA_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.UNSOL_RECEIVE_DATA)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
                else
                {
                    backwardFrameValid = (incoming[5] & 0x02) == 0x02; // incoming[5] == flagsData, bit 2 is "backwardFrameValid"
                    if ((!backwardFrameValid) || (incoming[4] != 0xFF))
                    {
                        Error("Error 1141 \"No communication with minimum transmission rate\"");
                    }
                }
            }

            Message("Sending BROADCAST QUERY CONTROL GEAR using data rate 1320Hz");

            i = 0;
            // 9 '1' bits
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            timings[i++] = 1136;
            // add sequence '10'
            timings[i++] = 1136;
            timings[i++] = 2 * 1136;
            timings[i++] = 1136;
            // add sequence '010'
            timings[i++] = 1136;
            timings[i++] = 2 * 1136;
            timings[i++] = 2 * 1136;
            timings[i++] = 1136;
            // add '0'
            timings[i++] = 1136;
            timings[i++] = 1136;
            // add sequence '01'
            timings[i++] = 1136;
            timings[i++] = 2 * 1136;
            l = i;

            // resetting timing list
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_RESET_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_RESET_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            // loading timing values
            i = 0;
            for (i = 0; i < l; i++)
            {
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_LOAD_DATA_TIMING_LIST;
                outgoing[2] = 0x00;
                outgoing[3] = (Byte)(((UInt16)(65536 - timings[i])) >> 8);
                outgoing[4] = (Byte)(((UInt16)(65536 - timings[i])) & 0xFF);
                outgoing[5] = 0xFF;
                outgoing[6] = 0xFF;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_LOAD_DATA_TIMING_LIST)
                    {
                        Debug(incoming[1].ToString());
                        Error("Wrong USB packet type received");
                    }
                }
            }

            // sending command
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_SEND_DATA_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_SEND_DATA_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.UNSOL_RECEIVE_DATA)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
                else
                {
                    backwardFrameValid = (incoming[5] & 0x02) == 0x02; // incoming[5] == flagsData, bit 2 is "backwardFrameValid"
                    if ((!backwardFrameValid) || (incoming[4] != 0xFF))
                    {
                        Error("Error 1142 \"No communication with maximum transmission rate\"");
                    }
                }
            }


            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_1_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            UInt16[] timings = new UInt16[40];
            int r, i, l;
            Boolean backwardFrameValid;

            Message("Starting test 12.1.1.5: Pulse width");

            MessageBox.Show("Waveform\nV_HIGH := 9.5V\nV_LOW := 6.5V\nt_Rise := t_Fall := 100μs\nI_IPS := 250mA", "Required action");
            Message("Set parameters:");
            Message("\tV_HIGH := 9.5V");
            Message("\tV_LOW := 6.5V");
            Message("\tt_Rise := t_Fall := 100μs");
            Message("\tI_IPS := 250mA");

            i = 0;
            // 9 '1' bits
            timings[i++] = 1386; // A1
            timings[i++] = 1386; // A2
            timings[i++] = 1386; // B1
            timings[i++] = 1386; // B2
            timings[i++] = 1136; // C1
            timings[i++] = 1386; // C2
            timings[i++] = 1386; // D1
            timings[i++] = 1386; // D2
            timings[i++] = 1386; // E1
            timings[i++] = 1136; // E2
            timings[i++] = 1136; // F1
            timings[i++] = 1136; // F2
            timings[i++] = 1136; // G1
            timings[i++] = 1386; // G2
            timings[i++] = 1136; // H1
            timings[i++] = 1136; // H2
            timings[i++] = 1136; // I1
            timings[i++] = 1136; // I2
            // add sequence '10'
            timings[i++] = 1386; // K1
            timings[i++] = 1136 + 1136; // K2 + L1
            timings[i++] = 1386; // L2
            // add sequence '010'
            timings[i++] = 1136; // M1
            timings[i++] = 1386 + 1386; // M2 + N1
            timings[i++] = 1386 + 1386; // N2 + O1
            timings[i++] = 1136; // O2
            // add '0'
            timings[i++] = 1386; // P1
            timings[i++] = 1136; // P2
            // add sequence '01'
            timings[i++] = 1136; // R1
            timings[i++] = 1136 + 1136; // R2 + S1
            l = i;

            // resetting timing list
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_RESET_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_RESET_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            // loading timing values
            i = 0;
            for (i = 0; i < l; i++)
            {
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_LOAD_DATA_TIMING_LIST;
                outgoing[2] = 0x00;
                outgoing[3] = (Byte)(((UInt16)(65536 - timings[i])) >> 8);
                outgoing[4] = (Byte)(((UInt16)(65536 - timings[i])) & 0xFF);
                outgoing[5] = 0xFF;
                outgoing[6] = 0xFF;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_LOAD_DATA_TIMING_LIST)
                    {
                        Debug(incoming[1].ToString());
                        Error("Wrong USB packet type received");
                    }
                }
            }

            // sending command
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_SEND_DATA_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_SEND_DATA_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.UNSOL_RECEIVE_DATA)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
                else
                {
                    backwardFrameValid = (incoming[5] & 0x02) == 0x02; // incoming[5] == flagsData, bit 2 is "backwardFrameValid"
                    if ((!backwardFrameValid) || (incoming[4] != 0xFF))
                    {
                        Error("Error 1151 \"No communication at pulse width test\"");
                    }
                }
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_1_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            UInt16[,] timings = new UInt16[22, 40];
            int i;
            int r, k, kMax;
            Boolean backwardFrameValid;

            Message("Starting test 12.1.1.6: Code violation");

            // Waveform k = 1
            k = 1;
            i = 0;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;


            // Waveform k = 2
            k = 2;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 3
            k = 3;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 4
            k = 4;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 5
            k = 5;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 6
            k = 6;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 7
            k = 7;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 8
            k = 8;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 9
            k = 9;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 10
            k = 10;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 11
            k = 11;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 4 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 12
            k = 12;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 13
            k = 13;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 14
            k = 14;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 4 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 15
            k = 15;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 16
            k = 16;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 0;

            // Waveform k = 17
            k = 17;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 0;

            // Waveform k = 18
            k = 18;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 0;

            // Waveform k = 19
            k = 19;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 0;

            // Waveform k = 20
            k = 20;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 3 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 0;

            // Waveform k = 21
            k = 21;
            i = 0;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 2 * 1249;
            timings[k, i++] = 4 * 1249;
            timings[k, i++] = 1249;
            timings[k, i++] = 0;


            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r =getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait 300ms + 710ms");
            Thread.Sleep(1010);

            k = 1;
            kMax = 21;

            for (; k <= kMax; k++)
            {
                // broadcast query control gear
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x91;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                    {
                        Error("Error 1161," + k.ToString() + " \"No communication possible at beginning of test step " + k.ToString() + "\"");
                    }
                }


                // resetting timing list
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_RESET_TIMING_LIST;
                outgoing[2] = 0x00;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_RESET_TIMING_LIST)
                    {
                        Debug(incoming[1].ToString());
                        Error("Wrong USB packet type received");
                    }
                }

                // loading timing values
                i = 0;
                while (timings[k, i] != 0)
                {
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_LOAD_DATA_TIMING_LIST;
                    outgoing[2] = 0x00;
                    outgoing[3] = (Byte)(((UInt16)(65536 - timings[k, i])) >> 8);
                    outgoing[4] = (Byte)(((UInt16)(65536 - timings[k, i])) & 0xFF);
                    outgoing[5] = 0xFF;
                    outgoing[6] = 0xFF;
                    sendData(outgoing);
                    r =getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_LOAD_DATA_TIMING_LIST)
                        {
                            Debug(incoming[1].ToString());
                            Error("Wrong USB packet type received");
                        }
                    }
                    i++;
                }

                // sending command
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_SEND_DATA_TIMING_LIST;
                outgoing[2] = 0x00;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_SEND_DATA_TIMING_LIST)
                    {
                        Debug(incoming[1].ToString());
                        Error("Wrong USB packet type received");
                    }
                }

                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.UNSOL_RECEIVE_DATA)
                    {
                        Debug(incoming[1].ToString());
                        Error("Wrong USB packet type received");
                    }
                    else
                    {
                        backwardFrameValid = (incoming[5] & 0x02) == 0x02; // incoming[5] == flagsData, bit 2 is "backwardFrameValid"
                        if (backwardFrameValid && (incoming[4] == 0xFF))
                        {
                            Error("Error 1162," + k.ToString() + " \"Query with code violation answered at test step " + k.ToString() + "\"");
                        }
                    }
                }

                // Since the previous command was a timing list, DALI specific timing may be violated. However, the firmware will not
                // allow this, and this 50ms delay assures us that we will not try to transmit data too soon on the DALI bus after the
                // previous reception
                Thread.Sleep(50);

                // broadcast query reset state
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x95;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 1, 255");
                        Error("Error 1163," + k.ToString() + " \"STATUS changed after query with code violation at test step " + k.ToString() + "\"");
                    }
                }
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            UInt16[] timings = new UInt16[120];
            int r, i, l;
            Byte flagsData, backwardData;
            UInt16 backwardFrameDelay;
            double delay;
            Boolean backwardFrameValid, backwardFrameDelayValid, txThisDevice, txError, rxTimingError, rxError;


            Message("Starting test 12.1.2: Frame structure timing");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data

            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
           
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait 300ms + 710ms");
            Thread.Sleep(1010);


            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    backwardData = incoming[3];
                    flagsData = incoming[4];
                    backwardFrameDelay = (UInt16)((incoming[7] << 8) | incoming[8]);

                    delay = (backwardFrameDelay - TE22_VALUE) / 3; // ( (65536 - value) / 3MHz ) * 1000000 = us

                    backwardFrameValid = (flagsData & 0x02) == 0x02;
                    backwardFrameDelayValid = (flagsData & 0x04) == 0x04;
                    txThisDevice = (flagsData & 0x08) == 0x08;
                    txError = (flagsData & 0x10) == 0x10;
                    rxTimingError = (flagsData & 0x20) == 0x20;
                    rxError = (flagsData & 0x40) == 0x40;

                    Debug("Delay: " + delay);

                    if (txThisDevice && (!txError) && (!rxError) && backwardFrameValid && backwardFrameDelayValid)
                    {
                        if ((delay < 2917) || (delay > 9167))
                        {
                            Error("Error 1211 \"tANSWER out of range\"");
                        }
                    }
                    else
                    {
                        Error("Tx/Rx Error");
                    }

                }
            }

            i = 0;
            // FF60 (Broadcast add to group 0)
            // 8 '1' bits
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            // '10110'
            timings[i++] = 1249;
            timings[i++] = 2 * 1249;
            timings[i++] = 2 * 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 2 * 1249;
            timings[i++] = 1249;
            // 4 '0' bits
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;

            // delay. 2 stop bits + 9167us = 10833.666us * 3 = 32501 counts
            timings[i++] = 32501 + 441;

            // FF60 (Broadcast add to group 0)
            // 8 '1' bits
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            // '10110'
            timings[i++] = 1249;
            timings[i++] = 2 * 1249;
            timings[i++] = 2 * 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 2 * 1249;
            timings[i++] = 1249;
            // 4 '0' bits
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;

            // delay. 2 stop bits + 9167us = 10833.666us * 3 = 32501 counts
            timings[i++] = 32501 + 441;

            // FFC0 (Broadcast query groups 0-7)
            // 10 '1' bits
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            // '10'
            timings[i++] = 1249;
            timings[i++] = 2 * 1249;
            timings[i++] = 1249;
            // 5 '0' bits
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            timings[i++] = 1249;
            l = i;

            // resetting timing list
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_RESET_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_RESET_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            // loading timing values
            i = 0;
            for (i = 0; i < l; i++)
            {
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_LOAD_DATA_TIMING_LIST;
                outgoing[2] = 0x00;
                outgoing[3] = (Byte)(((UInt16)(65536 - timings[i])) >> 8);
                outgoing[4] = (Byte)(((UInt16)(65536 - timings[i])) & 0xFF);
                outgoing[5] = 0xFF;
                outgoing[6] = 0xFF;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_LOAD_DATA_TIMING_LIST)
                    {
                        Debug(incoming[1].ToString());
                        Error("Wrong USB packet type received");
                    }
                }
            }

            // sending command
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_SEND_DATA_TIMING_LIST;
            outgoing[2] = 0x00;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_SEND_DATA_TIMING_LIST)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
            }

            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.UNSOL_RECEIVE_DATA)
                {
                    Debug(incoming[1].ToString());
                    Error("Wrong USB packet type received");
                }
                else
                {
                    backwardFrameValid = (incoming[5] & 0x02) == 0x02; // incoming[5] == flagsData, bit 2 is "backwardFrameValid"
                    if ((!backwardFrameValid) || (incoming[4] != 0x01))
                    {
                        Error("Error 1212 \"No reaction at minimum time between commands\"");
                    }
                }
            }


            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_3()
        {
            Message("Starting test 12.1.3: Insulation test");

            MessageBox.Show("Test according to 15.5 of IEC 61347-2-3 (basis insulation):\nTest the insulation of the control terminals towards\n\ta) shorted power and lamp terminals\n\tb) earth potential", "Required action");

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_4_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            int r;

            Message("Starting test 12.1.4.1: Polarity");

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    // change ref 0
                    // original code: Error("Error 1411 \"No communication at polarity A\"");
                    Error("Error 1411 \"No communication at this polarity\"");
                }
            }


            MessageBox.Show("data wire\nChange polarity of data wires\na<->b", "Required action");

            Message("Polarity changed");

            // broadcast query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    // change ref 1
                    // original code: Error("Error 1412 \"No communication at polarity A\"");
                    Error("Error 1412 \"No communication at this polarity\"");
                }
            }


            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_4_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            int r;
            long t0, time;

            Message("Starting test 12.1.4.2: Overvoltage protection");

            MessageBox.Show("data wire\nconnect mains power supply to control terminals", "Required action");

            Message("Mains power applied to control terminals");

            Message("Wait 1 minute");
            for (int d = 60; d > 0; d--)
            {
                if (d % 5 == 0) Message("\t" + d.ToString() + "s remaining");
                Thread.Sleep(1000);
            }

            MessageBox.Show("data wire\ndisconnect mains\nreconnect PC interface", "Required action");
            Message("Mains disconnected from the control terminals, PC interface reconnected");

            MessageBox.Show("mains terminals\nconnect mains\n(rated voltage)", "Required action");
            Message("Mains reconnected to power terminals");

            // flush input buffer
            while (getData(incoming, 100, 0) != 0)
                ;

            t0 = DateTime.Now.Ticks;
            time = t0;

            do
            {
                time = (DateTime.Now.Ticks - t0) / 10000;
                // broadcast query control gear
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x91;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] == (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                        {
                            break;
                        }
                    }
                }

            } while (time <= 15 * 60 * 1000);

            if (time > 15 * 60 * 1000)
            {
                Error("Error 1421 \"No communication after over voltage at control terminals\"");
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_1_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            int r;
            double readValue;
            Func<string, double> method = GetData;
            IAsyncResult cookie;

            Message("Starting test 12.1.5: Response time");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait 300ms + 710ms");
            Thread.Sleep(1010);

            // broadcast query phisical minimum level
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            Debug("PHM = " + incoming[3].ToString());

            if (incoming[3] == 254)
            {
                Message("Non dimmable control gear");
                Message("Test done");
                return;
            }

            // broadcast dapc 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 1;
            sendData(outgoing);
            r =getData(incoming, 100);

            cookie = method.BeginInvoke("Measure response time and enter value in the input field (seconds)", null, null);
            readValue = method.EndInvoke(cookie);

            Message("Response time = " + readValue.ToString() + " s");
            if (readValue >= 0.7)
            {
                // change ref 17
                Error("Error 1511 \"Response time too long (> 0.7 s)\"");
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] command = new Byte[40];

            Byte[] query = new Byte[25];

            Byte[] value = new Byte[25];
            String[] errorText = new String[25];

            int r, i, iMax, k, kMax;
            Byte PHM = 0;

            iMax = 39;
            kMax = 24;


            command[1] = 0x60; // add to group 0
            command[2] = 0x61; // add to group 1
            command[3] = 0x62; // add to group 2
            command[4] = 0x63; // add to group 3
            command[5] = 0x64; // add to group 4
            command[6] = 0x65; // add to group 5
            command[7] = 0x66; // add to group 6
            command[8] = 0x67; // add to group 7
            command[9] = 0x68; // add to group 8
            command[10] = 0x69; // add to group 9
            command[11] = 0x6A; // add to group 10
            command[12] = 0x6B; // add to group 11
            command[13] = 0x6C; // add to group 12
            command[14] = 0x6D; // add to group 13
            command[15] = 0x6E; // add to group 14
            command[16] = 0x6F; // add to group 15

            command[17] = 0x40; // store DTR as scene 0
            command[18] = 0x41; // store DTR as scene 1
            command[19] = 0x42; // store DTR as scene 2
            command[20] = 0x43; // store DTR as scene 3
            command[21] = 0x44; // store DTR as scene 4
            command[22] = 0x45; // store DTR as scene 5
            command[23] = 0x46; // store DTR as scene 6
            command[24] = 0x47; // store DTR as scene 7
            command[25] = 0x48; // store DTR as scene 8
            command[26] = 0x49; // store DTR as scene 9
            command[27] = 0x4A; // store DTR as scene 10
            command[28] = 0x4B; // store DTR as scene 11
            command[29] = 0x4C; // store DTR as scene 12
            command[30] = 0x4D; // store DTR as scene 13
            command[31] = 0x4E; // store DTR as scene 14
            command[32] = 0x4F; // store DTR as scene 15

            command[33] = 0x2A; // store DTR as max level
            command[34] = 0x2B; // store DTR as min level
            command[35] = 0x2C; // store DTR as system failure level
            command[36] = 0x2D; // store DTR as power on level
            command[37] = 0x2E; // store DTR as fade time
            command[38] = 0x2F; // store DTR as fade rate
            command[39] = 0x00; // OFF


            query[1] = 0xC0; // QUERY GROUP 0-7
            query[2] = 0xC1; // QUERY GROUP 8-15
            query[3] = 0xB0; // QUERY scene level 0
            query[4] = 0xB1; // QUERY scene level 1
            query[5] = 0xB2; // QUERY scene level 2
            query[6] = 0xB3; // QUERY scene level 3
            query[7] = 0xB4; // QUERY scene level 4
            query[8] = 0xB5; // QUERY scene level 5
            query[9] = 0xB6; // QUERY scene level 6
            query[10] = 0xB7; // QUERY scene level 7
            query[11] = 0xB8; // QUERY scene level 8
            query[12] = 0xB9; // QUERY scene level 9
            query[13] = 0xBA; // QUERY scene level 10
            query[14] = 0xBB; // QUERY scene level 11
            query[15] = 0xBC; // QUERY scene level 12
            query[16] = 0xBD; // QUERY scene level 13
            query[17] = 0xBE; // QUERY scene level 14
            query[18] = 0xBF; // QUERY scene level 15

            query[19] = 0xA1; // QUERY max level
            query[20] = 0xA2; // QUERY min level

            query[21] = 0xA4; // QUERY system failure level
            query[22] = 0xA3; // QUERY power on level
            query[23] = 0xA5; // QUERY fade time/fade rate
            query[24] = 0xA0; // QUERY actual level

            value[1] = 0x00;
            value[2] = 0x00;
            value[3] = 255;
            value[4] = 255;
            value[5] = 255;
            value[6] = 255;
            value[7] = 255;
            value[8] = 255;
            value[9] = 255;
            value[10] = 255;
            value[11] = 255;
            value[12] = 255;
            value[13] = 255;
            value[14] = 255;
            value[15] = 255;
            value[16] = 255;
            value[17] = 255;
            value[18] = 255;
            value[19] = 254;
            value[20] = 0;
            value[21] = 254;
            value[22] = 254;
            value[23] = 0x07;
            value[24] = 254;

            errorText[1] = "GROUP O-7";
            errorText[2] = "GROUP 8-15";
            errorText[3] = "SCENE 0";
            errorText[4] = "SCENE 1";
            errorText[5] = "SCENE 2";
            errorText[6] = "SCENE 3";
            errorText[7] = "SCENE 4";
            errorText[8] = "SCENE 5";
            errorText[9] = "SCENE 6";
            errorText[10] = "SCENE 7";
            errorText[11] = "SCENE 8";
            errorText[12] = "SCENE 9";
            errorText[13] = "SCENE 10";
            errorText[14] = "SCENE 11";
            errorText[15] = "SCENE 12";
            errorText[16] = "SCENE 13";
            errorText[17] = "SCENE 14";
            errorText[18] = "SCENE 15";
            errorText[19] = "MAX LEVEL";
            errorText[20] = "MIN LEVEL";
            errorText[21] = "SYSTEM FAILURE LEVEL";
            errorText[22] = "POWER ON LEVEL";
            errorText[23] = "FADE TIME / FADE RATE";
            errorText[24] = "ACTUAL LEVEL";

            Message("Starting test 12.2.1.1: RESET");

            // broadcast query phm
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }
            Debug("PHM = " + PHM.ToString());
            value[20] = PHM;

            // dtr = 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 0x01;
            sendData(outgoing);
            r =getData(incoming, 100);


            // loop 1
            for (i = 1; i <= iMax; i++)
            {

                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = command[i]; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = command[i]; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
            }

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Preheat Time + Response Time + 300ms = " + (preaheatTimeControl.Value + preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value + 300);


            // loop 2
            for (k = 1; k <= kMax; k++)
            {
                // broadcast query k
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = query[k];
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != value[k])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + value[k].ToString());
                            Error("ERROR (2111," + k.ToString() + ") \"No RESET of " + errorText[k] + "\"");
                        }
                    }
                }
            }

            // broadcast query reset state
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x95;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 1, 255");
                    Error("Error (2112) \"Wrong answer at QUERY RESET STATE\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Debug("\tReceived no answer");
                    Error("Error (2113) \"Wrong answer at QUERY STATUS\"");
                }
                else
                {
                    if ((incoming[3] & 0xB8) != 0x20) // check if status == 0x100xxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)0x100xxx");
                        Error("Error (2113) \"Wrong answer at QUERY STATUS\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            int r;

            Message("Starting test 12.2.1.2: RESET: timeout / command in-between");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }



            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x BROADCAST ADD TO GROUP 0
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x60; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x60; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            // BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x20;
            sendData(outgoing);
            r =getData(incoming, 100);


            Info("Wait 150ms");
            Thread.Sleep(150);


            // BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x20;
            sendData(outgoing);
            r =getData(incoming, 100);


            Info("Wait 300ms");
            Thread.Sleep(300);


            // BROADCAST QUERY GROUPS 0-7
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC0;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] != 0x01)
                {
                    Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 1");
                    Error("ERROR 2121 \"RESET with time-out executed\"");
                }
            }


            // 2x BROADCAST ADD TO GROUP 0
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x60; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x60; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            // BROADCAST RESET, BROADCAST QUERY CONTROL GEAR, BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 2 flags
            outgoing[10] = 0xFF; // Command 2 addr
            outgoing[11] = 0x20; // Command 2 data
            sendData(outgoing);
            r =getData(incoming, 400); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x03)
                {
                    Error("Command sequence error");
                }
                else if (incoming[6] != 0x00)
                {
                    Error("ERROR 2122 \"Command in-between RESET executed\"");
                }
            }

            Info("Wait 300ms");
            Thread.Sleep(300);

            // BROADCAST QUERY GROUPS 0-7
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC0;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] != 0x01)
                {
                    Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 1");
                    Error("ERROR 2123 \"RESET with command in-between executed\"");
                }
            }


            // 2x BROADCAST ADD TO GROUP 0
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x60; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x60; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            // BROADCAST RESET, BROADCAST GROUP 5 QUERY CONTROL GEAR, BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0x8B; // Command 1 addr
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 2 flags
            outgoing[10] = 0xFF; // Command 2 addr
            outgoing[11] = 0x20; // Command 2 data
            sendData(outgoing);
            r =getData(incoming, 400); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x03)
                {
                    Error("Command sequence error");
                }
                else if (incoming[6] != 0x00)
                {
                    Error("ERROR 2124 \"Command in-between RESET addressed to other control gear executed\"");
                }
            }


            Info("Wait 300ms");
            Thread.Sleep(300);


            // SEND BROADCAST QUERY GROUPS 0-7
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC0;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] != 0x00)
                {
                    Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 0");
                    Error("ERROR 2125 \"RESET not executed with command in-between addressed to other control gear\"");
                }
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] command = new Byte[39];

            Byte[] query = new Byte[24];

            Byte[] value = new Byte[24];
            String[] errorText = new String[24];

            int r, i, iMax, k, kMax;
            Byte PHM = 0;

            iMax = 38;
            kMax = 23;


            command[1] = 0x60; // add to group 0
            command[2] = 0x61; // add to group 1
            command[3] = 0x62; // add to group 2
            command[4] = 0x63; // add to group 3
            command[5] = 0x64; // add to group 4
            command[6] = 0x65; // add to group 5
            command[7] = 0x66; // add to group 6
            command[8] = 0x67; // add to group 7
            command[9] = 0x68; // add to group 8
            command[10] = 0x69; // add to group 9
            command[11] = 0x6A; // add to group 10
            command[12] = 0x6B; // add to group 11
            command[13] = 0x6C; // add to group 12
            command[14] = 0x6D; // add to group 13
            command[15] = 0x6E; // add to group 14
            command[16] = 0x6F; // add to group 15

            command[17] = 0x40; // store DTR as scene 0
            command[18] = 0x41; // store DTR as scene 1
            command[19] = 0x42; // store DTR as scene 2
            command[20] = 0x43; // store DTR as scene 3
            command[21] = 0x44; // store DTR as scene 4
            command[22] = 0x45; // store DTR as scene 5
            command[23] = 0x46; // store DTR as scene 6
            command[24] = 0x47; // store DTR as scene 7
            command[25] = 0x48; // store DTR as scene 8
            command[26] = 0x49; // store DTR as scene 9
            command[27] = 0x4A; // store DTR as scene 10
            command[28] = 0x4B; // store DTR as scene 11
            command[29] = 0x4C; // store DTR as scene 12
            command[30] = 0x4D; // store DTR as scene 13
            command[31] = 0x4E; // store DTR as scene 14
            command[32] = 0x4F; // store DTR as scene 15

            command[33] = 0x2A; // store DTR as max level
            command[34] = 0x2B; // store DTR as min level
            command[35] = 0x2C; // store DTR as system failure level
            command[36] = 0x2D; // store DTR as power on level
            command[37] = 0x2E; // store DTR as fade time
            command[38] = 0x2F; // store DTR as fade rate


            query[1] = 0xC0; // QUERY GROUP 0-7
            query[2] = 0xC1; // QUERY GROUP 8-15
            query[3] = 0xB0; // QUERY scene level 0
            query[4] = 0xB1; // QUERY scene level 1
            query[5] = 0xB2; // QUERY scene level 2
            query[6] = 0xB3; // QUERY scene level 3
            query[7] = 0xB4; // QUERY scene level 4
            query[8] = 0xB5; // QUERY scene level 5
            query[9] = 0xB6; // QUERY scene level 6
            query[10] = 0xB7; // QUERY scene level 7
            query[11] = 0xB8; // QUERY scene level 8
            query[12] = 0xB9; // QUERY scene level 9
            query[13] = 0xBA; // QUERY scene level 10
            query[14] = 0xBB; // QUERY scene level 11
            query[15] = 0xBC; // QUERY scene level 12
            query[16] = 0xBD; // QUERY scene level 13
            query[17] = 0xBE; // QUERY scene level 14
            query[18] = 0xBF; // QUERY scene level 15

            query[19] = 0xA1; // QUERY max level
            query[20] = 0xA2; // QUERY min level

            query[21] = 0xA4; // QUERY system failure level
            query[22] = 0xA3; // QUERY power on level
            query[23] = 0xA5; // QUERY fade time/fade rate


            value[1] = 0x00;
            value[2] = 0x00;
            value[3] = 255;
            value[4] = 255;
            value[5] = 255;
            value[6] = 255;
            value[7] = 255;
            value[8] = 255;
            value[9] = 255;
            value[10] = 255;
            value[11] = 255;
            value[12] = 255;
            value[13] = 255;
            value[14] = 255;
            value[15] = 255;
            value[16] = 255;
            value[17] = 255;
            value[18] = 255;
            value[19] = 254;
            value[20] = 0;
            value[21] = 254;
            value[22] = 254;
            value[23] = 0x07;

            errorText[1] = "GROUP O-7";
            errorText[2] = "GROUP 8-15";
            errorText[3] = "SCENE 0";
            errorText[4] = "SCENE 1";
            errorText[5] = "SCENE 2";
            errorText[6] = "SCENE 3";
            errorText[7] = "SCENE 4";
            errorText[8] = "SCENE 5";
            errorText[9] = "SCENE 6";
            errorText[10] = "SCENE 7";
            errorText[11] = "SCENE 8";
            errorText[12] = "SCENE 9";
            errorText[13] = "SCENE 10";
            errorText[14] = "SCENE 11";
            errorText[15] = "SCENE 12";
            errorText[16] = "SCENE 13";
            errorText[17] = "SCENE 14";
            errorText[18] = "SCENE 15";
            errorText[19] = "MAX LEVEL";
            errorText[20] = "MIN LEVEL";
            errorText[21] = "SYSTEM FAILURE LEVEL";
            errorText[22] = "POWER ON LEVEL";
            errorText[23] = "FADE TIME / FADE RATE";

            Message("Starting test 12.2.1.3: 100 ms - timeout");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            value[20] = PHM;
            Debug("PHM = " + PHM.ToString());

            // BROADCAST DTR(1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 0x01;
            sendData(outgoing);
            r =getData(incoming, 100);

            // loop 1
            for (i = 1; i <= iMax; i++)
            {
                // BROADCAST command i
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = command[i];
                sendData(outgoing);
                r =getData(incoming, 100);

                Info("Wait 150ms");
                Thread.Sleep(150);

                // BROADCAST command i
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = command[i];
                sendData(outgoing);
                r =getData(incoming, 100);

                /* See IEC 62386-102 11.2.1 General configuration commands, page 30
                 * 
                 * Added 150ms delay to overcome this.
                */

                // change ref 2
                Info("Wait 150ms");
                Thread.Sleep(150);
            }

            // loop 2
            for (k = 1; k <= kMax; k++)
            {
                // SEND BROADCAST query k
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = query[k];
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != value[k])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + value[k].ToString());
                            Error("ERROR (2131," + k.ToString() + ") \"Configuration of " + errorText[k] + "\"");
                        }
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] command = new Byte[39];

            Byte[] level = new Byte[39];

            Byte[] query = new Byte[25];
            Byte[,] value = new Byte[25, 4];

            Byte[] address = new Byte[4];

            String[] errorText = new String[25];

            int r, i, iMax, k, kMax, a, aMax;
            Byte PHM = 0;

            aMax = 3;


            command[1] = 0x2A; // store DTR as max level
            command[2] = 0x2B; // store DTR as min level
            command[3] = 0x40; // store DTR as scene 0
            command[4] = 0x41; // store DTR as scene 1
            command[5] = 0x42; // store DTR as scene 2
            command[6] = 0x43; // store DTR as scene 3
            command[7] = 0x44; // store DTR as scene 4
            command[8] = 0x45; // store DTR as scene 5
            command[9] = 0x46; // store DTR as scene 6
            command[10] = 0x47; // store DTR as scene 7
            command[11] = 0x48; // store DTR as scene 8
            command[12] = 0x49; // store DTR as scene 9
            command[13] = 0x4A; // store DTR as scene 10
            command[14] = 0x4B; // store DTR as scene 11
            command[15] = 0x4C; // store DTR as scene 12
            command[16] = 0x4D; // store DTR as scene 13
            command[17] = 0x4E; // store DTR as scene 14
            command[18] = 0x4F; // store DTR as scene 15

            command[19] = 0x2C; // store DTR as system failure level
            command[20] = 0x2D; // store DTR as power on level
            command[21] = 0x2E; // store DTR as fade time
            command[22] = 0x2F; // store DTR as fade rate

            command[23] = 0x60; // add to group 0
            command[24] = 0x61; // add to group 1
            command[25] = 0x62; // add to group 2
            command[26] = 0x63; // add to group 3
            command[27] = 0x64; // add to group 4
            command[28] = 0x65; // add to group 5
            command[29] = 0x66; // add to group 6
            command[30] = 0x67; // add to group 7
            command[31] = 0x68; // add to group 8
            command[32] = 0x69; // add to group 9
            command[33] = 0x6A; // add to group 10
            command[34] = 0x6B; // add to group 11
            command[35] = 0x6C; // add to group 12
            command[36] = 0x6D; // add to group 13
            command[37] = 0x6E; // add to group 14
            command[38] = 0x6F; // add to group 15


            for (i = 3; i <= 38; i++)
            {
                level[i] = 10;
            }


            query[1] = 0xC0; // QUERY GROUP 0-7
            query[2] = 0xC1; // QUERY GROUP 8-15
            query[3] = 0xB0; // QUERY scene level 0
            query[4] = 0xB1; // QUERY scene level 1
            query[5] = 0xB2; // QUERY scene level 2
            query[6] = 0xB3; // QUERY scene level 3
            query[7] = 0xB4; // QUERY scene level 4
            query[8] = 0xB5; // QUERY scene level 5
            query[9] = 0xB6; // QUERY scene level 6
            query[10] = 0xB7; // QUERY scene level 7
            query[11] = 0xB8; // QUERY scene level 8
            query[12] = 0xB9; // QUERY scene level 9
            query[13] = 0xBA; // QUERY scene level 10
            query[14] = 0xBB; // QUERY scene level 11
            query[15] = 0xBC; // QUERY scene level 12
            query[16] = 0xBD; // QUERY scene level 13
            query[17] = 0xBE; // QUERY scene level 14
            query[18] = 0xBF; // QUERY scene level 15

            query[19] = 0xA5; // QUERY fade time/fade rate
            query[20] = 0xA3; // QUERY power on level
            query[21] = 0xA4; // QUERY system failure level

            query[22] = 0xA2; // QUERY min level
            query[23] = 0xA1; // QUERY max level

            query[24] = 0xA0; // QUERY actual level


            value[1, 1] = 0x00;
            value[2, 1] = 0x00;
            value[3, 1] = 255;
            value[4, 1] = 255;
            value[5, 1] = 255;
            value[6, 1] = 255;
            value[7, 1] = 255;
            value[8, 1] = 255;
            value[9, 1] = 255;
            value[10, 1] = 255;
            value[11, 1] = 255;
            value[12, 1] = 255;
            value[13, 1] = 255;
            value[14, 1] = 255;
            value[15, 1] = 255;
            value[16, 1] = 255;
            value[17, 1] = 255;
            value[18, 1] = 255;
            value[19, 1] = 0x07;
            value[20, 1] = 254;
            value[21, 1] = 254;
            //value[22, 1] = PHM; // PHM unknown yet
            value[23, 1] = 254;
            value[24, 1] = 254;

            value[1, 3] = value[1, 2] = 0xFF;
            value[2, 3] = value[2, 2] = 0xFF;
            value[3, 3] = value[3, 2] = 10;
            value[4, 3] = value[4, 2] = 10;
            value[5, 3] = value[5, 2] = 10;
            value[6, 3] = value[6, 2] = 10;
            value[7, 3] = value[7, 2] = 10;
            value[8, 3] = value[8, 2] = 10;
            value[9, 3] = value[9, 2] = 10;
            value[10, 3] = value[10, 2] = 10;
            value[11, 3] = value[11, 2] = 10;
            value[12, 3] = value[12, 2] = 10;
            value[13, 3] = value[13, 2] = 10;
            value[14, 3] = value[14, 2] = 10;
            value[15, 3] = value[15, 2] = 10;
            value[16, 3] = value[16, 2] = 10;
            value[17, 3] = value[17, 2] = 10;
            value[18, 3] = value[18, 2] = 10;
            value[19, 3] = value[19, 2] = 0xAA;
            value[20, 3] = value[20, 2] = 10;
            value[21, 3] = value[21, 2] = 10;
            //value[22, 3] = value[22, 2] = (Byte)(PHM + 1);
            //value[23, 3] = value[23, 2] = (Byte)(PHM + 1);
            //value[24, 3] = value[24, 2] = (Byte)(PHM + 1);

            errorText[1] = "GROUP 0-7";
            errorText[2] = "GROUP 8-15";
            errorText[3] = "SCENE 0";
            errorText[4] = "SCENE 1";
            errorText[5] = "SCENE 2";
            errorText[6] = "SCENE 3";
            errorText[7] = "SCENE 4";
            errorText[8] = "SCENE 5";
            errorText[9] = "SCENE 6";
            errorText[10] = "SCENE 7";
            errorText[11] = "SCENE 8";
            errorText[12] = "SCENE 9";
            errorText[13] = "SCENE 10";
            errorText[14] = "SCENE 11";
            errorText[15] = "SCENE 12";
            errorText[16] = "SCENE 13";
            errorText[17] = "SCENE 14";
            errorText[18] = "SCENE 15";
            errorText[19] = "FADE TIME / FADE RATE";
            errorText[20] = "POWER ON LEVEL";
            errorText[21] = "SYSTEM FAILURE LEVEL";
            errorText[22] = "MIN LEVEL";
            errorText[23] = "MAX LEVEL";
            errorText[24] = "ACTUAL LEVEL";


            address[1] = 0xFF;
            address[2] = 0x0B; // Short address 5 0AAA AAA1 == 0000 1011
            address[3] = 0x9F; // Group address 15 100A AAA1 == 1001 1111

            Message("Starting test 12.2.1.4: Commands in-between");

            // loop 1
            for (a = 1; a <= aMax; a++)
            {

                // 2x BROADCAST RESET
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x20; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x20; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }


                Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                Thread.Sleep((int)preaheatTimeControl.Value + 300);

                // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x9A;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        PHM = incoming[3];
                    }
                }

                Debug("PHM = " + PHM.ToString());

                value[22, 1] = PHM;
                value[22, 3] = value[22, 2] = (Byte)(PHM + 1);
                value[23, 3] = value[23, 2] = (Byte)(PHM + 1);
                value[24, 3] = value[24, 2] = (Byte)(PHM + 1);


                if (PHM == 254)
                {
                    i = 3;
                    iMax = 38;
                    k = 1;
                    kMax = 21;
                }
                else
                {
                    i = 1;
                    iMax = 38;
                    k = 1;
                    kMax = 24;
                }

                level[1] = (Byte)(PHM + 1);
                level[2] = (Byte)(PHM + 1);

                for (; i <= iMax; i++)
                {
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x04; // 4 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xA3; // Command 0 addr
                    outgoing[5] = level[i]; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = command[i]; // Command 1 data
                    outgoing[9] = 0x00; // Command 2 flags
                    outgoing[10] = address[a]; // Command 2 addr
                    outgoing[11] = 0x04; // Command 2 data
                    outgoing[12] = 0x00; // Command 3 flags
                    outgoing[13] = 0xFF; // Command 3 addr
                    outgoing[14] = command[i]; // Command 3 data
                    sendData(outgoing);
                    r =getData(incoming, 500); // more time allowed for the sequence to finish

                    /* See IEC 62386-102, 11.2.1 General configuration commands, page 30
                     * 
                     * Added 150ms delay to overcome this.
                    */

                    // change ref 3
                    Info("Wait 150ms");
                    Thread.Sleep(150);

                } // end loop 1



                // loop 2
                for (; k <= kMax; k++)
                {
                    // BROADCAST query k
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = query[k];
                    sendData(outgoing);
                    r =getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if (incoming[3] != value[k, a])
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + ", expected = " + value[k, a].ToString());
                                Error("ERROR (2141," + k.ToString() + "," + a.ToString() + ") \"" + errorText[k] + " is wrong\"");
                            }
                        }
                    }
                } // loop 2

            }
            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.2.1.5: VERSION NUMBER");


            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // BROADCAST QUERY VERSION NUMBER
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x97;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    Message("Version number = " + incoming[3].ToString() + "");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            Byte PHM = 0;
            int r;

            Message("Starting test 12.2.1.6: STORE ACTUAL LEVEL IN THE DTR");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());


            // BROADCAST DTR(255)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 0xFF;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)

                // 2x BROADCAST ST. ACT LEV. DTR
                outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x21; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x21; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST QUERY CONTENT DTR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x98;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if (incoming[3] != 254)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 254");
                        Error("Error 2161 \"MAX LEVEL not stored in the DTR\"");
                    }
                }
            }

            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r =getData(incoming, 500);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // 2x BROADCAST ST. ACT LEV. DTR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x21; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x21; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST QUERY CONTENT DTR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x98;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if (incoming[3] != 0)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 0");
                        Error("Error 2162 \"0 (OFF) not stored in the DTR\"");
                    }
                }
            }

            // BROADCAST RECALL MIN LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x06;
            sendData(outgoing);
            r =getData(incoming, 500);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            // 2x BROADCAST ST. ACT LEV. DTR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x21; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x21; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST QUERY CONTENT DTR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x98;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if (incoming[3] != PHM)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + PHM.ToString());
                        Error("Error 2163 \"MIN LEVEL not stored in the DTR\"");
                    }
                }
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_7()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] command = new Byte[40];

            Byte[] level = new Byte[40];

            Byte[] query = new Byte[24];

            Byte[] value = new Byte[24];
            String[] errorText = new String[24];

            int r, i, iMax, k, kMax;
            Byte PHM = 0;

            iMax = 38;
            kMax = 23;


            command[1] = 0x60; // add to group 0
            command[2] = 0x61; // add to group 1
            command[3] = 0x62; // add to group 2
            command[4] = 0x63; // add to group 3
            command[5] = 0x64; // add to group 4
            command[6] = 0x65; // add to group 5
            command[7] = 0x66; // add to group 6
            command[8] = 0x67; // add to group 7
            command[9] = 0x68; // add to group 8
            command[10] = 0x69; // add to group 9
            command[11] = 0x6A; // add to group 10
            command[12] = 0x6B; // add to group 11
            command[13] = 0x6C; // add to group 12
            command[14] = 0x6D; // add to group 13
            command[15] = 0x6E; // add to group 14
            command[16] = 0x6F; // add to group 15

            command[17] = 0x40; // store DTR as scene 0
            command[18] = 0x41; // store DTR as scene 1
            command[19] = 0x42; // store DTR as scene 2
            command[20] = 0x43; // store DTR as scene 3
            command[21] = 0x44; // store DTR as scene 4
            command[22] = 0x45; // store DTR as scene 5
            command[23] = 0x46; // store DTR as scene 6
            command[24] = 0x47; // store DTR as scene 7
            command[25] = 0x48; // store DTR as scene 8
            command[26] = 0x49; // store DTR as scene 9
            command[27] = 0x4A; // store DTR as scene 10
            command[28] = 0x4B; // store DTR as scene 11
            command[29] = 0x4C; // store DTR as scene 12
            command[30] = 0x4D; // store DTR as scene 13
            command[31] = 0x4E; // store DTR as scene 14
            command[32] = 0x4F; // store DTR as scene 15

            command[33] = 0x2F; // store DTR as fade rate
            command[34] = 0x80; // store DTR as short address
            command[35] = 0x2C; // store DTR as system failure level
            command[36] = 0x2D; // store DTR as power on level
            command[37] = 0x2E; // store DTR as fade time
            command[38] = 0x2A; // store DTR as max level
            command[39] = 0x2B; // store DTR as min level


            level[1] = 10;
            level[2] = 10;
            level[3] = 10;
            level[4] = 10;
            level[5] = 10;
            level[6] = 10;
            level[7] = 10;
            level[8] = 10;
            level[9] = 10;
            level[10] = 10;
            level[11] = 10;
            level[12] = 10;
            level[13] = 10;
            level[14] = 10;
            level[15] = 10;
            level[16] = 10;
            level[17] = 10;
            level[18] = 10;
            level[19] = 10;
            level[20] = 10;
            level[21] = 10;
            level[22] = 10;
            level[23] = 10;
            level[24] = 10;
            level[25] = 10;
            level[26] = 10;
            level[27] = 10;
            level[28] = 10;
            level[29] = 10;
            level[30] = 10;
            level[31] = 10;
            level[32] = 10;
            level[33] = 10;
            level[34] = 11;
            level[35] = 10;
            level[36] = 10;
            level[37] = 10;
            level[38] = 0; // PHM + 1
            level[39] = 0; // PHM + 1


            query[1] = 0xC0; // QUERY GROUP 0-7
            query[2] = 0xC1; // QUERY GROUP 8-15
            query[3] = 0xB0; // QUERY scene level 0
            query[4] = 0xB1; // QUERY scene level 1
            query[5] = 0xB2; // QUERY scene level 2
            query[6] = 0xB3; // QUERY scene level 3
            query[7] = 0xB4; // QUERY scene level 4
            query[8] = 0xB5; // QUERY scene level 5
            query[9] = 0xB6; // QUERY scene level 6
            query[10] = 0xB7; // QUERY scene level 7
            query[11] = 0xB8; // QUERY scene level 8
            query[12] = 0xB9; // QUERY scene level 9
            query[13] = 0xBA; // QUERY scene level 10
            query[14] = 0xBB; // QUERY scene level 11
            query[15] = 0xBC; // QUERY scene level 12
            query[16] = 0xBD; // QUERY scene level 13
            query[17] = 0xBE; // QUERY scene level 14
            query[18] = 0xBF; // QUERY scene level 15

            query[19] = 0xA3; // QUERY power on level
            query[20] = 0xA4; // QUERY system failure level
            query[21] = 0xA5; // QUERY fade time/fade rate
            query[22] = 0xA1; // QUERY max level
            query[23] = 0xA2; // QUERY min level

            value[1] = 0xFF;
            value[2] = 0xFF;
            value[3] = 10;
            value[4] = 10;
            value[5] = 10;
            value[6] = 10;
            value[7] = 10;
            value[8] = 10;
            value[9] = 10;
            value[10] = 10;
            value[11] = 10;
            value[12] = 10;
            value[13] = 10;
            value[14] = 10;
            value[15] = 10;
            value[16] = 10;
            value[17] = 10;
            value[18] = 10;
            value[19] = 10;
            value[20] = 10;
            value[21] = 0xAA;
            value[22] = 0;
            value[23] = 0;


            errorText[1] = "GROUP O-7";
            errorText[2] = "GROUP 8-15";
            errorText[3] = "SCENE 0";
            errorText[4] = "SCENE 1";
            errorText[5] = "SCENE 2";
            errorText[6] = "SCENE 3";
            errorText[7] = "SCENE 4";
            errorText[8] = "SCENE 5";
            errorText[9] = "SCENE 6";
            errorText[10] = "SCENE 7";
            errorText[11] = "SCENE 8";
            errorText[12] = "SCENE 9";
            errorText[13] = "SCENE 10";
            errorText[14] = "SCENE 11";
            errorText[15] = "SCENE 12";
            errorText[16] = "SCENE 13";
            errorText[17] = "SCENE 14";
            errorText[18] = "SCENE 15";
            errorText[19] = "POWER ON LEVEL";
            errorText[20] = "SYSTEM FAILURE LEVEL";
            errorText[21] = "FADE TIME / FADE RATE";
            errorText[22] = "MAX LEVEL";
            errorText[23] = "MIN LEVEL";


            Message("Starting test 12.2.1.7: PERSISTENT MEMORY");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());


            if (PHM == 254)
            {
                i = 1;
                iMax = 37;
                k = 1;
                kMax = 21;
            }
            else
            {
                i = 1;
                iMax = 39;
                k = 1;
                kMax = 23;
            }

            level[38] = (Byte)(PHM + 1); // PHM + 1
            level[39] = (Byte)(PHM + 1); // PHM + 1

            value[22] = (Byte)(PHM + 1);
            value[23] = (Byte)(PHM + 1);

            // loop 1
            for (; i <= iMax; i++)
            {
                // BROADCAST DTR(level i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = level[i];
                sendData(outgoing);
                r =getData(incoming, 100);


                // 2x BROADCAST command i
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = command[i]; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = command[i]; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
            }

            MessageBox.Show("Switch off mains power", "Required action");
            Message("Switched off mains power");

            Message("Wait 5s");
            Thread.Sleep(5000);

            MessageBox.Show("Switch on mains power", "Required action");
            Message("Switched on mains power");

            // flush input buffer
            while (getData(incoming, 100, 0) != 0)
                ;

            Info("Wait Preheat Time + Response Time + 600ms = " + (preaheatTimeControl.Value + preaheatTimeControl.Value + 600).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value + 600);

            // loop 2
            for (; k <= kMax; k++)
            {
                // BROADCAST query k
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = query[k];
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != value[k])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + value[k].ToString());
                            // change ref 4
                            // original code: Error("ERROR (2171," + k.ToString() + ",a) \"" + errorText[k] + " not stored in persistent memory\"");
                            Error("ERROR (2171," + k.ToString() + ") \"" + errorText[k] + " not stored in persistent memory\"");
                        }
                    }
                }
            }

            // SHORT ADDRESS 5 QUERY CONTROL GEAR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0x0B; // Short address 5 command
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("Error 2172 \"Short address not stored in persistent memory\"");
                }
            }

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // SHORT ADDRESS 5 QUERY CONTROL GEAR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0x0B; // Short address 5 command
            outgoing[3] = 0x91;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("Error 2173 \"RESET removes Short Address\"");
                }
            }

            // BROADCAST DTR(255)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 0xFF;
            sendData(outgoing);
            r =getData(incoming, 100);


            // 2x BROADCAST STORE DTR AS SHORT ADDRESS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_8()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] level = new Byte[40];

            int r, i, iMax;
            Byte PHM = 0;

            iMax = 6;

            level[0] = 0;
            level[1] = 1;

            level[4] = 254;
            level[5] = 255;
            level[6] = 0;


            Message("Starting test 12.2.1.8: DTR1");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());

            level[2] = PHM;
            level[3] = (Byte)((PHM + 254) / 2);

            // loop
            for (i = 0; i <= iMax; i++)
            {
                // BROADCAST DTR1(level i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xC3;
                outgoing[3] = level[i];
                sendData(outgoing);
               getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY CONTENT DTR1
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x9C;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Error("Error (2181," + i.ToString() + ") \"Content of DTR1 not correct!\"");
                        }
                    }
                }

            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_1_9()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] level = new Byte[40];

            int r, i, iMax;
            Byte PHM = 0;

            iMax = 6;

            level[0] = 0;
            level[1] = 1;

            level[4] = 254;
            level[5] = 255;
            level[6] = 0;


            Message("Starting test 12.2.1.9: DTR2");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());

            level[2] = PHM;
            level[3] = (Byte)((PHM + 254) / 2);

            // loop
            for (i = 0; i <= iMax; i++)
            {
                // BROADCAST DTR2(level i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xC5;
                outgoing[3] = level[i];
                sendData(outgoing);
               getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY CONTENT DTR2
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x9D;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Error("Error (2191," + i.ToString() + ") \"Content of DTR2 not correct!\"");
                        }
                    }
                }

            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_2_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] value = new Byte[3];
            Byte[] max = new Byte[3];
            Byte[] level = new Byte[3];

            int r, i, iMax;
            Byte PHM = 0;

            iMax = 2;

            Message("Starting test 12.2.2.1: STORE THE DTR AS MAX LEVEL");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());


            value[0] = 255;
            value[1] = 0;
            value[2] = 253;

            max[0] = 254;
            max[1] = (Byte)(PHM + 1);
            max[2] = 253;

            level[0] = 254;
            level[1] = (Byte)(PHM + 1);
            level[2] = (Byte)(PHM + 1);

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            // BROADCAST DTR(PHM + 1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = (Byte)(PHM + 1);
            sendData(outgoing);
           getData(incoming, 100);


            // 2x BROADCAST STORE DTR AS MIN LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2B; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2B; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            // BROADCAST QUERY RESET STATE
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x95;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Error("Error 2211 \"Wrong answer at QUERY RESET STATE and changed MIN LEVEL\"");
                }
            }

            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x20) == 0x20) // check if status == xx1xxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxx");
                        Error("Error 2212 \"Wrong answer at QUERY STATUS and changed MIN LEVEL\"");
                    }
                }
            }


            // loop
            for (i = 0; i <= iMax; i++)
            {

                // BROADCAST DTR(value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
               getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2A; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2A; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST QUERY MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA1;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != max[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + max[i].ToString());
                            Error("Error (2213," + i.ToString() + ") \"Wrong MAX LEVEL stored\"");
                        }
                    }
                }

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[i].ToString());
                            Error("Error (2214," + i.ToString() + ") \"Wrong ACTUAL LEVEL\"");
                        }
                    }
                }

            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_2_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] value = new Byte[3];
            Byte[] min = new Byte[3];
            Byte[] level = new Byte[3];

            int r, i, iMax;
            Byte PHM = 0;

            iMax = 2;

            Message("Starting test 12.2.2.2: STORE THE DTR AS MIN LEVEL");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());


            value[0] = (Byte)(PHM + 1);
            value[1] = 254;
            value[2] = 0;

            min[0] = (Byte)(PHM + 1);
            min[1] = 253;
            min[2] = PHM;

            // change ref 18
            // original code: level[0] = (Byte)(PHM + 1);
            level[0] = 253;
            level[1] = 253;
            level[2] = 253;

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            // BROADCAST DTR(253)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 253;
            sendData(outgoing);
           getData(incoming, 100);


            // 2x BROADCAST STORE DTR AS MAX LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2A; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2A; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            // BROADCAST QUERY RESET STATE
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x95;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Error("Error 2221 \"Wrong answer at QUERY RESET STATE and changed MAX LEVEL\"");
                }
            }

            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x20) == 0x20) // check if status == xx1xxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxx");
                        Error("Error 2222 \"Wrong answer at QUERY STATUS and changed MAX LEVEL\"");
                    }
                }
            }


            // loop
            for (i = 0; i <= iMax; i++)
            {

                // BROADCAST DTR(value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
               getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2B; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2B; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST QUERY MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA2;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        // change ref 19
                        if (incoming[3] != min[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + min[i].ToString());
                            Error("Error (2223," + i.ToString() + ") \"Wrong MIN LEVEL stored\"");
                        }
                    }
                }

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[i].ToString());
                            Error("Error (2224," + i.ToString() + ") \"Wrong ACTUAL LEVEL\"");
                        }
                    }
                }

            }

            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_2_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] value = new Byte[5];
            Byte[] sys = new Byte[5];
            Byte[] level = new Byte[5];

            int r, i, iMax;
            Byte PHM = 0;

            iMax = 4;

            Message("Starting test 12.2.2.3: STORE THE DTR AS SYSTEM FAILURE LEVEL");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());


            value[0] = 252;
            value[1] = 255;
            value[2] = 254;
            value[3] = 0;
            value[4] = 1;

            sys[0] = 252;
            sys[1] = 255;
            sys[2] = 254;
            sys[3] = 0;
            sys[4] = 1;

            if (PHM == 254)
            {
                level[0] = 254;
                level[1] = 254;
                level[2] = 254;
                level[3] = 0;
                level[4] = 254;
            }
            else
            {
                level[0] = 252;
                level[1] = 252;
                level[2] = 253;
                level[3] = 0;
                level[4] = (Byte)(PHM + 1);
            }

            // BROADCAST DTR(253)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 253;
            sendData(outgoing);
           getData(incoming, 100);


            // 2x BROADCAST STORE DTR AS MAX LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2A; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2A; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST DTR(PHM + 1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = (Byte)(PHM + 1);
            sendData(outgoing);
           getData(incoming, 100);


            // 2x BROADCAST STORE DTR AS MIN LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2B; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2B; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }



            // loop
            for (i = 0; i <= iMax; i++)
            {

                // BROADCAST DTR(value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
               getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS SYSTEM FAILURE LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2C; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2C; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST QUERY SYSTEM FAILURE LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA4;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != sys[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + sys[i].ToString());
                            Error("Error (2231," + i.ToString() + ") \"Wrong SYSTEM FAILURE LEVEL stored\"");
                        }
                    }
                }

                MessageBox.Show("Disconnect Interface", "Required action");

                Message("Interface disconnected");
                Message("Wait 1s");
                Thread.Sleep(1000);

                MessageBox.Show("Reconnect Interface", "Required action");
                Message("Interface reconnected");

                // flush input buffer
                while (getData(incoming, 100, 0) != 0)
                    ;

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[i].ToString());
                            Error("Error (2232," + i.ToString() + ") \"Wrong ACTUAL LEVEL\"");
                        }
                    }
                }

                Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                Thread.Sleep((int)preaheatTimeControl.Value + 300);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[i].ToString());
                            Error("Error (2233," + i.ToString() + ") \"Wrong ACTUAL LEVEL\"");
                        }
                    }
                }

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_2_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] value = new Byte[5];
            Byte[] power = new Byte[5];
            Byte[] level = new Byte[5];

            int r, i, iMax;
            Byte PHM = 0;

            iMax = 2;

            Message("Starting test 12.2.2.4: STORE THE DTR AS POWER ON LEVEL");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (responseTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)responseTimeControl.Value + 300);

            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());


            value[0] = 0;
            value[1] = (Byte)((PHM + 254) / 2);
            value[2] = 255;

            power[0] = 0;
            power[1] = (Byte)((PHM + 254) / 2);
            power[2] = 255;

            level[0] = 0;
            level[1] = (Byte)((PHM + 254) / 2);
            level[2] = PHM;


            // loop
            for (i = 0; i <= iMax; i++)
            {

                // BROADCAST DTR(value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
               getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS POWER ON LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2D; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2D; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
               getData(incoming, 100);


                // BROADCAST QUERY POWER ON LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA3;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != power[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + power[i].ToString());
                            Error("Error (2241," + i.ToString() + ") \"Wrong POWER ON LEVEL stored\"");
                        }
                    }
                }

                MessageBox.Show("Disconnect all power sources", "Required action");
                Message("Disconnected all power sources");

                Message("Wait 5s");
                Thread.Sleep(5000);

                MessageBox.Show("Reconnect all power sources", "Required action");
                Message("Reconnected all power sources");

                // flush input buffer
                while (getData(incoming, 100, 0) != 0)
                    ;


                Info("Wait Preheat Time + Response Time + 600ms = " + (preaheatTimeControl.Value + responseTimeControl.Value + 600).ToString() + "ms");
                Thread.Sleep((int)preaheatTimeControl.Value + (int)responseTimeControl.Value + 600);


                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[i].ToString());
                            Error("Error (2242," + i.ToString() + ") \"Wrong ACTUAL LEVEL after power on\"");
                        }
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_2_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] value = new Byte[4];
            Byte[] time_rate = new Byte[4];

            int r, i, iMax;

            iMax = 3;

            Message("Starting test 12.2.2.5: STORE THE DTR AS FADE TIME");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (responseTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)responseTimeControl.Value + 300);


            value[0] = 15;
            value[1] = 0;
            value[2] = 5;
            value[3] = 128;

            time_rate[0] = 0xF7;
            time_rate[1] = 0x07;
            time_rate[2] = 0x57;
            time_rate[3] = 0xF7;


            // loop
            for (i = 0; i <= iMax; i++)
            {

                // BROADCAST DTR(value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
               getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS FADE TIME
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2E; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2E; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }


                // BROADCAST QUERY FADE TIME/FADE RATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA5;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != time_rate[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + time_rate[i].ToString());
                            Error("Error (2251," + i.ToString() + ") \"Wrong FADE TIME stored\"");
                        }
                    }
                }

            }

            // BROADCAST QUERY RESET STATE
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x95;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Error("Error 2252 \"Wrong answer at QUERY RESET STATE and changed FADE TIME\"");
                }
            }

            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x20) == 0x20) // check if status == xx1xxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxx");
                        Error("Error 2253 \"Wrong answer at QUERY STATUS and changed FADE TIME\"");
                    }
                }
            }
            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_2_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] value = new Byte[5];
            Byte[] time_rate = new Byte[5];

            int r, i, iMax;

            iMax = 4;

            Message("Starting test 12.2.2.6: STORE THE DTR AS FADE RATE");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            value[0] = 15;
            value[1] = 0;
            value[2] = 5;
            value[3] = 128;
            value[4] = 1;

            time_rate[0] = 0x0F;
            time_rate[1] = 0x01;
            time_rate[2] = 0x05;
            time_rate[3] = 0x0F;
            time_rate[4] = 0x01;


            // loop
            for (i = 0; i <= iMax; i++)
            {

                // BROADCAST DTR(value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS FADE RATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2F; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2F; // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }


                // BROADCAST QUERY FADE TIME/FADE RATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA5;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != time_rate[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + time_rate[i].ToString());
                            Error("Error (2261," + i.ToString() + ") \"Wrong FADE RATE stored\"");
                        }
                    }
                }

            }

            // BROADCAST QUERY RESET STATE
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x95;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Error("Error 2262 \"Wrong answer at QUERY RESET STATE and changed FADE RATE\"");
                }
            }

            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x20) == 0x20) // check if status == xx1xxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxx");
                        Error("Error 2263 \"Wrong answer at QUERY STATUS and changed FADE RATE\"");
                    }
                }
            }
            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_2_7()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            Byte[] value = new Byte[5];
            Byte[] scene = new Byte[5];
            Byte[] level = new Byte[5];

            Byte PHM = 0;

            int r, i, k, iMax, kMax;

            iMax = 15;
            kMax = 4;

            Message("Starting test 12.2.2.7: STORE THE DTR AS SCENE / GO TO SCENE");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r =getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());

            // BROADCAST DTR(253)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 253;
            sendData(outgoing);
           getData(incoming, 100);

            // 2x BROADCAST STORE DTR AS MAX LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2A; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2A; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST DTR(PHM + 1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = (Byte)(PHM + 1);
            sendData(outgoing);
           getData(incoming, 100);

            // 2x BROADCAST STORE DTR AS MIN LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2B; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2B; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            value[0] = 1;
            value[1] = 0;
            value[2] = 255;
            value[3] = 252;
            value[4] = 254;

            scene[0] = 1;
            scene[1] = 0;
            scene[2] = 255;
            scene[3] = 252;
            scene[4] = 254;

            if (PHM == 254)
            {
                level[0] = 254;
                level[1] = 0;
                level[2] = 0;
                level[3] = 254;
                level[4] = 254;
            }
            else
            {
                level[0] = (Byte)(PHM + 1);
                level[1] = 0;
                level[2] = 0;
                level[3] = 252;
                level[4] = 253;
            }

            // loop 1
            for (i = 0; i <= iMax; i++)
            {
                // loop 2
                for (k = 0; k <= kMax; k++)
                {
                    // BROADCAST DTR(value k)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA3;
                    outgoing[3] = value[k];
                    sendData(outgoing);
                   getData(incoming, 100);

                    // 2x BROADCAST STORE DTR AS SCENE(i)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xFF; // Command 0 addr
                    outgoing[5] = (Byte)(0x40 + i); // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = (Byte)(0x40 + i); // Command 1 data
                    sendData(outgoing);
                    r =getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    // BROADCAST QUERY SCENE LEVEL i
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = (Byte)(0xB0 + i);
                    sendData(outgoing);
                    r =getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if (incoming[3] != scene[k])
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + scene[k].ToString());
                                Error("Error (2271," + i.ToString() + "," + k.ToString() + ") \"Wrong SCENE LEVEL stored\"");
                            }
                        }
                    }

                    // BROADCAST GO TO SCENE i
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = (Byte)(0x10 + i);
                    sendData(outgoing);
                   getData(incoming, 100);

                    Info("Wait Preheat Time + Response Time");
                    Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

                    // BROADCAST QUERY ACTUAL LEVEL
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0xA0;
                    sendData(outgoing);
                    r =getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if (incoming[3] != level[k])
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[k].ToString());
                                Error("Error (2272," + i.ToString() + "," + k.ToString() + ") \"Wrong ACTUAL LEVEL\"");
                            }
                        }
                    }
                }
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_3_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];


            int r, i, iMax;

            iMax = 15;

            Message("Starting test 12.2.3.1: REMOVE FROM SCENE");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // BROADCAST DTR(127)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 127;
            sendData(outgoing);
           getData(incoming, 100);


            for (i = 0; i <= iMax; i++)
            {

                // 2x BROADCAST STORE DTR AS SCENE i
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = (Byte)(0x40 + i); // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = (Byte)(0x40 + i); // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST QUERY RESET STATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x95;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                    {
                        Error("Error (2311," + i.ToString() + ") \"Wrong answer at QUERY RESET STATE and programmed SCENE\"");
                    }
                }

                // BROADCAST QUERY STATUS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if ((incoming[3] & 0x20) == 0x20) // check if status == xx1xxxxx
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxx");
                            Error("Error (2312," + i.ToString() + ") \"Wrong answer at QUERY STATUS and programmed SCENE\"");
                        }
                    }
                }

                // 2x BROADCAST REMOVE FROM SCENE i
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = (Byte)(0x50 + i); // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = (Byte)(0x50 + i); // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST QUERY RESET STATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x95;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 1, 255");
                        Error("Error (2313," + i.ToString() + ") \"Wrong answer at QUERY RESET STATE and removed SCENE\"");
                    }
                }

                // BROADCAST QUERY STATUS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r =getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if ((incoming[3] & 0x20) != 0x20) // check if status != xx1xxxxx
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx1xxxxx");
                            Error("Error (2314," + i.ToString() + ") \"Wrong answer at QUERY STATUS and removed SCENE\"");
                        }
                    }
                }


            }


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_3_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] group = new Byte[16];

            int r, i, iMax;

            iMax = 15;

            group[0] = 1;
            group[1] = 2;
            group[2] = 4;
            group[3] = 8;
            group[4] = 16;
            group[5] = 32;
            group[6] = 64;
            group[7] = 128;
            group[8] = 1;
            group[9] = 2;
            group[10] = 4;
            group[11] = 8;
            group[12] = 16;
            group[13] = 32;
            group[14] = 64;
            group[15] = 128;

            Message("Starting test 12.2.3.2: ADD TO GROUP / REMOVE FROM GROUP");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r =getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            for (i = 0; i <= iMax; i++)
            {
                // 2x BROADCAST ADD TO GROUP i
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = (Byte)(0x60 + i); // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = (Byte)(0x60 + i); // Command 1 data
                sendData(outgoing);
                r =getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST QUERY RESET STATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x95;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 0, 0");
                        Error("Error (2321," + i.ToString() + ") \"Wrong answer at QUERY RESET STATE and added to group\"");
                    }
                }

                // GROUP i QUERY STATUS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = (Byte)((i << 1) + 0x81);
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if ((incoming[3] & 0x20) == 0x20) // check if status == xx1xxxxx
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxx");
                            Error("Error (2322," + i.ToString() + ") \"Wrong answer at QUERY STATUS and added to group\"");
                        }
                    }
                }

                if (i > 7)
                {
                    // GROUP i QUERY GROUPS 8-15
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = (Byte)((i << 1) + 0x81);
                    outgoing[3] = 0xC1;
                    sendData(outgoing);
                }
                else
                {
                    // GROUP i QUERY GROUPS 0-7
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = (Byte)((i << 1) + 0x81);
                    outgoing[3] = 0xC0;
                    sendData(outgoing);
                }
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if (incoming[3] != group[i])
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + group[i].ToString());
                        Error("Error (2323," + i.ToString() + ") \"Wrong answering of QUERY GROUP\"");
                    }
                }

                // 2x GROUP i REMOVE FROM GROUP i
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = (Byte)((i << 1) + 0x81); // Command 0 addr
                outgoing[5] = (Byte)(0x70 + i); // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = (Byte)((i << 1) + 0x81); // Command 1 addr
                outgoing[8] = (Byte)(0x70 + i); // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST QUERY RESET STATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x95;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expexted = 1, 255");
                        Error("Error (2324," + i.ToString() + ") \"Wrong answer at QUERY RESET STATE and removed from group\"");
                    }
                }

                // BROADCAST QUERY STATUS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if ((incoming[3] & 0x20) != 0x20) // check if status != xx1xxxxx
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx1xxxxx");
                            Error("Error (2325," + i.ToString() + ") \"Wrong answer at QUERY STATUS and removed from group\"");
                        }
                    }
                }

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_3_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] value = new Byte[7];
            Byte[] address1 = new Byte[7];
            Byte[] address2 = new Byte[7];
            bool[] test1 = new bool[7];
            Byte[] test2_bit6mask = new Byte[7];

            bool answer;

            int r, i, iMax;

            iMax = 6;

            value[0] = 3;
            value[1] = 127;
            value[2] = 31;
            value[3] = 129;
            value[4] = 30;
            value[5] = 1;
            value[6] = 255;

            // these values include the S==1 bit, i.e. 0AAA AAA1
            address1[0] = 0xFF;
            address1[1] = 0x03;
            address1[2] = 0x7F;
            address1[3] = 0x1F;
            address1[4] = 0x1F;
            address1[5] = 0x1F;
            address1[6] = 0x01;

            address2[0] = 0x03;
            address2[1] = 0x7F;
            address2[2] = 0x1F;
            address2[3] = 0x1F;
            address2[4] = 0x1F;
            address2[5] = 0x01;
            address2[6] = 0xFF;

            test1[0] = false;
            test1[1] = false;
            test1[2] = false;
            test1[3] = false;
            test1[4] = false;
            test1[5] = false;
            test1[6] = true;

            test2_bit6mask[0] = 0x00;
            test2_bit6mask[1] = 0x00;
            test2_bit6mask[2] = 0x00;
            test2_bit6mask[3] = 0x00;
            test2_bit6mask[4] = 0x00;
            test2_bit6mask[5] = 0x00;
            test2_bit6mask[6] = 0x40;


            Message("Starting test 12.2.3.3: STORE THE DTR AS SHORT ADDRESS");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r <= 0)
            {
                Error("USB Communication error");
            }
            else
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            for (i = 0; i <= iMax; i++)
            {
                // BROADCAST DTR(value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
                r = getData(incoming, 100);

                // address i STORE DTR AS SHORT ADDRESS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = address1[i]; // Command 0 addr
                outgoing[5] = 0x80; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = address1[i]; // Command 1 addr
                outgoing[8] = 0x80; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                answer = false;
                // BROADCAST QUERY MISSING SHORT ADDRESS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x96;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x01)
                    {
                        answer = true;
                    }
                    else
                    {
                        answer = false;
                    }
                }

                if (answer != test1[i])
                {
                    Error("Error (2331," + i.ToString() + ") \"Wrong answer at QUERY MISSING SHORT ADDRESS\"");
                }

                // address i QUERY STATUS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = address2[i];
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if ((incoming[3] & 0x40) != test2_bit6mask[i]) // check if status == xTxxxxxx
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + test2_bit6mask[i].ToString() + " bit 6 mask");
                            Error("Error (2332," + i.ToString() + ") \"Wrong answer at QUERY STATUS\"");
                        }
                    }
                }
            }

            // BROADCAST DTR(3)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);


            // BROADCAST STORE DTR AS SHORT ADDRESS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x80;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait 150ms");
            Thread.Sleep(150);

            // BROADCAST STORE DTR AS SHORT ADDRESS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x80;
            sendData(outgoing);
            r = getData(incoming, 100);

            // change ref 5
            Info("Wait 150ms");
            Thread.Sleep(150);

            // SHORT ADDRESS 1 QUERY CONTROL GEAR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0x03; // Short address 1 command
            outgoing[3] = 0x91;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x01)
                {
                    Error("Error (2333) \"Command with timeout > 100ms executed\"");
                }
            }

            // BROADCAST DTR(5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 5;
            sendData(outgoing);
            r = getData(incoming, 100);

            // BROADCAST STORE DTR AS SHORT ADDRESS, BROADCAST QUERY CONTROL GEAR, BROADCAST STORE DTR AS SHORT ADDRESS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 2 flags
            outgoing[10] = 0xFF; // Command 2 addr
            outgoing[11] = 0x80; // Command 2 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
                else if (incoming[6] != 0x00)
                {
                    Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 0");
                    Error("ERROR 2334 \"Command in-between executed\"");
                }

            }

            // change ref 6
            Info("Wait 150ms");
            Thread.Sleep(150);


            // SHORT ADDRESS 2 QUERY CONTROL GEAR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0x05; // Short address 2 command
            outgoing[3] = 0x91;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x01)
                {
                    Error("Error 2335 \"Command executed with command in-between\"");
                }
            }

            // BROADCAST DTR(7)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 7;
            sendData(outgoing);
            r = getData(incoming, 100);



            // BROADCAST STORE DTR AS SHORT ADDRESS, SHORT ADDRESS 9 QUERY CONTROL GEAR BROADCAST STORE DTR AS SHORT ADDRESS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0x13; // Command 1 addr -- short address 9
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 2 flags
            outgoing[10] = 0xFF; // Command 2 addr
            outgoing[11] = 0x80; // Command 2 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            /* IF the command in between addressed to other DUT is ignored (which should be the case)
             * then the two STORE DTR AS SHORT ADDRs will be executed, thus the 100ms timeout will be
             * cancelled and the next query will execute properly.
             * IF the command in between is not ignored and disturbs the two enclosing commands then
             * the first STORE DTR AS SHORT ADDR will be cancelled by the command in between and the second
             * STORE DTR AS SHORT ADDR will trigger a new 100ms time-out which will make the DUT
             * ignore the QUERY CONTROL GEAR that follows. The query being ignored will not generate
             * a response and thus the testing algorithm will interpret this as a "No" coming from the
             * DUT and will generate the error 
             * 
             * Hence the logic does work out in this case but only accidentally. For it to work as intended
             * the following 150ms delay is added.
             */
            // change ref 7
            Info("Wait 150ms");
            Thread.Sleep(150);


            // SHORT ADDRESS 3 QUERY CONTROL GEAR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0x07; // Short address 3 command
            outgoing[3] = 0x91;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Error("Error 2336 \"Command not executed with command in-between addressed to other DUT\"");
                }
            }

            // BROADCAST DTR(255)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);


            // 2x BROADCAST STORE DTR AS SHORT ADDRESS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_4_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte next = 0, value = 0;

            int r, i, iMax, CS;

            Message("Starting test 12.2.4.1: Memory Bank 0");

            // SPECIAL COMMAND DTR(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 0;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND DTR1(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC3;
            outgoing[3] = 0;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND DTR2(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC5;
            outgoing[3] = 0;
            sendData(outgoing);
            r = getData(incoming, 100);


            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }

            if (incoming[3] <= 13)
            {
                Error("Error 2411 \"Not all mandatory memory locations implemented\"");
                Message("Test done");
                return;
            }

            iMax = incoming[3];

            i = 1;
            CS = 0;


            // BROADCAST QUERY CONTENT DTR
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x98;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] != 1)
                {
                    Error("Error 2412 \"Address / DTR not incremented\"");
                }
            }


            for (; i <= iMax; i++)
            {

                // BROADCAST QUERY CONTENT DTR2
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x9D;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        next = incoming[3];
                    }
                }

                // BROADCAST READ MEMORY LOCATION
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC5;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("Error (2413," + i.ToString() + ") \"No answer reading memory location " + i.ToString() + "\"");
                    }
                    else
                    {
                        Message("Content of memory location " + i.ToString() + " is " + incoming[3].ToString());
                        if (incoming[3] != next)
                        {
                            Error("Error (2414," + i.ToString() + ") \"Content of DTR2 wrong after test step " + (i - 1).ToString() + "\"");
                        }
                    }
                }

                CS += incoming[3];
            }

            CS = CS % 256;

            if (CS != 0)
            {
                Debug("\tchecksum = " + CS.ToString());
                Error("Error 2415 \"Checksum Error\"");
            }


            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 2416 \"Answer when reading invalid address");
                }
            }


            // SPECIAL COMMAND DTR(5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 5;
            sendData(outgoing);
            r = getData(incoming, 100);

            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    value = (Byte)(255 - incoming[3]);
                }
            }

            // SPECIAL COMMAND DTR(5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 5;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x81; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x81; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // SPECIAL COMMAND WRITE MEMORY LOCATION value
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = value;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                // change ref 20
                // yes/no branches swapped in the schematic, no reply means the read-only
                // location hasn't been written to
                else if (incoming[2] != 0x00)
                {
                    Error("Error 2417 \"Writing to read-only memory location confirmed\"");
                }
            }

            // SPECIAL COMMAND DTR(5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 5;
            sendData(outgoing);
            r = getData(incoming, 100);

            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] == value)
                {
                    Error("Error 2418 \"Content of read-only memory location changed\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_4_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r, i, iMax, CS = 0;

            Message("Starting test 12.2.4.2: Memory Bank 1");

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND DTR1(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC3;
            outgoing[3] = 0;
            sendData(outgoing);
            r = getData(incoming, 100);

            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }

            if (incoming[3] == 0)
            {
                Message("Memory bank 1 not accessible");
                Message("Test done");
                return;
            }

            // SPECIAL COMMAND DTR1(1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC3;
            outgoing[3] = 1;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND DTR(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 0;
            sendData(outgoing);
            r = getData(incoming, 100);

            incoming[3] = 0; // if the following command does not send a reply, the answer (iMax) will be undefined
            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("Error 2421 \"No answer reading size of memory bank 1\"");
                }
            }

            i = 3;
            iMax = incoming[3];

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);


            // 2x BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x81; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x81; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // SPECIAL COMMAND WRITE MEMORY LOCATION (0x55)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = 0x55;
            sendData(outgoing);
            r = getData(incoming, 100);

            for (; i <= iMax; i++)
            {
                // SPECIAL COMMAND WRITE MEMORY LOCATION (255 - i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xC7;
                outgoing[3] = (Byte)(255 - i);
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("Error 2422 \"Writing memory location " + i.ToString() + " not confirmed\"");
                    }
                    else if (incoming[3] != (Byte)(255 - i))
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + (255 - i).ToString());
                        Error("Error 2422 \"Writing memory location " + i.ToString() + " not confirmed\"");
                    }
                }
            }

            // SPECIAL COMMAND DTR(1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 1;
            sendData(outgoing);
            r = getData(incoming, 100);

            incoming[3] = 0;
            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }

            CS = incoming[3];
            i = 3;

            incoming[3] = 0;
            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] != 0x55)
                {
                    Error("Error 2423 \"Memory lock byte wrong\"");
                }
            }

            CS += incoming[3];

            for (; i <= iMax; i++)
            {
                incoming[3] = 0;
                // BROADCAST READ MEMORY LOCATION
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC5;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if (incoming[3] != (Byte)(255 - i))
                    {
                        Error("Error (2424," + i.ToString() + ") \"Content of memory location " + i.ToString() + " is wrong\"");
                    }
                }

                CS += incoming[3];
            }

            CS = CS % 256;

            if (CS != 0)
            {
                Error("Error 2425 \"Checksum Error\"");
            }

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x81; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x81; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // SPECIAL COMMAND WRITE MEMORY LOCATION (0xAA)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = 0xAA;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received, " + incoming[1].ToString());
                }
            }

            // SPECIAL COMMAND DTR(3)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x81; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x81; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // SPECIAL COMMAND WRITE MEMORY LOCATION (0xAA)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = 0xAA;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 2426 \"Writing to locked memory location confirmed\"");
                }
            }

            // SPECIAL COMMAND DTR(3)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);

            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] != 252)
                {
                    Error("Error 2427 \"Content of locked memory location changed\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_4_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r, i, iMax, k, kMax, CS = 0;

            Message("Starting test 12.2.4.3: Other Memory Banks");

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND DTR1(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC3;
            outgoing[3] = 0;
            sendData(outgoing);
            r = getData(incoming, 100);

            incoming[3] = 0;
            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }

            k = 2;
            kMax = incoming[3];

            if (kMax <= 1)
            {
                Message("\"No additional memory bank accessible\"");
                Message("Test done");
                return;
            }

            for (; k <= kMax; k++)
            {
                // SPECIAL COMMAND DTR(0)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 0;
                sendData(outgoing);
                r = getData(incoming, 100);

                // SPECIAL COMMAND DTR1(k)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xC3;
                outgoing[3] = (Byte)(k);
                sendData(outgoing);
                r = getData(incoming, 100);

                incoming[3] = 0;
                // BROADCAST READ MEMORY LOCATION
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC5;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("Error 2421 \"No answer reading size of memory bank 1\"");
                    }
                }

                i = 2;
                iMax = incoming[3];

                incoming[3] = 0;
                // BROADCAST READ MEMORY LOCATION
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC5;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("Error 2421 \"No answer reading size of memory bank 1\"");
                    }
                }

                CS = incoming[3];

                for (; i <= iMax; i++)
                {
                    incoming[3] = 0;
                    // BROADCAST READ MEMORY LOCATION
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0xC5;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("Error (2431," + i.ToString() + "," + k.ToString() + ") \"No answer reading memory location " + i.ToString() + " of memory bank " + k.ToString() + "\"");
                        }
                        else
                        {
                            Message("\"Content of memory location " + i.ToString() + " of memory bank " + k.ToString() + " is " + incoming[3].ToString() + "\"");
                            CS += incoming[3];
                        }
                    }
                }

                CS = CS % 256;

                if (CS != 0)
                {
                    Error("Error (2432," + k.ToString() + ") \"Checksum of memory bank " + k.ToString() + " is wrong\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_2_4_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] address = new Byte[2];
            String[] decisionText = new String[2];

            String[] text1 = new String[2];
            String[] text2 = new String[2];

            address[0] = 0xFF;
            address[1] = 0x81; // group address 0

            decisionText[0] = "answer = 0";
            decisionText[1] = "answer != 1";

            text1[0] = "changed";
            text1[1] = "not changed";

            text2[0] = "same control gear";
            text2[1] = "other control gear";

            int r, i, iMax;

            Message("Starting test 12.2.4.4: ENABLE WRITE MEMORY");

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND DTR1(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC3;
            outgoing[3] = 0;
            sendData(outgoing);
            r = getData(incoming, 100);

            incoming[3] = 0;
            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }

            if (incoming[3] == 0)
            {
                Message("\"Memory bank 1 not accessible\"");
                Message("Test done");
                return;
            }

            // SPECIAL COMMAND DTR1(1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC3;
            outgoing[3] = 1;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x81; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x81; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // SPECIAL COMMAND WRITE MEMORY LOCATION (0x55)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = 0x55;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
            }

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);

            // BROADCAST READ MEMORY LOCATION
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else if (incoming[3] != 0x55)
                {
                    Error("Error 2441 \"Memory bank 1 not unlocked\"");
                }
            }

            // SPECIAL COMMAND DTR(2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);

            // BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x81;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND WRITE MEMORY LOCATION (5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = 0x05;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x01)
                {
                    Error("Error 2442 \"Writing confirmed without enable sent twice\"");
                }
            }

            i = 0;
            iMax = 1;

            for (; i <= iMax; i++)
            {
                // SPECIAL COMMAND DTR(2)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 2;
                sendData(outgoing);
                r = getData(incoming, 100);


                // BROADCAST ENABLE WRITE MEMORY, address i RECALL MAX LEVEL, BROADCAST ENABLE WRITE MEMORY
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x81; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = address[i]; // Command 1 addr
                outgoing[8] = 0x05; // Command 1 data
                outgoing[9] = 0x00; // Command 1 flags
                outgoing[10] = 0xFF; // Command 1 addr
                outgoing[11] = 0x81; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // SPECIAL COMMAND WRITE MEMORY LOCATION (i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xC7;
                outgoing[3] = (Byte)(i);
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                }

                // SPECIAL COMMAND DTR(2)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 2;
                sendData(outgoing);
                r = getData(incoming, 100);

                // BROADCAST READ MEMORY LOCATION
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC5;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if (i == 0)
                    {
                        if (incoming[3] == 0)
                        {
                            Error("Error (2443," + i.ToString() + ") \"Content " + text1[i] + " with command in-between addressed to " + text2[i] + "\"");
                        }
                    }
                    else if (i == 1)
                    {
                        if (incoming[3] != 1)
                        {
                            Error("Error (2443," + i.ToString() + ") \"Content " + text1[i] + " with command in-between addressed to " + text2[i] + "\"");
                        }
                    }
                }
            }

            // BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x81;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait 150ms");
            Thread.Sleep(150);

            // BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x81;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND WRITE MEMORY LOCATION (2)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = 2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x01)
                {
                    Error("Error 2444 \"Writing confirmed with time-out\"");
                }
            }

            // 2x BROADCAST ENABLE WRITE MEMORY
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x81; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x81; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST RECALL MAX LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x05;
            sendData(outgoing);
            r = getData(incoming, 100);

            // SPECIAL COMMAND WRITE MEMORY LOCATION (3)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xC7;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x01)
                {
                    Error("Error 2445 \"Write Enabled State not cleared\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_1_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte mid, PHM;
            Byte[] value = new Byte[4];

            int[] tMin = new int[16];
            int[] tMax = new int[16];

            long time, t0;
            int r, i, iMax, k, kMax;

            tMin[1] = 640;
            tMin[2] = 900;
            tMin[3] = 1270;
            tMin[4] = 1800;
            tMin[5] = 2550;
            tMin[6] = 3600;
            tMin[7] = 5090;
            tMin[8] = 7200;
            tMin[9] = 10180;
            tMin[10] = 14400;
            tMin[11] = 20360;
            tMin[12] = 28800;
            tMin[13] = 40730;
            tMin[14] = 57600;
            tMin[15] = 81460;

            tMax[1] = 780;
            tMax[2] = 1100;
            tMax[3] = 1560;
            tMax[4] = 2200;
            tMax[5] = 3110;
            tMax[6] = 4400;
            tMax[7] = 6220;
            tMax[8] = 8800;
            tMax[9] = 12450;
            tMax[10] = 17600;
            tMax[11] = 24890;
            tMax[12] = 35200;
            tMax[13] = 49780;
            tMax[14] = 70400;
            tMax[15] = 99560;


            Message("Starting test 12.3.1.1: FADE TIME");


            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            i = 1;
            iMax = 15;

            mid = (Byte)((PHM + 254) / 2);
            value[0] = 1;
            value[1] = 254;
            value[2] = mid;
            value[3] = 254;

            for (; i <= iMax; i++)
            {
                // BROADCAST DTR(i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = (Byte)(i);
                sendData(outgoing);
                getData(incoming, 100);

                // 2x BROADCAST STORE DTR FADE TIME
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2E; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2E; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                k = 0;
                kMax = 3;

                for (; k <= kMax; k++)
                {

                    // BROADCAST DIRECT ARC POWER CONTROL (value k)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFE;
                    outgoing[3] = value[k];
                    sendData(outgoing);
                    getData(incoming, 100);

                    time = 0;
                    t0 = DateTime.Now.Ticks;


                    do
                    {
                        time = (DateTime.Now.Ticks - t0) / 10000;
                    } while (time < (tMin[i] - 300));


                    int exitLoop = 0;
                    do
                    {
                        // BROADCAST QUERY STATUS
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                        outgoing[2] = 0xFF;
                        outgoing[3] = 0x90;
                        sendData(outgoing);
                        r = getData(incoming, 100);
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if (incoming[2] != 0x01)
                            {
                                Error("No answer on DALI bus");
                            }
                            else
                            {
                                if ((incoming[3] & 0x10) != 0x10) // check if status == xxx1xxxx
                                {
                                    exitLoop = 1;
                                }
                            }
                        }

                        time = (DateTime.Now.Ticks - t0) / 10000;
                        if (time > tMax[i])
                        {
                            exitLoop = 1;
                        }

                    } while (exitLoop == 0);

                    Debug("\tTime = " + time.ToString() + " tMin = " + tMin[i].ToString() + " tMax = " + tMax[i].ToString());
                    if ((tMin[i] > time) || (time > tMax[i]))
                    {
                        Error("Error (3111," + i.ToString() + "," + k.ToString() + ") \"FADE TIME out of range\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_1_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            Byte[] n = new Byte[16];
            int[] sMin = new int[16];
            int[] sMax = new int[16];

            int s, r, i, iMax, j;

            n[1] = 1;
            n[2] = 2;
            n[3] = 3;
            n[4] = 5;
            n[5] = 7;
            n[6] = 11;
            n[7] = 15;
            n[8] = 22;
            n[9] = 31;
            n[10] = 45;
            n[11] = 63;
            n[12] = 90;
            n[13] = 127;
            n[14] = 181;
            n[15] = 255;


            sMin[1] = 64;
            sMin[2] = 68;
            sMin[3] = 64;
            sMin[4] = 68;
            sMin[5] = 64;
            sMin[6] = 67;
            sMin[7] = 63;
            sMin[8] = 64;
            sMin[9] = 62;
            sMin[10] = 63;
            sMin[11] = 61;
            sMin[12] = 60;
            sMin[13] = 58;
            sMin[14] = 55;
            sMin[15] = 51;

            sMax[1] = 78;
            sMax[2] = 83;
            sMax[3] = 78;
            sMax[4] = 83;
            sMax[5] = 79;
            sMax[6] = 84;
            sMax[7] = 79;
            sMax[8] = 81;
            sMax[9] = 80;
            sMax[10] = 82;
            sMax[11] = 81;
            sMax[12] = 83;
            sMax[13] = 85;
            sMax[14] = 88;
            sMax[15] = 91;


            Message("Starting test 12.3.1.2: FADE RATE");


            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            i = 1;
            iMax = 15;

            for (; i <= iMax; i++)
            {
                // BROADCAST DTR(i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = (Byte)(i);
                sendData(outgoing);
                getData(incoming, 100);

                // 2x BROADCAST STORE DTR FADE RATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2F; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2F; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST RECALL MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x05;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                j = n[i];

                // change ref 15
                do
                {
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_PING_DELAY;
                    outgoing[2] = 0x00;
                    if (j == n[i])
                    {
                        outgoing[3] = 115; // 100ms + 15.8ms delay up to which the device can't know what to do -1ms for USB
                    }
                    else
                    {
                        outgoing[3] = 99; // 100ms -1ms for usb
                    }
                    sendData(outgoing);

                    // BROADCAST DOWN
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x02;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    j--;

                    Info("Wait 100ms");
                    getData(incoming, 300); // waiting for the REPLY_PING

                } while (j > 0);


                Info("Wait Response Time + 200ms = " + (preaheatTimeControl.Value + 200).ToString() + "ms");
                Thread.Sleep((int)preaheatTimeControl.Value + 200);


                incoming[3] = 0;
                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                s = (254 - incoming[3]);

                Debug("\t s = " + s.ToString() + " sMin = " + sMin[i].ToString() + " sMax = " + sMax[i].ToString());
                if ((sMin[i] > s) || (s > sMax[i]))
                {
                    Debug("\t s = " + s.ToString() + " sMin = " + sMin[i].ToString() + " sMax = " + sMax[i].ToString());
                    Error("Error (3121," + i.ToString() + ") \"Wrong number of steps at command DOWN and FADE RATE " + i.ToString() + "\"");
                }
            }


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            i = 1;
            iMax = 15;

            for (; i <= iMax; i++)
            {
                // BROADCAST DTR(i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = (Byte)(i);
                sendData(outgoing);
                getData(incoming, 100);

                // 2x BROADCAST STORE DTR FADE RATE
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2F; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2F; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);


                j = n[i];
                // change ref 16
                do
                {
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_PING_DELAY;
                    outgoing[2] = 0x00;
                    if (j == n[i])
                    {
                        outgoing[3] = 115; // 100ms + 15.8ms delay up to which the device can't know what to do -1ms for USB 
                    }
                    else
                    {
                        outgoing[3] = 99; // 100ms -1ms for USB
                    }
                    sendData(outgoing);

                    // BROADCAST UP
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x01;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    j--;

                    Info("Wait 100ms");
                    getData(incoming, 300); // waiting for the REPLY_PING

                } while (j > 0);


                Info("Wait Response Time + 200ms = " + (preaheatTimeControl.Value + 200).ToString() + "ms");
                Thread.Sleep((int)preaheatTimeControl.Value + 200);


                incoming[3] = 0;
                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                s = (incoming[3] - PHM);

                if ((sMin[i] > s) || (s > sMax[i]))
                {
                    Debug("\t s = " + s.ToString() + " sMin = " + sMin[i].ToString() + " sMax = " + sMax[i].ToString());
                    Error("Error (3122," + i.ToString() + ") \"Wrong number of steps at command UP and FADE RATE " + i.ToString() + "\"");
                }
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_2_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            Byte[] level = new Byte[8];
            double[] minimum = new double[8];
            double[] nominal = new double[8];
            double[] maximum = new double[8];

            double[] value = new double[8];

            double value100;
            double readValue;

            Func<string, double> method = GetData;
            IAsyncResult cookie;


            int r, i, iMax;

            level[1] = 229;
            level[2] = 195;
            level[3] = 170;
            level[4] = 126;
            level[5] = 85;
            level[6] = 60;
            level[7] = 1;

            minimum[1] = 40;
            minimum[2] = 15;
            minimum[3] = 7;
            minimum[4] = 2;
            minimum[5] = 0.5;
            minimum[6] = 0.25;
            minimum[7] = 0;

            nominal[1] = 50;
            nominal[2] = 20;
            nominal[3] = 10;
            nominal[4] = 3;
            nominal[5] = 1;
            nominal[6] = 0.5;
            nominal[7] = 0.1;

            maximum[1] = 71;
            maximum[2] = 30;
            maximum[3] = 15;
            maximum[4] = 4.5;
            maximum[5] = 2;
            maximum[6] = 1;
            maximum[7] = 0.2;

            Message("Starting test 12.3.2.1: LOGARITHMIC DIMMING CURVE");


            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            i = 1;
            iMax = 7;

            Message("Wait 10 minutes");
            for (int d = 600; d > 0; d--)
            {
                if (d % 5 == 0) Message("\t" + d.ToString() + "s remaining");
                Thread.Sleep(1000);
            }

            cookie = method.BeginInvoke("Measure relative light output of lamps and enter value in the input field", null, null);
            readValue = method.EndInvoke(cookie);

            Message("Storing " + readValue.ToString() + " as 100%");
            value100 = readValue;

            for (; i <= iMax; i++)
            {
                if (level[i] < PHM)
                {
                    break;
                }

                // BROADCAST DIRECT ARC POWER CONTROL (level i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFE;
                outgoing[3] = level[i];
                sendData(outgoing);
                getData(incoming, 100);

                Message("Wait 10 minutes");
                for (int d = 600; d > 0; d--)
                {
                    if (d % 5 == 0) Message("\t" + d.ToString() + "s remaining");
                    Thread.Sleep(1000);
                }

                cookie = method.BeginInvoke("Measure relative light output of lamps and enter value in the input field", null, null);
                readValue = method.EndInvoke(cookie);

                value[i] = 100.0 * readValue / value100;
                Message("Storing " + readValue.ToString() + ". Relative light output is " + value[i].ToString() + "%");

                if ((minimum[i] >= value[i]) || (value[i] >= maximum[i]))
                {
                    Debug("\tvalue = " + value[i].ToString() + " min = " + minimum[i].ToString() + " max = " + maximum[i].ToString());
                    Error("Error 3211," + i.ToString() + " \"value " + i.ToString() + " out of tolerances\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_2_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r;

            Message("Starting test 12.3.2.2: CONTINUOUS DIMMING: DAPC");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            // BROADCAST DTR(5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 5;
            sendData(outgoing);
            getData(incoming, 100);

            // 2x BROADCAST STORE DTR AS FADE TIME
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2E; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2E; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST DIRECT ARC POWER CONTROL (1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 1;
            sendData(outgoing);
            getData(incoming, 100);

            Info("Wait 3s");
            Thread.Sleep(3000);

            if (MessageBox.Show("Dimming curve continuous?", "Question", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 3221 \"Dimming curve not continuous!\"");
            }

            // BROADCAST DIRECT ARC POWER CONTROL (254)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 254;
            sendData(outgoing);
            getData(incoming, 100);

            Info("Wait 3s");
            Thread.Sleep(3000);

            if (MessageBox.Show("Dimming curve continuous?", "Question", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 3222 \"Dimming curve not continuous!\"");
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_2_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r, i, iMax;

            Message("Starting test 12.3.2.3: CONTINUOUS DIMMING: UP / DOWN");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            i = 0;
            iMax = 60;

            for (; i <= iMax; i++)
            {
                // BROADCAST DOWN
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x02;
                sendData(outgoing);
                getData(incoming, 100);

                Info("Wait 100ms");
                Thread.Sleep(100);
            }

            if (MessageBox.Show("Dimming curve continuous?", "Question", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 3231 \"Dimming curve not continuous!\"");
            }

            i = 0;
            iMax = 60;

            for (; i <= iMax; i++)
            {
                // BROADCAST UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x01;
                sendData(outgoing);
                getData(incoming, 100);

                Info("Wait 100ms");
                Thread.Sleep(100);
            }

            if (MessageBox.Show("Dimming curve continuous?", "Question", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 3232 \"Dimming curve not continuous!\"");
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_2_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r, i, iMax;

            Message("Starting test 12.3.2.4: CONTINUOUS DIMMING: STEP UP / STEP DOWN");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            i = 0;
            iMax = 254 - PHM;

            for (; i <= iMax; i++)
            {
                // BROADCAST STEP DOWN
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x04;
                sendData(outgoing);
                getData(incoming, 100);

                Info("Wait 50ms");
                Thread.Sleep(50);
            }

            if (MessageBox.Show("Dimming curve continuous?", "Question", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 3241 \"Dimming curve not continuous!\"");
            }

            i = 0;
            iMax = 254 - PHM;

            for (; i <= iMax; i++)
            {
                // BROADCAST STEP UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x03;
                sendData(outgoing);
                getData(incoming, 100);

                Info("Wait 50ms");
                Thread.Sleep(50);
            }

            if (MessageBox.Show("Dimming curve continuous?", "Question", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 3242 \"Dimming curve not continuous!\"");
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_2_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];
            Byte[] value = new Byte[14];

            Byte PHM;

            long t0, time;
            int r, i, iMax;

            value[0] = 254;
            value[1] = 250;
            value[2] = 246;
            value[3] = 241;
            value[4] = 235;
            value[5] = 229;
            value[6] = 221;
            value[7] = 210;
            value[8] = 195;
            value[9] = 170;
            value[10] = 145;
            value[11] = 85;
            value[12] = 60;
            value[13] = 1;


            Message("Starting test 12.3.2.5: Dimming curve: DAPC SEQUENCE");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM == 254)
            {
                Message("Control gear not dimmable");
                Message("Test done");
                return;
            }

            i = 0;
            iMax = 13;

            // BROADCAST ENABLE DAPC SEQUENCE
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x09;
            sendData(outgoing);
            getData(incoming, 100);

            for (; i <= iMax; i++)
            {
                if (value[i] < PHM)
                {
                    break;
                }

                // change ref 24
                // The 180ms delay needs to include the sending of the command as well, since
                // sending the command takes ~15.8ms. Thus the delay would be 195.8ms and adding
                // some variation could easily allow the 200ms timer to expire. The change makes
                // the actual executions of the commands 180ms apart.
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_PING_DELAY;
                outgoing[2] = 0x00;
                outgoing[3] = 180;
                sendData(outgoing);

                // BROADCAST DIRECT ARC POWER CONTROL (value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFE;
                outgoing[3] = value[i];
                sendData(outgoing);
                getData(incoming, 100);

                Info("Wait 180ms");
                getData(incoming, 400);
            }

            if (MessageBox.Show("Dimming curve continuous?", "Question", MessageBoxButtons.YesNo) == DialogResult.No)
            {
                Error("Error 3251 \"Dimming curve not continuous!\"");
            }

            // BROADCAST ENABLE DAPC SEQUENCE
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x09;
            sendData(outgoing);
            getData(incoming, 100);

            // BROADCAST DIRECT ARC POWER CONTROL (254)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 254;
            sendData(outgoing);
            getData(incoming, 100);

            t0 = DateTime.Now.Ticks;

            do
            {
                incoming[3] = 0;
                // BROADCAST QUERY STATUS
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                    }
                }
            } while ((incoming[3] & 0x10) != 0);

            time = (DateTime.Now.Ticks - t0) / 10000;
            if (time < 200)
            {
                Debug("\ttime = " + time.ToString() + "ms min = 200ms");

                // change ref 8
                // original code: Error("Error 3252 \"Fade time to small during DAPC sequence\"");
                Error("Error 3252 \"Fade time too small during DAPC sequence\"");
            }

            // BROADCAST ENABLE DAPC SEQUENCE
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x09;
            sendData(outgoing);
            getData(incoming, 100);

            // BROADCAST DIRECT ARC POWER CONTROL (PHM)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = PHM;
            sendData(outgoing);
            getData(incoming, 100);

            Info("Wait 250ms");
            Thread.Sleep(250);

            // BROADCAST DIRECT ARC POWER CONTROL (254)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 254;
            sendData(outgoing);
            getData(incoming, 100);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x10) == 0x10)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3253 \"DAPC sequence not cancelled by timeout\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.3.3.1: OFF");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);


            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3311 \"DUT is not switched off!\"");
                    }
                }
            }

            incoming[3] = 0;
            // BROADCAST QUERY ACTUAL LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xA0;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if (incoming[3] != 0)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 0");
                        Error("Error 3312 \"ACTUAL LEVEL is not 0!\"");
                    }
                }
            }

            // BROADCAST RECALL MAX LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x05;
            sendData(outgoing);
            getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] value = new Byte[6];
            Byte[] level = new Byte[6];

            Byte PHM, mid;

            int r, i, iMax;

            Message("Starting test 12.3.3.2: DIRECT ARC POWER CONTROL");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            mid = (Byte)((PHM + 254) / 2);

            i = 1;
            iMax = 5;

            value[1] = 0;
            value[2] = 1;
            value[3] = mid;
            value[4] = 255;
            value[5] = 254;

            if (PHM == 254)
            {
                level[1] = 0;
                level[2] = 254;
                level[3] = 254;
                level[4] = 254;
                level[5] = 254;
            }
            else
            {
                level[1] = 0;
                level[2] = (Byte)(PHM + 1);
                level[3] = mid;
                level[4] = mid;
                level[5] = 253;
            }

            // BROADCAST DTR(PHM + 1)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = (Byte)(PHM + 1);
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x BROADCAST STORE DTR AS MIN LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2B; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2B; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            // BROADCAST DTR(253)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 253;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x BROADCAST STORE DTR AS MAX LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2A; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2A; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            for (; i <= iMax; i++)
            {
                // BROADCAST DIRECT ARC POWER CONTROL (value i)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFE;
                outgoing[3] = value[i];
                sendData(outgoing);
                getData(incoming, 100);

                Info("Wait Preheat Time + Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[i].ToString());
                            Error("Error 3321 \"Wrong level at test step " + i.ToString() + "\"");
                        }
                    }
                }
            }

            // BROADCAST DTR(4)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 4;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x BROADCAST STORE DTR AS FADE TIME
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2E; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2E; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // BROADCAST DIRECT ARC POWER CONTROL (0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 0;
            sendData(outgoing);
            getData(incoming, 100);

            // BROADCAST DIRECT ARC POWER CONTROL (255)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 255;
            sendData(outgoing);
            getData(incoming, 100);

            Info("Wait Response Time + 2.5s");
            Thread.Sleep((int)preaheatTimeControl.Value + 2500);

            incoming[3] = 0;
            // BROADCAST QUERY LAMP POWER ON
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x93;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Debug("\tReceived answer = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 1, 255");
                    Error("Error 3322 \"MASK does not stop a fade task!\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r;

            Message("Starting test 12.3.3.3: UP");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM != 254)
            {
                // BROADCAST DTR(253)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 253;
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2A; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2A; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x01;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait 200ms");
                Thread.Sleep(200);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 253)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 253");
                            Error("Error 3331 \"Change of actual level at MAX LEVEL\"");
                        }
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x01;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait 200ms");
                Thread.Sleep(200);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] <= PHM)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected > " + PHM.ToString());
                            Error("Error 3332 \"No change of actual level at MIN LEVEL\"");
                        }
                    }
                }

            }
            else
            {
                Message("Control gear not dimmable");
            }


            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST UP
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x01;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3333 \"UP switches on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r;

            Message("Starting test 12.3.3.4: DOWN");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM != 254)
            {
                // BROADCAST DTR (PHM + 1)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = (Byte)(PHM + 1);
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2B; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2B; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST DOWN
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x02;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait 200ms");
                Thread.Sleep(200);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] == 254)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 254");
                            Error("Error 3341 \"No change of actual level at MAX LEVEL\"");
                        }
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST DOWN
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x02;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait 200ms");
                Thread.Sleep(200);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != (Byte)(PHM + 1))
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + (PHM + 1).ToString());
                            Error("Error 3342 \"Change of actual level at MIN LEVEL\"");
                        }
                    }
                }

            }
            else
            {
                Message("Control gear not dimmable");
            }

            // BROADCAST DOWN
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x02;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait 200ms");
            Thread.Sleep(200);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) != 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3343 \"DOWN switches off\"");
                    }
                }
            }

            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST DOWN
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x02;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3344 \"DOWN switches on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r;

            Message("Starting test 12.3.3.5: STEP UP");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM != 254)
            {
                // BROADCAST DTR(253)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 253;
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2A; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2A; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST STEP UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x03;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 253)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 253");
                            Error("Error 3351 \"Change of actual level at MAX LEVEL\"");
                        }
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST STEP UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x03;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != (Byte)(PHM + 1))
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + (PHM + 1).ToString());
                            Error("Error 3352 \"No change of actual level at MIN LEVEL\"");
                        }
                    }
                }

            }
            else
            {
                Message("Control gear not dimmable");
            }


            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST STEP UP
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x03;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3353 \"STEP UP switches on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r;

            Message("Starting test 12.3.3.6: STEP DOWN");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            if (PHM != 254)
            {
                // BROADCAST DTR (PHM + 1)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = (Byte)(PHM + 1);
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2B; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2B; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST STEP DOWN
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x04;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 253)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 253");
                            Error("Error 3361 \"No change of actual level at MAX LEVEL\"");
                        }
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST STEP DOWN
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x04;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != (Byte)(PHM + 1))
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + (PHM + 1).ToString());
                            Error("Error 3362 \"Change of actual level at MIN LEVEL\"");
                        }
                    }
                }

            }
            else
            {
                Message("Control gear not dimmable");
            }

            // BROADCAST STEP DOWN
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x04;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) != 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3363 \"STEP DOWN switches off\"");
                    }
                }
            }

            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST STEP DOWN
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x04;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3364 \"STEP DOWN switches on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_7()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM, level;

            int r;

            Message("Starting test 12.3.3.7: RECALL MAX LEVEL");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            level = 254;

            if (PHM != 254)
            {
                level = 253;

                // BROADCAST DIRECT ARC POWER CONTROL (1)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFE;
                outgoing[3] = 1;
                sendData(outgoing);
                getData(incoming, 100);

                // BROADCAST DTR (level)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = level;
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2A; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2A; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST RECALL MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x05;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level.ToString());
                            Error("Error 3371 \"Wrong actual level after RECALL MAX LEVEL\"");
                        }
                    }
                }
            }
            else
            {
                Message("Control gear not dimmable");
            }

            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST RECALL MAX LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x05;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) != 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3372 \"No switching on with RECALL MAX LEVEL\"");
                    }
                }
            }

            // BROADCAST QUERY ACTUAL LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xA0;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if (incoming[3] != level)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level.ToString());
                        Error("Error 3373 \"MAX LEVEL not reached at switching on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_8()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM, level;

            int r;

            Message("Starting test 12.3.3.8: RECALL MIN LEVEL");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());

            level = PHM;

            if (PHM != 254)
            {
                level = (Byte)(PHM + 1);

                // BROADCAST DTR (level);
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = level;
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2B; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2B; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level.ToString());
                            Error("Error 3381 \"Wrong actual level after RECALL MIN LEVEL\"");
                        }
                    }
                }
            }
            else
            {
                Message("Control gear not dimmable");
            }

            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST RECALL MIN LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x06;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) != 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3382 \"No switching on with RECALL MIN LEVEL\"");
                    }
                }
            }

            // BROADCAST QUERY ACTUAL LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xA0;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if (incoming[3] != level)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level.ToString());
                        Error("Error 3383 \"MIN LEVEL not reached at switching on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_9()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r;

            Message("Starting test 12.3.3.9: ON AND STEP UP");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());


            if (PHM != 254)
            {
                // BROADCAST DTR(253)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 253;
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MAX LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2A; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2A; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST ON AND STEP UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x08;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 253)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 253");
                            Error("Error 3391 \"Change of actual level at MAX LEVEL\"");
                        }
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST ON AND STEP UP
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x08;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != (Byte)(PHM + 1))
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + (PHM + 1).ToString());
                            Error("Error 3392 \"No change of actual level at MIN LEVEL\"");
                        }
                    }
                }

            }
            else
            {
                Message("Control gear not dimmable");
            }

            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST ON AND STEP UP
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x08;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) != 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3393 \"ON AND STEP UP does not switch on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_3_3_10()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte PHM;

            int r;

            Message("Starting test 12.3.3.10: STEP DOWN AND OFF");

            // 2x BROADCAST RESET
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            incoming[3] = 0;
            // BROADCAST QUERY PHYSICAL MINIMUM LEVEL
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
            }
            PHM = incoming[3];
            Debug("PHM = " + PHM.ToString());


            if (PHM != 254)
            {
                // BROADCAST DTR (PHM + 1);
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = (Byte)(PHM + 1);
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x BROADCAST STORE DTR AS MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2B; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2B; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // BROADCAST STEP DOWN AND OFF
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x07;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // BROADCAST QUERY ACTUAL LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 253)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 253");
                            Error("Error 3301 \"No change of actual level at MAX LEVEL\"");
                        }
                    }
                }

                // BROADCAST RECALL MIN LEVEL
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x06;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

            }
            else
            {
                Message("Control gear not dimmable");
            }

            // BROADCAST STEP DOWN AND OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x07;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3302 \"STEP DOWN AND OFF does not switch off\"");
                    }
                }
            }

            // BROADCAST OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // BROADCAST STEP DOWN AND OFF
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x07;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            incoming[3] = 0;
            // BROADCAST QUERY STATUS
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04)
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString());
                        Error("Error 3303 \"STEP DOWN AND OFF switches on\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;
            long time, t0;

            Message("Starting test 12.4: Physical address allocation");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // pyhisical selection
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBD;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x01)
                {
                    Error("Error 4001 \"DUT selected at beginning of the test\"");
                }
            }


            MessageBox.Show("Disconnect Lamp(s)", "Required action");
            Message("Lamp(s) disconnected");

            time = 0;
            t0 = DateTime.Now.Ticks;

            do
            {
                // query short address
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xBB;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x01)
                    {

                        // program short addr (1)
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                        outgoing[2] = 0xB7;
                        outgoing[3] = 0x01;
                        sendData(outgoing);
                        r = getData(incoming, 100);

                        break;
                    }
                }
                time = (DateTime.Now.Ticks - t0) / 10000;
            } while (time <= 10000);

            if (time > 10000)
            {
                Error("Error 4002 \"Timeout, no Physical Selection possible\"");
            }


            // short addr 0 query control gear
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0x01;
            outgoing[3] = 0x91;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Error("Error 4003 \"Programming of short address failed\"");
                }
            }

            MessageBox.Show("Reconnect Lamp(s)", "Required action");
            Message("Lamp(s) reconnected");

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            // pyhisical selection
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBD;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x01)
                {
                    Error("Error 4004 \"Command 'PHYSICAL SELECTION' does not clear the selection\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_1_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.1.1: INITIALISE: 15 minutes timer");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5111 \"Special processes not terminated\"");
                }
            }


            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("Error 5112 \"Initialisation not possible\"");
                }
            }

            Message("Wait 13 minutes");
            for (int d = 780; d > 0; d--)
            {
                if (d % 5 == 0) Message("\t" + d.ToString() + "s remaining");
                Thread.Sleep(1000);
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("Error 5113 \"Timer expires too early\"");
                }
            }

            Message("Wait 4 minutes");
            for (int d = 240; d > 0; d--)
            {
                if (d % 5 == 0) Message("\t" + d.ToString() + "s remaining");
                Thread.Sleep(1000);
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5114 \"Timer expires too late\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_1_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.1.2: TERMINATE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("Error 5121 \"Initialisation not possible\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5122 \"Initialisation not terminated");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_1_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.1.3: INITIALISE: short address");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // dtr = 3
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // 2x initialise(5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x05; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x05; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5131 \"Initialisation of DUT with wrong short address\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x initialise(255)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0xFF; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0xFF; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5132 \"INITIALISE addressed to DUT without short address executed\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x initialise(3)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x03; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x03; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("Error 5133 \"INITIALISE addressed to DUT not executed\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_1_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.1.4: INITIALISE: no short address");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // 2x initialise(5)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x05; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x05; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5141 \"INITIALISE addressed to DUT with short address executed!");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x initialise(255)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0xFF; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0xFF; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("Error 5142 \"INITIALISE addressed to DUT without short address not executed\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_1_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.1.5: INITIALISE: 100 ms timeout");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA5;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);


            Info("Wait 150ms");
            Thread.Sleep(150);

            // initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA5;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // change ref 14
            /* IF the INITIALISE command is executed with the timeout (which it shouln't) then the
             * QUERY SHORT ADDRESS will also be properly answered and the error will be triggered
             * IF the INITIALISE command is not executed (correct behaviour) then the second INITIALISE
             * will start its 100ms timer and then the QUERY SHORT ADDRESS will be ignored solely because
             * it came while this timer was running. Being ignored no answer will be executed and no error
             * will be generated. Thus the logic works out, but not exactly for the intended reasons.
             */
            Info("Wait 150ms");
            Thread.Sleep(150);

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5151 \"INITIALISE with timeout > 100 ms executed\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_1_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.1.6: INITIALISE: command in-between");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);


            // initialise(0), broadcast query control gear, initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 1 flags
            outgoing[10] = 0xA5; // Command 1 addr
            outgoing[11] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
                else if (incoming[6] != 0x00)
                {
                    Error("Error 5161 \"Command inbetween INITIALISE executed\"");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x00)
                {
                    Error("Error 5162 \"INITIALISE executed with command in-between\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);


            // initialise(0), group 5 query control gear, initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0x8B; // Command 1 addr
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 1 flags
            outgoing[10] = 0xA5; // Command 1 addr
            outgoing[11] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
                else if (incoming[6] != 0x00)
                {
                    Error("Error 5163 \"Command inbetween INITIALISE addressed to other control gear executed\"");
                }
            }

            // query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("Error 5164 \"INITIALISE not executed with command in-between addressed to other control gear\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_2_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte addrH, addrM, addrL;
            Byte newAddrH, newAddrM, newAddrL;

            int r;

            Message("Starting test 12.5.2.1: RANDOMISE: reset values");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // broadcast query random address (h)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrH = incoming[3];

            // broadcast query random address (m)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrM = incoming[3];

            // broadcast query random address (l)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC4;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrL = incoming[3];

            Debug("\tRandom address = " + addrH.ToString("X2") + " " + addrM.ToString("X2") + " " + addrL.ToString("X2"));

            if ((addrH != 0xFF) || (addrM != 0xFF) || (addrL != 0xFF))
            {
                Error("Error 5211 \"Wrong random address after reset\"");
            }

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // 2x randomise
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA7; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA7; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait 100ms");
            Thread.Sleep(100);

            // broadcast query random address (h)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrH = incoming[3];

            // broadcast query random address (m)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrM = incoming[3];

            // broadcast query random address (l)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC4;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrL = incoming[3];

            Debug("\tRandom address = " + addrH.ToString("X2") + " " + addrM.ToString("X2") + " " + addrL.ToString("X2"));

            if ((addrH == 0xFF) && (addrM == 0xFF) && (addrL == 0xFF))
            {
                Error("Error 5212 \"No new random address generated\"");
            }

            MessageBox.Show("Disconnect all power sources", "Required action");
            Message("Disconnected all power sources");

            Message("Wait 5s");
            Thread.Sleep(5000);

            MessageBox.Show("Reconnect all power sources", "Required action");
            Message("Reconnected all power sources");

            // flush input buffer
            while (getData(incoming, 100, 0) != 0)
                ;

            Message("Wait 500ms");
            Thread.Sleep(500);

            // broadcast query random address (h)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            newAddrH = incoming[3];

            // broadcast query random address (m)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            newAddrM = incoming[3];

            // broadcast query random address (l)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC4;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            newAddrL = incoming[3];

            Debug("\tRandom address = " + newAddrH.ToString("X2") + " " + newAddrM.ToString("X2") + " " + newAddrL.ToString("X2"));

            if ((addrH != newAddrH) || (addrM != newAddrM) || (addrL != newAddrL))
            {
                Error("Error 5213 \"Random address not stored in persistent memory\"");
            }


            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // broadcast query random address (h)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrH = incoming[3];

            // broadcast query random address (m)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrM = incoming[3];

            // broadcast query random address (l)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC4;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrL = incoming[3];

            Debug("\tRandom address = " + addrH.ToString("X2") + " " + addrM.ToString("X2") + " " + addrL.ToString("X2"));

            if ((addrH != 0xFF) || (addrM != 0xFF) || (addrL != 0xFF))
            {
                Error("Error 5214 \"No reset of the random address\"");
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_2_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte addrH, addrM, addrL;

            int r;

            Message("Starting test 12.5.2.2: RANDOMISE: 100 ms timeout");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // randomise
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA7;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait 150ms");
            Thread.Sleep(150);

            // randomise
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA7;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait 100ms");
            Thread.Sleep(100);

            // broadcast query random address (h)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrH = incoming[3];

            // broadcast query random address (m)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrM = incoming[3];

            // broadcast query random address (l)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC4;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrL = incoming[3];

            Debug("\tRandom address = " + addrH.ToString("X2") + " " + addrM.ToString("X2") + " " + addrL.ToString("X2"));

            if ((addrH != 0xFF) || (addrM != 0xFF) || (addrL != 0xFF))
            {
                Error("Error 5221 \"Randomise with timeout > 100 ms executed\"");
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_2_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte addrH, addrM, addrL;

            int r;

            Message("Starting test 12.5.2.3: RANDOMISE: command in-between");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // randomise, broadcast query control gear, randomise
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA7; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 1 flags
            outgoing[10] = 0xA7; // Command 1 addr
            outgoing[11] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
                else if (incoming[6] != 0x00)
                {
                    Error("Error 5231 \"Command inbetween RANDOMISE executed\"");
                }
            }

            Info("Wait 100ms");
            Thread.Sleep(100);

            // broadcast query random address (h)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrH = incoming[3];

            // broadcast query random address (m)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrM = incoming[3];

            // broadcast query random address (l)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC4;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrL = incoming[3];

            Debug("\tRandom address = " + addrH.ToString("X2") + " " + addrM.ToString("X2") + " " + addrL.ToString("X2"));

            if ((addrH != 0xFF) || (addrM != 0xFF) || (addrL != 0xFF))
            {
                Error("Error 5232 \"RANDOMISE executed with command in-between\"");
            }

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // randomise, group 5 query control gear, randomise
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA7; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0x8B; // Command 1 addr
            outgoing[8] = 0x91; // Command 1 data
            outgoing[9] = 0x00; // Command 1 flags
            outgoing[10] = 0xA7; // Command 1 addr
            outgoing[11] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
                else if (incoming[6] != 0x00)
                {
                    Error("Error 5233 \"Command inbetween RANDOMISE addressed to other control gear executed\"");
                }
            }

            Info("Wait 100ms");
            Thread.Sleep(100);

            // broadcast query random address (h)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC2;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrH = incoming[3];

            // broadcast query random address (m)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC3;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrM = incoming[3];

            // broadcast query random address (l)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0xC4;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }
            addrL = incoming[3];

            Debug("\tRandom address = " + addrH.ToString("X2") + " " + addrM.ToString("X2") + " " + addrL.ToString("X2"));

            if ((addrH == 0xFF) && (addrM == 0xFF) && (addrL == 0xFF))
            {
                Error("Error 5234 \"RANDOMISE not executed with command in-between addressed to other control gear\"");
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_3_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte x = 0, y = 0, z = 0;

            Byte[] R = new Byte[8];
            Byte[] S = new Byte[8];
            Byte[] T = new Byte[8];
            bool[] test = new bool[8];
            String[] errorText = new String[8];

            int r, i, iMax, k, kMax;

            errorText[1] = "No answer at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[2] = "No answer at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[3] = "No answer at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[4] = "Answer at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[5] = "Answer at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[6] = "Answer at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[7] = "Wrong answer at SEARCH ADDRESS = RANDOM ADDRESS";

            test[1] = true;
            test[2] = true;
            test[3] = true;
            test[4] = false;
            test[5] = false;
            test[6] = false;
            test[7] = true;

            Message("Starting test 12.5.3.1: COMPARE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            i = 0;
            iMax = 50;

            for (; i <= iMax; i++)
            {
                // 2x randomise
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xA7; // Command 0 addr
                outgoing[5] = 0x00; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xA7; // Command 1 addr
                outgoing[8] = 0x00; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                Info("Wait 100ms");
                Thread.Sleep(100);

                // broadcast query random address (h)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC2;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                x = incoming[3];

                // broadcast query random address (m)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC3;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                y = incoming[3];

                // broadcast query random address (l)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC4;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                z = incoming[3];

                Debug("\ti = " + i.ToString() + " Random address = " + x.ToString("X2") + " " + y.ToString("X2") + " " + z.ToString("X2"));

                if ((x != 0x00) && (x != 0xFF) && (y != 0x00) && (y != 0xFF) && (z != 0x00) && (z != 0xFF))
                {
                    break;
                }
            }

            if (i <= iMax)
            {
                k = 1;
                kMax = 7;

                R[1] = (Byte)(x + 1);
                R[2] = x;
                R[3] = x;
                R[4] = (Byte)(x - 1);
                R[5] = x;
                R[6] = x;
                R[7] = x;

                S[1] = y;
                S[2] = (Byte)(y + 1);
                S[3] = y;
                S[4] = y;
                S[5] = (Byte)(y - 1);
                S[6] = y;
                S[7] = y;

                T[1] = z;
                T[2] = z;
                T[3] = (Byte)(z + 1);
                T[4] = z;
                T[5] = z;
                T[6] = (Byte)(z - 1);
                T[7] = z;

                for (; k <= kMax; k++)
                {
                    // special command searchaddrh (R[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB1;
                    outgoing[3] = R[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrm (S[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB3;
                    outgoing[3] = S[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrl (T[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB5;
                    outgoing[3] = T[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command compare
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA9;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if ((incoming[2] != 0x00) != test[k])
                        {
                            Error("Error 5312," + k.ToString() + "\"" + errorText[k] + "\"");
                        }
                    }

                }
            }
            else
            {
                Error("Error 5311 \"Bad random generator\"");
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_3_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte x = 0, y = 0, z = 0;

            Byte[] R = new Byte[8];
            Byte[] S = new Byte[8];
            Byte[] T = new Byte[8];
            bool[] test = new bool[8];
            String[] errorText = new String[8];

            int r, i, iMax, k, kMax;

            errorText[1] = "WITHDRAW executed at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[2] = "WITHDRAW executed at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[3] = "WITHDRAW executed at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[4] = "WITHDRAW executed at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[5] = "WITHDRAW executed at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[6] = "WITHDRAW executed at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[7] = "WITHDRAW not executed at SEARCH ADDRESS = RANDOM ADDRESS";

            test[1] = true;
            test[2] = true;
            test[3] = true;
            test[4] = true;
            test[5] = true;
            test[6] = true;
            test[7] = false;

            Message("Starting test 12.5.3.2: WITHDRAW");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            i = 0;
            iMax = 50;

            for (; i <= iMax; i++)
            {
                // 2x randomise
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xA7; // Command 0 addr
                outgoing[5] = 0x00; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xA7; // Command 1 addr
                outgoing[8] = 0x00; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                Info("Wait 100ms");
                Thread.Sleep(100);

                // broadcast query random address (h)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC2;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                x = incoming[3];

                // broadcast query random address (m)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC3;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                y = incoming[3];

                // broadcast query random address (l)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC4;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                z = incoming[3];

                Debug("\ti = " + i.ToString() + " Random address = " + x.ToString("X2") + " " + y.ToString("X2") + " " + z.ToString("X2"));

                if ((x != 0x00) && (x != 0xFF) && (y != 0x00) && (y != 0xFF) && (z != 0x00) && (z != 0xFF))
                {
                    break;
                }
            }

            if (i <= iMax)
            {
                k = 1;
                kMax = 7;

                R[1] = (Byte)(x + 1);
                R[2] = x;
                R[3] = x;
                R[4] = (Byte)(x - 1);
                R[5] = x;
                R[6] = x;
                R[7] = x;

                S[1] = y;
                S[2] = (Byte)(y + 1);
                S[3] = y;
                S[4] = y;
                S[5] = (Byte)(y - 1);
                S[6] = y;
                S[7] = y;

                T[1] = z;
                T[2] = z;
                T[3] = (Byte)(z + 1);
                T[4] = z;
                T[5] = z;
                T[6] = (Byte)(z - 1);
                T[7] = z;

                for (; k <= kMax; k++)
                {

                    // terminate
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA1;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // 2x initialise(0)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xA5; // Command 0 addr
                    outgoing[5] = 0x00; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xA5; // Command 1 addr
                    outgoing[8] = 0x00; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    // special command searchaddrh (R[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB1;
                    outgoing[3] = R[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrm (S[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB3;
                    outgoing[3] = S[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrl (T[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB5;
                    outgoing[3] = T[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command withdraw
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xAB;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrh (x)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB1;
                    outgoing[3] = x;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrm (y)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB3;
                    outgoing[3] = y;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrl (z)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB5;
                    outgoing[3] = z;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command compare
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA9;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if ((incoming[2] != 0x00) != test[k])
                        {
                            Error("Error 5322," + k.ToString() + "\"" + errorText[k] + "\"");
                        }
                    }
                }

                // initialise
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA5;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r = getData(incoming, 100);

                // special command compare
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA9;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x00)
                    {
                        Error("Error 5323 \"INITIALISE restarts compare process\"");
                    }
                }
            }
            else
            {
                Error("Error 5321 \"Bad random generator\"");
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_4_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte x = 0, y = 0, z = 0;

            Byte a;

            Byte[] R = new Byte[8];
            Byte[] S = new Byte[8];
            Byte[] T = new Byte[8];
            bool[] test = new bool[8];
            String[] errorText = new String[8];

            int r, i, iMax, k, kMax;

            errorText[1] = "Command executed at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[2] = "Command executed at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[3] = "Command executed at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[4] = "Command executed at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[5] = "Command executed at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[6] = "Command executed at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[7] = "Command not executed at SEARCH ADDRESS = RANDOM ADDRESS";

            test[1] = false;
            test[2] = false;
            test[3] = false;
            test[4] = false;
            test[5] = false;
            test[6] = false;
            test[7] = true;

            Message("Starting test 12.5.4.1: PROGRAM SHORT ADDRESS");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            i = 0;
            iMax = 50;

            for (; i <= iMax; i++)
            {
                // 2x randomise
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xA7; // Command 0 addr
                outgoing[5] = 0x00; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xA7; // Command 1 addr
                outgoing[8] = 0x00; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                Info("Wait 100ms");
                Thread.Sleep(100);

                // broadcast query random address (h)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC2;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                x = incoming[3];

                // broadcast query random address (m)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC3;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                y = incoming[3];

                // broadcast query random address (l)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC4;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                z = incoming[3];

                Debug("\ti = " + i.ToString() + " Random address = " + x.ToString("X2") + " " + y.ToString("X2") + " " + z.ToString("X2"));

                if ((x != 0x00) && (x != 0xFF) && (y != 0x00) && (y != 0xFF) && (z != 0x00) && (z != 0xFF))
                {
                    break;
                }
            }

            if (i <= iMax)
            {
                k = 1;
                kMax = 7;

                R[1] = (Byte)(x + 1);
                R[2] = x;
                R[3] = x;
                R[4] = (Byte)(x - 1);
                R[5] = x;
                R[6] = x;
                R[7] = x;

                S[1] = y;
                S[2] = (Byte)(y + 1);
                S[3] = y;
                S[4] = y;
                S[5] = (Byte)(y - 1);
                S[6] = y;
                S[7] = y;

                T[1] = z;
                T[2] = z;
                T[3] = (Byte)(z + 1);
                T[4] = z;
                T[5] = z;
                T[6] = (Byte)(z - 1);
                T[7] = z;

                for (; k <= kMax; k++)
                {

                    // special command searchaddrh (R[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB1;
                    outgoing[3] = R[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrm (S[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB3;
                    outgoing[3] = S[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrl (T[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB5;
                    outgoing[3] = T[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    a = (Byte)(2 * k + 1);

                    // special command program short address (a)
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB7;
                    outgoing[3] = a;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // short address a query control gear
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = a;
                    outgoing[3] = 0x91;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if ((incoming[2] != 0x00) != test[k])
                        {
                            Error("Error 5412," + k.ToString() + " \"" + errorText[k] + "\"");
                        }
                    }
                }

                // dtr = 255
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 255;
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x broadcast store dtr as short addr
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x80; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x80; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

            }
            else
            {
                Error("Error 5411 \"Bad random generator\"");
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_4_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.4.2: VERIFY SHORT ADDRESS");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // dtr = 21
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 21;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // verify short address 5
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xB9;
            outgoing[3] = 5;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Error("Error 5421 \"Answer at VERIFY SHORT ADDRESS smaller than SHORT ADDRESS of DUT\"");
                }
            }

            // verify short address 31
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xB9;
            outgoing[3] = 31;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Error("Error 5422 \"Answer at SHORT ADDRESS of DUT smaller than VERIFY SHORT ADDRESS\"");
                }
            }

            // verify short address 21
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xB9;
            outgoing[3] = 21;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x01) || (incoming[3] != 0xFF))
                {
                    Error("Error 5423 \"No answer at SHORT ADDRESS of DUT equal to VERIFY SHORT ADDRESS\"");
                }
            }

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);


            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_4_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte x = 0, y = 0, z = 0;

            Byte[] R = new Byte[8];
            Byte[] S = new Byte[8];
            Byte[] T = new Byte[8];
            bool[] test = new bool[8];
            String[] errorText = new String[8];

            int r, i, iMax, k, kMax;

            errorText[1] = "Answer at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[2] = "Answer at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[3] = "Answer at SEARCH ADDRESS > RANDOM ADDRESS";
            errorText[4] = "Answer at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[5] = "Answer at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[6] = "Answer at SEARCH ADDRESS < RANDOM ADDRESS";
            errorText[7] = "No answer at SEARCH ADDRESS = RANDOM ADDRESS";

            test[1] = false;
            test[2] = false;
            test[3] = false;
            test[4] = false;
            test[5] = false;
            test[6] = false;
            test[7] = true; // check for addr == 27 (which includes formatting) if this is true

            Message("Starting test 12.5.4.3: QUERY SHORT ADDRESS");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // dtr = 27
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 27;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            i = 0;
            iMax = 50;

            for (; i <= iMax; i++)
            {
                // 2x randomise
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xA7; // Command 0 addr
                outgoing[5] = 0x00; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xA7; // Command 1 addr
                outgoing[8] = 0x00; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                Info("Wait 100ms");
                Thread.Sleep(100);

                // broadcast query random address (h)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC2;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                x = incoming[3];

                // broadcast query random address (m)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC3;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                y = incoming[3];

                // broadcast query random address (l)
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xC4;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                }
                z = incoming[3];

                Debug("\ti = " + i.ToString() + " Random address = " + x.ToString("X2") + " " + y.ToString("X2") + " " + z.ToString("X2"));

                if ((x != 0x00) && (x != 0xFF) && (y != 0x00) && (y != 0xFF) && (z != 0x00) && (z != 0xFF))
                {
                    break;
                }
            }

            if (i <= iMax)
            {
                k = 1;
                kMax = 7;

                R[1] = (Byte)(x + 1);
                R[2] = x;
                R[3] = x;
                R[4] = (Byte)(x - 1);
                R[5] = x;
                R[6] = x;
                R[7] = x;

                S[1] = y;
                S[2] = (Byte)(y + 1);
                S[3] = y;
                S[4] = y;
                S[5] = (Byte)(y - 1);
                S[6] = y;
                S[7] = y;

                T[1] = z;
                T[2] = z;
                T[3] = (Byte)(z + 1);
                T[4] = z;
                T[5] = z;
                T[6] = (Byte)(z - 1);
                T[7] = z;

                for (; k <= kMax; k++)
                {

                    // special command searchaddrh (R[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB1;
                    outgoing[3] = R[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrm (S[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB3;
                    outgoing[3] = S[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // special command searchaddrl (T[k])
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xB5;
                    outgoing[3] = T[k];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // short address a query short address
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xBB;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (((incoming[2] != 0x00) != test[k]) || (test[k] && (incoming[3] != 27)))
                        {
                            Error("Error 5432," + k.ToString() + " \"" + errorText[k] + "\"");
                        }
                    }
                }
            }
            else
            {
                Error("Error 5431 \"Bad random generator\"");
            }

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // short address a query short address
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xBB;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                {
                    Error("Error 5433 \"Answer is not 'MASK' if DUT has no Short Address\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_5_4_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.5.4.4: SEARCH ADDRESS: reset value");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x initialise(0)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xA5; // Command 0 addr
            outgoing[5] = 0x00; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xA5; // Command 1 addr
            outgoing[8] = 0x00; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // special command searchaddrh (16)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xB1;
            outgoing[3] = 16;
            sendData(outgoing);
            r = getData(incoming, 100);

            // special command searchaddrm (32)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xB3;
            outgoing[3] = 32;
            sendData(outgoing);
            r = getData(incoming, 100);

            // special command searchaddrl (64)
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xB5;
            outgoing[3] = 64;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // special command compare
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA9;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("Error 5441 \"No reset of the SEARCH ADDRESS\"");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_1_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.6.1.1: QUERY DEVICE TYPE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // broadcast query device type
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x99;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    Message("Device type: " + incoming[3].ToString());
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_1_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.6.1.2: QUERY LAMP FAILURE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            MessageBox.Show("Disconnect Lamp", "Required action");
            Message("Lamp disconnected");

            Info("Wait 5s");
            Thread.Sleep(5000);

            // broadcast query lamp failure
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x92;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                {
                    Error("Error 6121 \"LAMP FAILURE not detected\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                    Error("Error 6122 \"Lamp failure bit in answer of QUERY STATUS not set\"");
                }
                else
                {
                    if ((incoming[3] & 0x02) != 0x02) // check if status != xxxxxx1x
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx1x");
                        Error("Error 6122 \"Lamp failure bit in answer of QUERY STATUS not set\"");
                    }
                }
            }

            MessageBox.Show("Reconnect Lamp", "Required action");
            Message("Lamp reconnected");

            Info("Wait Preheat Time + Response Time = " + (preaheatTimeControl.Value + preaheatTimeControl.Value).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            // broadcast query lamp failure
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x92;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x00) && (incoming[3] == 0xFF))
                {
                    Error("Error 6123 \"End of LAMP FAILURE not detected\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                    Error("Error 6124 \"Lamp failure bit in answer of QUERY STATUS not cleared\"");
                }
                else
                {
                    if ((incoming[3] & 0x02) == 0x02) // check if status == xxxxxx1x
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx0x");
                        Error("Error 6124 \"Lamp failure bit in answer of QUERY STATUS not cleared\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_1_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.6.1.3: QUERY LAMP POWER ON");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            MessageBox.Show("Disconnect Lamp", "Required action");
            Message("Lamp disconnected");

            Info("Wait 5s");
            Thread.Sleep(5000);

            // broadcast query lamp power on
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x93;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x00) && (incoming[3] == 0xFF))
                {
                    Error("Error 6131 \"Query answered with YES during lamp failure\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04) // check if status == xxxxx1xx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxx0xx");
                        Error("Error 6132 \"Lamp power on bit in answer of QUERY STATUS is set\"");
                    }
                }
            }

            MessageBox.Show("Reconnect Lamp", "Required action");
            Message("Lamp reconnected");

            Info("Wait Preheat Time + Response Time = " + (preaheatTimeControl.Value + preaheatTimeControl.Value).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            // broadcast query lamp power on
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x93;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                {
                    Error("Error 6133 \"Query not answered with YES when lamp power is on\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                    Error("Error 6134 \"Lamp power on bit in answer of QUERY STATUS is not set\"");
                }
                else
                {
                    if ((incoming[3] & 0x04) != 0x04) // check if status != xxxxx1xx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxx1xx");
                        Error("Error 6134 \"Lamp power on bit in answer of QUERY STATUS is not set\"");
                    }
                }
            }

            // broadcast off
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // broadcast query lamp power on
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x93;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x00) && (incoming[3] == 0xFF))
                {
                    Error("Error 6135 \"Query answered with YES at DUT switched off\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                }
                else
                {
                    if ((incoming[3] & 0x04) == 0x04) // check if status == xxxxx1xx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxx0xx");
                        Error("Error 6136 \"Lamp power on bit set when DUT is switched off\"");
                    }
                }
            }

            // broadcast recall max level
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x05;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Preheat Time + Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_1_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] address = new Byte[5];
            Byte[] command = new Byte[5];

            bool[] answer1 = new bool[5];

            Byte[] answer2 = new Byte[5];

            String[] text1 = new String[5];
            String[] text2 = new String[5];
            String[] commandText = new String[5];

            int r, i, iMax;
            Byte PHM = 0;

            address[0] = 0xFE;
            command[0] = 0xFE;
            commandText[0] = "DAPC (254)";

            address[1] = 0xFF;
            command[1] = 0x05;
            commandText[1] = "RECALL MAX LEVEL";

            address[2] = 0xFE;
            command[2] = 0x01;
            commandText[2] = "DAPC (1)";

            address[3] = 0xFF;
            command[3] = 0x06;
            commandText[3] = "RECALL MIN LEVEL";

            address[4] = 0xFF;
            command[4] = 0x00;
            commandText[4] = "OFF";

            answer1[0] = true;
            answer1[1] = false;
            answer1[2] = true;
            answer1[3] = false;
            answer1[4] = false;

            answer2[0] = 0x08;
            answer2[1] = 0x00;
            answer2[2] = 0x08;
            answer2[3] = 0x00;
            answer2[4] = 0x00;

            text1[0] = "detected";
            text1[1] = "cleared";
            text1[2] = "detected";
            text1[3] = "cleared";
            text1[4] = "cleared";

            text2[0] = "set";
            text2[1] = "cleared";
            text2[2] = "set";
            text2[3] = "cleared";
            text2[4] = "cleared";

            Message("Starting test 12.6.1.4: QUERY LIMIT ERROR");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // broadcast query phm
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());


            if (PHM == 254)
            {
                i = 1;
                iMax = 4;
            }
            else
            {
                i = 0;
                iMax = 4;

                // dtr = phm + 1
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = (Byte)(PHM + 1);
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x store dtr as min level
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2B; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2B; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }


                // dtr = 253
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = 253;
                sendData(outgoing);
                r = getData(incoming, 100);

                // 2x store dtr as max level
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = 0x2A; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0x2A; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
            }

            // loop 1
            for (; i <= iMax; i++)
            {
                // send address[i] command[i]
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = address[i];
                outgoing[3] = command[i];
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // broadcast query limit error
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x94;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (((incoming[2] != 0x00) && (incoming[3] == 0xFF)) != answer1[i])
                    {
                        Error("Error 6141," + i.ToString() + " \"LIMIT ERROR not " + text1[i] + " at " + commandText[i] + "\"");
                    }
                }

                // broadcast query status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Debug("\tReceived no answer");
                    }
                    else
                    {
                        if ((incoming[3] & 0x08) != answer2[i]) // check if status == xxxx1xxx
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxTxxx");
                            Error("Error 6142," + i.ToString() + " \"Limit error bit not " + text2[i] + " at " + commandText[i] + "\"");
                        }
                    }
                }
            }


            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            Message("Test done");

        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_1_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.6.1.5: QUERY POWER FAILURE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            MessageBox.Show("Switch off mains power", "Required action");
            Message("Mains power switched off");

            Info("Wait 5s");
            Thread.Sleep(5000);

            MessageBox.Show("Switch on mains power", "Required action");
            Message("Mains power switched on");

            // flush input buffer
            while (getData(incoming, 100, 0) != 0)
                ;

            Info("Wait Preheat Time + Response Time + 500ms = " + (preaheatTimeControl.Value + preaheatTimeControl.Value + 500).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value + 500);

            // broadcast query power failure
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9B;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                {
                    Error("Error 6151 \"Power failure not detected\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                    Error("Error 6152 \"Power failure bit in answer of QUERY STATUS not set\"");
                }
                else
                {
                    if ((incoming[3] & 0x80) != 0x80) // check if status != 1xxxxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)1xxxxxxxx");
                        Error("Error 6152 \"Power failure bit in answer of QUERY STATUS not set\"");
                    }
                }
            }

            // broadcast recall min level
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x06;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            // broadcast query power failure
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9B;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] != 0x00) && (incoming[3] == 0xFF))
                {
                    Error("Error 6153 \"Power failure not cleared after receiving light level command\"");
                }
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                    Error("Error 6152 \"Power failure bit in answer of QUERY STATUS not set\"");
                }
                else
                {
                    if ((incoming[3] & 0x80) == 0x80) // check if status == 1xxxxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)0xxxxxxxx");
                        Error("Error 6154 \"Power failure bit not cleared after receiving light level command\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_1_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.6.1.6: QUERY STATUS: control gear ok");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Debug("\tReceived no answer");
                }
                else
                {
                    if ((incoming[3] & 0x01) == 0x01) // check if status == xxxxxxx1
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxxxx0");
                        Error("Error 6161 \"CONTROL GEAR not ok!\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_1_7()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.6.1.7: QUERY STATUS: fade running");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // dtr = 6
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 6;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as fade time
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x2E; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x2E; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // broadcast dapc = 0
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x04) != 0x04) // check if status != xxxxx1xx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx1xx");
                        Error("Error 6171 \"Load switched off immediately\"");
                    }
                    if ((incoming[3] & 0x10) != 0x10) // check if status != xxx1xxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxx1xxxxx");
                        Error("Error 6172 \"Fade running bit not set during fading\"");
                    }
                }
            }

            Info("Wait 5s");
            Thread.Sleep(5000);

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x10) == 0x10) // check if status == xxx1xxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxx0xxxxx");
                        Error("Error 6173 \"Fade running bit not cleared after fading\"");
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_2_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] offset = new Byte[7];
            Byte[] counterMax = new Byte[7];

            Byte command;

            int r, i, iMax, k, kMax;

            offset[0] = 10;
            offset[1] = 34;
            offset[2] = 48;
            offset[3] = 130;
            offset[4] = 158;
            offset[5] = 166;
            offset[6] = 198;

            counterMax[0] = 5;
            counterMax[1] = 7;
            counterMax[2] = 15;
            counterMax[3] = 13;
            counterMax[4] = 1;
            counterMax[5] = 9;
            counterMax[6] = 25;

            Message("Starting test 12.6.2.1: RESERVED: standard commands");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            i = 0;
            iMax = 6;

            for (; i <= iMax; i++)
            {
                k = 0;
                kMax = counterMax[i];
                for (; k <= kMax; k++)
                {
                    command = (Byte)(offset[i] + k);

                    // 2x broadcast command
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xFF; // Command 0 addr
                    outgoing[5] = command; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = command; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[3] != 0x00) || (incoming[6] != 0x00))
                    {
                        Error("Error 6211 \"Answer after receiving reserved command " + command.ToString() + "\"");
                    }

                    // broadcast query reset state
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x95;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                        {
                            Error("Error 6212 \"DUT not in reset state after receiving reserved command " + command.ToString() + "\"");

                            // 2x broadcast reset
                            outgoing[0] = 0x00;
                            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                            outgoing[2] = 0x02; // 2 commands
                            outgoing[3] = 0x00; // Command 0 flags
                            outgoing[4] = 0xFF; // Command 0 addr
                            outgoing[5] = 0x20; // Command 0 data
                            outgoing[6] = 0x00; // Command 1 flags
                            outgoing[7] = 0xFF; // Command 1 addr
                            outgoing[8] = 0x20; // Command 1 data
                            sendData(outgoing);
                            r = getData(incoming, 200); // more time allowed for the sequence to finish
                            if (r > 0)
                            {
                                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                                {
                                    Error("Wrong USB packet type received");
                                }
                                else if (incoming[2] != outgoing[2])
                                {
                                    Error("Command sequence error");
                                }
                            }

                            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                            Thread.Sleep((int)preaheatTimeControl.Value + 300);
                        }
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_2_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte command;

            int r, k, kMax;

            Message("Starting test 12.6.2.2: Application extended commands");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            k = 0;
            kMax = 31;

            for (; k <= kMax; k++)
            {
                command = (Byte)(224 + k);

                // 2x broadcast command
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xFF; // Command 0 addr
                outgoing[5] = command; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = command; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if ((incoming[3] != 0x00) || (incoming[6] != 0x00))
                {
                    Error("Error 6221 \"Answer after receiving APPLICATION EXTENDED COMMAND " + command.ToString() + "\"");
                }

                // broadcast query reset state
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x95;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                    {
                        Error("Error 6222 \"DUT not in reset state after receiving APPLICATION EXTENDED COMMAND " + command.ToString() + "\"");

                        // 2x broadcast reset
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                        outgoing[2] = 0x02; // 2 commands
                        outgoing[3] = 0x00; // Command 0 flags
                        outgoing[4] = 0xFF; // Command 0 addr
                        outgoing[5] = 0x20; // Command 0 data
                        outgoing[6] = 0x00; // Command 1 flags
                        outgoing[7] = 0xFF; // Command 1 addr
                        outgoing[8] = 0x20; // Command 1 data
                        sendData(outgoing);
                        r = getData(incoming, 200); // more time allowed for the sequence to finish
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if (incoming[2] != outgoing[2])
                            {
                                Error("Command sequence error");
                            }
                        }

                        Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                        Thread.Sleep((int)preaheatTimeControl.Value + 300);
                    }
                }
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_2_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] offset = new Byte[8];
            Byte[] counterMax = new Byte[8];
            Byte[] valueMax = new Byte[8];

            Byte command, address;

            int r, i, iMax, k, kMax, m, mMax;

            offset[0] = 173;
            offset[1] = 191;
            offset[2] = 201;
            offset[3] = 209;
            offset[4] = 225;
            offset[5] = 241;
            offset[6] = 249;
            offset[7] = 253;

            counterMax[0] = 1;
            counterMax[1] = 0;
            counterMax[2] = 3;
            counterMax[3] = 7;
            counterMax[4] = 7;
            counterMax[5] = 3;
            counterMax[6] = 1;
            counterMax[7] = 0;

            valueMax[0] = 0;
            valueMax[1] = 255;
            valueMax[2] = 255;
            valueMax[3] = 255;
            valueMax[4] = 255;
            valueMax[5] = 255;
            valueMax[6] = 255;
            valueMax[7] = 255;


            Message("Starting test 12.6.2.3: RESERVED: special commands 1");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            i = 0;
            iMax = 7;

            for (; i <= iMax; i++)
            {
                k = 0;
                kMax = counterMax[i];
                for (; k <= kMax; k++)
                {
                    address = (Byte)(offset[i] + 2 * k);

                    m = 0;
                    mMax = valueMax[i];

                    for (; m <= mMax; m++)
                    {
                        command = (Byte)m;

                        // 2x address command
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                        outgoing[2] = 0x02; // 2 commands
                        outgoing[3] = 0x00; // Command 0 flags
                        outgoing[4] = address; // Command 0 addr
                        outgoing[5] = command; // Command 0 data
                        outgoing[6] = 0x00; // Command 1 flags
                        outgoing[7] = address; // Command 1 addr
                        outgoing[8] = command; // Command 1 data
                        sendData(outgoing);
                        r = getData(incoming, 200); // more time allowed for the sequence to finish
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if (incoming[2] != outgoing[2])
                            {
                                Error("Command sequence error");
                            }
                        }

                        if ((incoming[3] != 0x00) || (incoming[6] != 0x00))
                        {
                            Error("Error 6231," + i.ToString() + "," + k.ToString() + "," + m.ToString() + " \"Answer after receiving reserved command\"");
                        }

                        // broadcast query reset state
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                        outgoing[2] = 0xFF;
                        outgoing[3] = 0x95;
                        sendData(outgoing);
                        r = getData(incoming, 100);
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                            {
                                Error("Error 6232," + i.ToString() + "," + k.ToString() + "," + m.ToString() + " \"DUT not in reset state after receiving reserved command\"");

                                // 2x broadcast reset
                                outgoing[0] = 0x00;
                                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                                outgoing[2] = 0x02; // 2 commands
                                outgoing[3] = 0x00; // Command 0 flags
                                outgoing[4] = 0xFF; // Command 0 addr
                                outgoing[5] = 0x20; // Command 0 data
                                outgoing[6] = 0x00; // Command 1 flags
                                outgoing[7] = 0xFF; // Command 1 addr
                                outgoing[8] = 0x20; // Command 1 data
                                sendData(outgoing);
                                r = getData(incoming, 200); // more time allowed for the sequence to finish
                                if (r > 0)
                                {
                                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                                    {
                                        Error("Wrong USB packet type received");
                                    }
                                    else if (incoming[2] != outgoing[2])
                                    {
                                        Error("Command sequence error");
                                    }
                                }

                                Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                                Thread.Sleep((int)preaheatTimeControl.Value + 300);
                            }
                        }
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_2_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] offset = new Byte[6];
            Byte[] factor = new Byte[6];
            Byte[] counterKmax = new Byte[6];
            Byte[] counterNmax = new Byte[6];

            Byte command, address;

            int r, i, iMax, k, kMax, m, mMax;

            // change ref 23
            // this whole test is implemented by some guesswork, as the logic diagram
            // doesn't have the same variables as the table defines

            offset[0] = 160;
            offset[1] = 192;
            offset[2] = 224;
            offset[3] = 240;
            offset[4] = 248;
            offset[5] = 252;

            factor[0] = 15;
            factor[1] = 15;
            factor[2] = 7;
            factor[3] = 3;
            factor[4] = 1;
            factor[5] = 0;

            counterKmax[0] = 255;
            counterKmax[1] = 255;
            counterKmax[2] = 255;
            counterKmax[3] = 255;
            counterKmax[4] = 255;
            counterKmax[5] = 255;

            counterNmax[0] = 0;
            counterNmax[1] = 1;
            counterNmax[2] = 2;
            counterNmax[3] = 3;
            counterNmax[4] = 4;
            counterNmax[5] = 5;


            Message("Starting test 12.6.2.4: RESERVED: special commands 2");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            i = 0;
            iMax = 5;

            for (; i <= iMax; i++)
            {
                k = 0;
                kMax = factor[i];
                for (; k <= kMax; k++)
                {
                    address = (Byte)(offset[i] + 2 * k);

                    m = 0;
                    mMax = counterKmax[i];

                    for (; m <= mMax; m++)
                    {
                        command = (Byte)m;

                        // 2x address command
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                        outgoing[2] = 0x02; // 2 commands
                        outgoing[3] = 0x00; // Command 0 flags
                        outgoing[4] = address; // Command 0 addr
                        outgoing[5] = command; // Command 0 data
                        outgoing[6] = 0x00; // Command 1 flags
                        outgoing[7] = address; // Command 1 addr
                        outgoing[8] = command; // Command 1 data
                        sendData(outgoing);
                        r = getData(incoming, 200); // more time allowed for the sequence to finish
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if (incoming[2] != outgoing[2])
                            {
                                Error("Command sequence error");
                            }
                        }

                        if ((incoming[3] != 0x00) || (incoming[6] != 0x00))
                        {
                            Error("Error 6241," + i.ToString() + "," + k.ToString() + "," + m.ToString() + " \"Answer after receiving reserved command\"");
                        }

                        // broadcast query reset state
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                        outgoing[2] = 0xFF;
                        outgoing[3] = 0x95;
                        sendData(outgoing);
                        r = getData(incoming, 100);
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                            {
                                Error("Error 6242," + i.ToString() + "," + k.ToString() + "," + m.ToString() + " \"DUT not in reset state after receiving reserved command\"");

                                // 2x broadcast reset
                                outgoing[0] = 0x00;
                                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                                outgoing[2] = 0x02; // 2 commands
                                outgoing[3] = 0x00; // Command 0 flags
                                outgoing[4] = 0xFF; // Command 0 addr
                                outgoing[5] = 0x20; // Command 0 data
                                outgoing[6] = 0x00; // Command 1 flags
                                outgoing[7] = 0xFF; // Command 1 addr
                                outgoing[8] = 0x20; // Command 1 data
                                sendData(outgoing);
                                r = getData(incoming, 200); // more time allowed for the sequence to finish
                                if (r > 0)
                                {
                                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                                    {
                                        Error("Wrong USB packet type received");
                                    }
                                    else if (incoming[2] != outgoing[2])
                                    {
                                        Error("Command sequence error");
                                    }
                                }

                                Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                                Thread.Sleep((int)preaheatTimeControl.Value + 300);
                            }
                        }
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-102
        public void RunTest_12_6_2_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte command;

            int r, i, iMax, k, kMax;

            Message("Starting test 12.6.2.5: Not supported device types");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            i = 0;
            iMax = 254;

            for (; i <= iMax; i++)
            {

                // enable device type i, query extended version number
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = (Byte)i; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xFF; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] != 0x00)
                {
                    Message("DEVICE TYPE (" + i.ToString() + ") is supported");
                }
                else
                {

                    k = 0;
                    kMax = 30;
                    for (; k <= kMax; k++)
                    {
                        command = (Byte)(224 + k);

                        // enable device type i, 2x broadcast command
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                        outgoing[2] = 0x03; // 3 commands
                        outgoing[3] = 0x00; // Command 0 flags
                        outgoing[4] = 0xC1; // Command 0 addr
                        outgoing[5] = (Byte)i; // Command 0 data
                        outgoing[6] = 0x00; // Command 1 flags
                        outgoing[7] = 0xFF; // Command 1 addr
                        outgoing[8] = command; // Command 1 data
                        outgoing[9] = 0x00; // Command 2 flags
                        outgoing[10] = 0xFF; // Command 2 addr
                        outgoing[11] = command; // Command 2 data
                        sendData(outgoing);
                        r = getData(incoming, 200); // more time allowed for the sequence to finish
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if (incoming[2] != outgoing[2])
                            {
                                Error("Command sequence error");
                            }
                        }

                        if ((incoming[3] != 0x00) || (incoming[6] != 0x00) || (incoming[9] != 0x00))
                        {
                            Error("Error 6251," + i.ToString() + "," + k.ToString() + " \"Answer after receiving APPLICATION EXTENDED COMMAND " + command.ToString() + "\"");
                        }

                        // broadcast query reset state
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                        outgoing[2] = 0xFF;
                        outgoing[3] = 0x95;
                        sendData(outgoing);
                        r = getData(incoming, 100);
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                            {
                                Error("Error 6252," + i.ToString() + "," + k.ToString() + " \"DUT not in reset state after receiving APPLICATION EXTENDED COMMAND " + command.ToString() + "\"");

                                // 2x broadcast reset
                                outgoing[0] = 0x00;
                                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                                outgoing[2] = 0x02; // 2 commands
                                outgoing[3] = 0x00; // Command 0 flags
                                outgoing[4] = 0xFF; // Command 0 addr
                                outgoing[5] = 0x20; // Command 0 data
                                outgoing[6] = 0x00; // Command 1 flags
                                outgoing[7] = 0xFF; // Command 1 addr
                                outgoing[8] = 0x20; // Command 1 data
                                sendData(outgoing);
                                r = getData(incoming, 200); // more time allowed for the sequence to finish
                                if (r > 0)
                                {
                                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                                    {
                                        Error("Wrong USB packet type received");
                                    }
                                    else if (incoming[2] != outgoing[2])
                                    {
                                        Error("Command sequence error");
                                    }
                                }

                                Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                                Thread.Sleep((int)preaheatTimeControl.Value + 300);
                            }
                        }
                    }
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting Device Type 6 test 12.7.1.1: QUERY FEATURES");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if (incoming[6] == 0x00)
            {
                Error("Error 7111 \"QUERY FEATURES not answered for device type 6\"");
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.1.2: QUERY SHORT CIRCUIT");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x01) != 0x01))
            {
                Message("\"QUERY SHORT CIRCUIT FAILURE not supported by DUT \"");
            }
            else
            {
                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x01) != 0x00)
                {
                    Error("Error 7121 \"QUERY FAILURE STATUS answered wrong without short circuit\"");
                }

                MessageBox.Show("Apply short circuit to output", "Required action");
                Message("Short circuit applied");

                Info("Wait 5s");
                Thread.Sleep(5000);

                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x01) == 0x00) // xxxxxxx1 required
                {
                    Error("Error 7122 \"QUERY FAILURE STATUS answered wrong with short circuit\"");
                }


                // broadcast query status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[3] & 0x06) != 0x02) // xxxxx01x required
                    {
                        Error("Error 7123 \"QUERY STATUS answered wrong with short circuit\"");
                    }
                }

                // broadcast query lamp failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x92;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("Error 7124 \"QUERY LAMP FAILURE not answered with short circuit applied\"");
                    }
                }


                // broadcast query lamp power on
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x93;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x00)
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 0, 0");
                        Error("Error 7125 \"QUERY LAMP POWER ON answered wrong with short circuit applied\"");
                    }
                }

                // broadcast query actual level
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 255)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 255");
                            Error("Error 7126 \"QUERY ACTUAL LEVEL answered wrong with short circuit applied\"");
                        }
                    }
                }

                // enable device type 6, broadcast query short circuit failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF2; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] == 0x00)
                {
                    Error("Error 7127 \"QUERY SHORT CIRCUIT FAILURE not answered\"");
                }

                MessageBox.Show("Remove short circuit at output", "Required action");
                Message("Short circuit removed");

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.1.3: QUERY OPEN CIRCUIT");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x02) != 0x02))
            {
                Message("\"QUERY OPEN CIRCUIT FAILURE not supported by DUT \"");
            }
            else
            {
                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x02) != 0x00)
                {
                    Error("Error 7131 \"QUERY FAILURE STATUS answered wrong without open circuit\"");
                }

                MessageBox.Show("Disconnect Lamp(s)", "Required action");
                Message("Lamp(s) disconnected");

                Info("Wait 5s");
                Thread.Sleep(5000);

                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x02) != 0x02) // xxxxxx1x required
                {
                    Error("Error 7132 \"QUERY FAILURE STATUS answered wrong with open circuit\"");
                }

                // broadcast query status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if ((incoming[3] & 0x06) != 0x02) // check if status != xxxxx01x
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx01x");
                            Error("Error 7133 \"QUERY STATUS answered wrong with open circuit\"");
                        }
                    }
                }

                // broadcast query lamp failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x92;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("Error 7134 \"QUERY LAMP FAILURE not answered with open circuit applied\"");
                    }
                }

                // broadcast query lamp power on
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x93;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x00)
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 0, 0");
                        Error("Error 7135 \"QUERY LAMP POWER ON answered wrong with open circuit applied\"");
                    }
                }

                // broadcast query actual level
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 255)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 255");
                            Error("Error 7136 \"QUERY ACTUAL LEVEL answered wrong with open circuit applied\"");
                        }
                    }
                }

                // enable device type 6, broadcast query open circuit failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF3; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] == 0x00)
                {
                    Error("Error 7137 \"QUERY OPEN CIRCUIT FAILURE not answered\"");
                }

                MessageBox.Show("Reconnect Lamp(s)", "Required action");
                Message("Lamp(s) reconnected");

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;
            long t0, time;

            Message("Starting test 12.7.1.4: QUERY LOAD DECREASE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x04) != 0x04))
            {
                Message("\"QUERY LOAD DECREASE FAILURE not supported by DUT\"");
            }
            else
            {

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 1 flags
                outgoing[10] = 0xFF; // Command 1 addr
                outgoing[11] = 0xE0; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                t0 = DateTime.Now.Ticks;
                time = t0;

                do
                {
                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                    time = DateTime.Now.Ticks;
                } while (((time - t0) / 1e7) < 17 * 60);

                if (((time - t0) / 1e7) < 17 * 60)
                {

                    // enable device type 6, broadcast query load decrease failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF4; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] != 0x00)
                    {
                        Error("Error 7142 \"QUERY LOAD DECREASE FAILURE answered without load decrease\"");
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x04) == 0x04) // xxxxx0xx required
                    {
                        Error("Error 7143 \"QUERY FAILURE STATUS answered wrong without load decrease\"");
                    }

                    MessageBox.Show("Disconnect part of the load according to manufacturer", "Required action");
                    Message("Load decreased");

                    // broadcast query status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x90;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if ((incoming[3] & 0x06) != 0x02) // check if status != xxxxx01x
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx01x");
                                Error("Error 7144 \"QUERY STATUS answered wrong at load decrease\"");
                            }
                        }
                    }

                    // broadcast query lamp failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x92;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("Error 7145 \"QUERY LAMP FAILURE not answered at load decrease\"");
                        }
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x04) != 0x04) // xxxxx1xx required
                    {
                        Error("Error 7146 \"QUERY FAILURE STATUS answered wrong at load decrease\"");
                    }


                    // enable device type 6, broadcast query load decrease failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF4; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if (incoming[6] == 0x00)
                    {
                        Error("Error 7147 \"QUERY LOAD DECREASE FAILURE not answered\"");
                    }

                    MessageBox.Show("Reconnect complete load", "Required action");
                    Message("Complete load reconnected");

                }
                else
                {
                    Error("Error 7141 Timeout");
                }

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;
            long t0, time;

            Message("Starting test 12.7.1.5: QUERY LOAD INCREASE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x08) != 0x08))
            {
                Message("\"QUERY LOAD INCREASE FAILURE not supported by DUT\"");
            }
            else
            {

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 1 flags
                outgoing[10] = 0xFF; // Command 1 addr
                outgoing[11] = 0xE0; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                t0 = DateTime.Now.Ticks;
                time = t0;

                do
                {
                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                    time = DateTime.Now.Ticks;
                } while (((time - t0) / 1e7) < 17 * 60);

                if (((time - t0) / 1e7) < 17 * 60)
                {

                    // enable device type 6, broadcast query load increase failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF5; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] != 0x00)
                    {
                        Error("Error 7152 \"QUERY LOAD INCREASE FAILURE answered without load increase\"");
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x08) == 0x08) // xxxx0xxx required
                    {
                        Error("Error 7153 \"QUERY FAILURE STATUS answered wrong without load increase\"");
                    }

                    MessageBox.Show("Connect additional load according to manufacturer", "Required action");
                    Message("Load increased");

                    // broadcast query status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x90;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if ((incoming[3] & 0x06) != 0x02) // check if status != xxxxx01x
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx01x");
                                Error("Error 7154 \"QUERY STATUS answered wrong at load increase\"");
                            }
                        }
                    }

                    // broadcast query lamp failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x92;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("Error 7155 \"QUERY LAMP FAILURE not answered at load increase\"");
                        }
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x08) != 0x08) // xxxx1xxx required
                    {
                        Error("Error 7156 \"QUERY FAILURE STATUS answered wrong at load increase\"");
                    }


                    // enable device type 6, broadcast query load increase failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF5; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if (incoming[6] == 0x00)
                    {
                        Error("Error 7157 \"QUERY LOAD INCREASE FAILURE not answered\"");
                    }

                    MessageBox.Show("Disconnect additional load", "Required action");
                    Message("Additional load disconnected");

                }
                else
                {
                    Error("Error 7151 Timeout");
                }

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;
            long t0, time;

            Message("Starting test 12.7.1.6: QUERY CURRENT PROTECTOR ACTIVE: Underload");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x10) != 0x10))
            {
                Message("\"QUERY CURRENT PROTECTOR FAILURE not supported by DUT\"");
            }
            else
            {

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 1 flags
                outgoing[10] = 0xFF; // Command 1 addr
                outgoing[11] = 0xE0; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                t0 = DateTime.Now.Ticks;
                time = t0;

                do
                {
                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                    time = DateTime.Now.Ticks;
                } while (((time - t0) / 1e7) < 17 * 60);

                if (((time - t0) / 1e7) < 17 * 60)
                {

                    // enable device type 6, broadcast query current protector failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF6; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] != 0x00)
                    {
                        Error("Error 7162 \"QUERY CURRENT PROTECTOR FAILURE answered with nominal load\"");
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x10) == 0x10) // xxx0xxxx required
                    {
                        Error("Error 7163 \"QUERY FAILURE STATUS answered wrong with nominal load\"");
                    }

                    MessageBox.Show("Disconnect part of the load according to manufacturer", "Required action");
                    Message("Part of the load disconnected");

                    // broadcast query status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x90;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if ((incoming[3] & 0x06) != 0x02) // check if status != xxxxx01x
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx01x");
                                Error("Error 7164 \"QUERY STATUS answered wrong with underload\"");
                            }
                        }
                    }

                    // broadcast query lamp failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x92;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("Error 7165 \"QUERY LAMP FAILURE not answered with underload\"");
                        }
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x10) != 0x10) // xxx1xxxx required
                    {
                        Error("Error 7166 \"QUERY FAILURE STATUS answered wrong with underload\"");
                    }


                    // enable device type 6, broadcast query current protector failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF6; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if (incoming[6] == 0x00)
                    {
                        Error("Error 7167 \"QUERY CURRENT PROTECTOR FAILURE not answered\"");
                    }

                    // broadcast query actual level
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0xA0;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if (incoming[3] != 255)
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 255");
                                Error("Error 7168 \"QUERY ACTUAL LEVEL answered wrong with underload\"");
                            }
                        }
                    }

                    MessageBox.Show("Reconnect complete load", "Required action");
                    Message("Complete load reconnected");

                }
                else
                {
                    Error("Error 7161 Timeout");
                }

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_7()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;
            long t0, time;

            Message("Starting test 12.7.1.7: QUERY CURRENT PROTECTOR ACTIVE: Overload");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x10) != 0x10))
            {
                Message("\"QUERY CURRENT PROTECTOR FAILURE not supported by DUT\"");
            }
            else
            {

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 1 flags
                outgoing[10] = 0xFF; // Command 1 addr
                outgoing[11] = 0xE0; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                t0 = DateTime.Now.Ticks;
                time = t0;

                do
                {
                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                    time = DateTime.Now.Ticks;
                } while (((time - t0) / 1e7) < 17 * 60);

                if (((time - t0) / 1e7) < 17 * 60)
                {

                    // enable device type 6, broadcast query current protector failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF6; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] != 0x00)
                    {
                        Error("Error 7172 \"QUERY CURRENT PROTECTOR FAILURE answered with nominal load\"");
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x10) == 0x10) // xxx0xxxx required
                    {
                        Error("Error 7173 \"QUERY FAILURE STATUS answered wrong with nominal load\"");
                    }

                    MessageBox.Show("Connect additional load according to manufacturer", "Required action");
                    Message("Additional load connected");

                    // broadcast query status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x90;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if ((incoming[3] & 0x06) != 0x02) // check if status != xxxxx01x
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx01x");
                                Error("Error 7174 \"QUERY STATUS answered wrong with overload\"");
                            }
                        }
                    }

                    // broadcast query lamp failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x92;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Error("Error 7175 \"QUERY LAMP FAILURE not answered with overload\"");
                        }
                    }

                    // enable device type 6, broadcast query failure status
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF1; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else if ((incoming[8] & 0x10) != 0x10) // xxx1xxxx required
                    {
                        Error("Error 7176 \"QUERY FAILURE STATUS answered wrong with overload\"");
                    }


                    // enable device type 6, broadcast query current protector failure
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF6; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if (incoming[6] == 0x00)
                    {
                        Error("Error 7177 \"QUERY CURRENT PROTECTOR FAILURE not answered\"");
                    }

                    // broadcast query actual level
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0xA0;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if (incoming[3] != 255)
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 255");
                                Error("Error 7178 \"QUERY ACTUAL LEVEL answered wrong with overload\"");
                            }
                        }
                    }

                    MessageBox.Show("Disconnect additional load", "Required action");
                    Message("Additional load disconnected");

                }
                else
                {
                    Error("Error 7171 Timeout");
                }

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_8()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.1.8: QUERY THERMAL SHUT DOWN");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x20) != 0x20))
            {
                Message("\"QUERY THERMAL SHUT DOWN FAILURE not supported by DUT\"");
            }
            else
            {
                // enable device type 6, broadcast query thermal shutdown failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF7; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] != 0x00)
                {
                    Error("Error 7181 \"QUERY THERMAL SHUT DOWN FAILURE answered without thermal shut down\"");
                }

                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x20) == 0x20) // xx0xxxxx required
                {
                    Error("Error 7182 \"QUERY FAILURE STATUS answered wrong without thermal shut down\"");
                }

                MessageBox.Show("Heat up DUT according to manufacturer", "Required action");
                Message("Thermal failure simulated");

                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x20) != 0x20) // xx1xxxxx required
                {
                    Error("Error 7183 \"QUERY FAILURE STATUS answered wrong at thermal shut down\"");
                }

                // broadcast query status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x90;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] == 0x00)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if ((incoming[3] & 0x06) != 0x00) // check if status != xxxxx00x
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xxxxxx00x");
                            Error("Error 7184 \"QUERY STATUS answered wrong at thermal shut down\"");
                        }
                    }
                }

                // broadcast query lamp failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x92;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x00)
                    {
                        Error("Error 7185 \"QUERY LAMP FAILURE answered at thermal shut down\"");
                    }
                }

                // broadcast query lamp power on
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x93;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x00)
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 0, 0");
                        Error("Error 7186 \"QUERY LAMP POWER ON answered wrong at thermal shut down\"");
                    }
                }

                // broadcast query actual level
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 255)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 255");
                            Error("Error 7187 \"QUERY ACTUAL LEVEL answered wrong at thermal shut down\"");
                        }
                    }
                }

                // enable device type 6, broadcast query thermal shutdown failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF7; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("Error 7188 \"QUERY THERMAL SHUT DOWN FAILURE not answered\"");
                }

                MessageBox.Show("Cool down DUT to normal operating temperature", "Required action");
                Message("DUT back at normal operating temperature");

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_9()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.1.9: QUERY THERMAL OVERLOAD");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x40) != 0x40))
            {
                Message("\"QUERY THERMAL OVERLOAD FAILURE not supported by DUT\"");
            }
            else
            {
                // enable device type 6, broadcast query thermal overload failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF8; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] != 0x00)
                {
                    Error("Error 7191 \"QUERY THERMAL OVERLOAD FAILURE answered without over temperature\"");
                }

                // change ref 9
                // the symbol used in the schematic is that for "send twice to device type" but the command doesn't need
                // resending. Furthermore, other "send twice to device type" symbols include the words "send twice to device type"
                // which are not present here.

                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x40) == 0x40) // x0xxxxxx required
                {
                    Error("Error 7192 \"QUERY FAILURE STATUS answered wrong at normal temperature\"");
                }

                MessageBox.Show("Heat up DUT according to manufacturer", "Required action");
                Message("Thermal failure simulated");

                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x40) != 0x40) // x1xxxxxx required
                {
                    Error("Error 7193 \"QUERY FAILURE STATUS answered wrong at over temperature\"");
                }

                // broadcast query actual level
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != 255)
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 255");
                            Error("Error 7194 \"QUERY ACTUAL LEVEL answered wrong at over temperature\"");
                        }
                    }
                }

                // enable device type 6, broadcast query thermal overload failure
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF8; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    // change ref 10
                    // original code: Error("Error 7195,i \"QUERY THERMAL OVERLOAD FAILURE not answered\"");
                    Error("Error 7195 \"QUERY THERMAL OVERLOAD FAILURE not answered\"");
                }

                MessageBox.Show("Cool down DUT to normal operating temperature", "Required action");
                Message("DUT back at normal operating temperature");

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_1_10()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.1.10: Query control gear information");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // broadcast query device type
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x99;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    Message("Device type: " + incoming[3].ToString());
                }
            }


            // enable device type 6, broadcast query gear type
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xED; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if (incoming[6] == 0x00)
            {
                Message("Error 71101 \"No answer to QUERY GEAR TYPE\"");
            }

            Message("\"Answer to QUERY GEAR TYPE is " + incoming[8].ToString() + ".\"");


            // enable device type 6, broadcast query possible operating modes
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xEF; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if (incoming[6] == 0x00)
            {
                Error("Error 71102 \"No answer to QUERY POSSIBLE OPERATING MODES\"");
            }

            Message("Message: \"Answer to QUERY POSSIBLE OPERATING MODES is " + incoming[8].ToString() + ".\"");

            // enable device type 6, broadcast query operating mode
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xFC; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if (incoming[6] == 0x00)
            {
                Error("Error 71103 \"No answer to QUERY OPERATING MODE\"");
            }

            Message("\"Answer to QUERY OPERATING MODE is " + incoming[8].ToString() + ".\"");

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.2.1: REFERENCE SYSTEM POWER");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x1C) == 0x00))
            {
                Message("\"REFERENCE SYSTEM POWER not supported by DUT\"");
            }
            else
            {
                // enable device type 6, broadcast query reference running
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF9; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] != 0x00)
                {
                    Error("Error 7211 \"QUERY REFERENCE RUNNING answered when no measurement started\"");
                }

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xE0; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // enable device type 6, broadcast query reference running
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF9; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] == 0x00)
                {
                    Error("Error 7212 \"REFERENCE SYSTEM POWER not started\"");
                }

                // terminate
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA1;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r = getData(incoming, 100);
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.2.2: REFERENCE SYSTEM POWER: 100 ms-timeout");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x1C) == 0x00))
            {
                Message("\"REFERENCE SYSTEM POWER not supported by DUT\"");
            }
            else
            {
                // enable device type 6, broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                Info("Wait 150ms");
                Thread.Sleep(150);

                // broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xE0;
                sendData(outgoing);
                r = getData(incoming, 100);

                Info("Wait 150ms");
                Thread.Sleep(150);

                // enable device type 6, broadcast query reference running
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF9; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] != 0x00)
                {
                    Error("Error 7221 \"Command executed with timeout\"");
                }

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xE0; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // terminate
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA1;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r = getData(incoming, 100);

                // enable device type 6, broadcast query reference running
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF9; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] != 0x00)
                {
                    Error("Error 7222 \"TERMINATE does not stop reference measurement\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] address = new Byte[5];
            bool[] answer = new bool[5];
            String[] errorText = new String[5];

            int r, i, iMax;

            address[0] = 0x03; // Short address 1
            address[1] = 0x83; // Group address 1 100A AAA1 == 1000 0011
            address[2] = 0xFF; // Broadcast
            address[3] = 0x05; // Short address 2
            address[4] = 0x85; // Group address 2 100A AAA1 == 1000 0101

            answer[0] = false;
            answer[1] = false;
            answer[2] = false;
            answer[3] = true;
            answer[4] = true;

            errorText[0] = "executed";
            errorText[1] = "executed";
            errorText[2] = "executed";
            errorText[3] = "not executed";
            errorText[4] = "not executed";

            Message("Starting test 12.7.2.3: REFERENCE SYSTEM POWER: Command in-between");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x broadcast add to group 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x61; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x61; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // dtr = 3
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x1C) == 0x00))
            {
                Message("\"REFERENCE SYSTEM POWER not supported by DUT\"");
            }
            else
            {

                i = 0;
                iMax = 4;

                for (; i <= iMax; i++)
                {
                    // enable device type 6, broadcast reference system power, address i off, broadcast reference system power
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x04; // 4 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xE0; // Command 1 data
                    outgoing[9] = 0x00; // Command 2 flags
                    outgoing[10] = address[i]; // Command 2 addr
                    outgoing[11] = 0x00; // Command 2 data
                    outgoing[12] = 0x00; // Command 3 flags
                    outgoing[13] = 0xFF; // Command 3 addr
                    outgoing[14] = 0xE0; // Command 3 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    Info("Wait 150ms");
                    Thread.Sleep(150);

                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] != 0x00) != answer[i])
                    {
                        Error("Error 7231," + i.ToString() + " \"Command 'REFERENCE SYSTEM POWER " + errorText[i] + "\"");
                    }

                    // broadcast query actual level
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0xA0;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if (i < 3)
                            {
                                if (incoming[3] != 254)
                                {
                                    Debug("\tReceived answer = " + incoming[3].ToString() + " expected = 254");
                                    Error("Error 7232," + i.ToString() + " \"Command in-between executed\"");
                                }
                            }
                            else
                            {
                                if (incoming[3] == 0)
                                {
                                    Debug("\tReceived answer = " + incoming[3].ToString() + " expected != 0");
                                    Error("Error 7232," + i.ToString() + " \"Command in-between executed\"");
                                }
                            }
                        }
                    }

                    // terminate
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA1;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // recall max level
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x05;
                    sendData(outgoing);
                    r = getData(incoming, 100);


                    // change ref 11
                    Info("Wait Preheat Time + Response Time");
                    Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);
                }

            }

            // 2x broadcast remove from group 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x71; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x71; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_4()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;
            long t0, time;

            Message("Starting test 12.7.2.4: REFERENCE SYSTEM POWER: 15 minutes timer");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x1C) == 0x00))
            {
                Message("\"REFERENCE SYSTEM POWER not supported by DUT\"");
            }
            else
            {
                t0 = DateTime.Now.Ticks;
                time = t0;

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xE0; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                do
                {
                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                    time = DateTime.Now.Ticks;
                } while (((time - t0) / 1e7) < 17 * 60);

                if (((time - t0) / 1e7) < 17 * 60)
                {
                    Message("\"Measurement finished after " + ((time - t0) / 1e7).ToString() + "s\"");
                }
                else
                {
                    Error("Error 7241: \"Measurement not finished after 15 min + 10%\"");

                    // terminate
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA1;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_5()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.2.5: REFERENCE SYSTEM POWER: failed");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x1C) == 0x00))
            {
                Message("\"REFERENCE SYSTEM POWER not supported by DUT\"");
            }
            else
            {
                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xE0; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                DialogResult res = DialogResult.OK;

                do
                {
                    res = MessageBox.Show("Disturb measurement according to manufacturer", "Required action", MessageBoxButtons.OKCancel);

                    if (res == DialogResult.OK)
                    {
                        Message("Measurement disturbed");
                    }

                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                } while (res == DialogResult.OK);

                if (res == DialogResult.Cancel)
                {
                    Message("Test cancelled");
                    return;
                }

                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x80) != 0x80) // 1xxxxxxx required
                {
                    Error("Error 7251 \"QUERY FAILURE STATUS answered wrong with reference measurement failed\"");
                }

                // enable device type 6, broadcast query reference measurement failed
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xFA; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("Error 7252 \"QUERY REFERENCE MEASUREMENT FAILED not answered\"");
                }

                MessageBox.Show("Set DUT to normal operation", "Required action");
                Message("DUT set to normal operation");

                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xE0; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                do
                {
                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                } while (true);


                // change ref 12
                // enable device type 6, broadcast query failure status
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF1; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else if ((incoming[8] & 0x80) == 0x80) // 0xxxxxxx required
                {
                    Error("Error 7253 \"QUERY FAILURE STATUS answered wrong with successful measurement\"");
                }


                // enable device type 6, broadcast query reference measurement failed
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xFA; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] != 0x00)
                {
                    Error("Error 7254 \"QUERY REFERENCE MEASUREMENT FAILED answered after successful measurement\"");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_6()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;
            long t0, time;

            Message("Starting test 12.7.2.6: ENABLE / DISABLE CURRENT PROTECTOR");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);


            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x10) != 0x10))
            {
                Message("\"Current Protector not supported by DUT\"");
            }
            else
            {
                // enable device type 6, 2x broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xE0; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                t0 = DateTime.Now.Ticks;
                time = t0;

                do
                {
                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] == 0x00) || (incoming[8] != 0xFF))
                    {
                        break;
                    }

                    time = DateTime.Now.Ticks;
                } while (((time - t0) / 1e7) < 17 * 60);

                if (((time - t0) / 1e7) < 17 * 60)
                {
                    // enable device type 6, 2x broadcast disable current protector
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x03; // 3 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xE2; // Command 1 data
                    outgoing[9] = 0x00; // Command 2 flags
                    outgoing[10] = 0xFF; // Command 2 addr
                    outgoing[11] = 0xE2; // Command 2 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    MessageBox.Show("Interrupt mains supply for 5 seconds", "Required action");
                    Message("Mains supply interrupted for 5 seconds");

                    Info("Wait 5s");
                    Thread.Sleep(5000);

                    // flush input buffer
                    while (getData(incoming, 100, 0) != 0)
                        ;

                    // enable device type 6, broadcast query current protector enabled
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xFB; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] != 0x00)
                    {
                        Error("Error 7262 \"QUERY CURRENT PROTECTOR ENABLED answered wrong when disabled\"");
                    }

                    MessageBox.Show("Connect additional load", "Required action");
                    Message("Additional load connected");

                    Info("Wait 5s");
                    Thread.Sleep(5000);

                    // broadcast query lamp power on
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x93;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] == 0x00)
                        {
                            Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 1, x");
                            Error("Error 7263 \"CURRENT PROTECTOR not disabled\"");
                        }
                    }

                    // enable device type 6, 2x broadcast enable current protector
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x03; // 3 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xE1; // Command 1 data
                    outgoing[9] = 0x00; // Command 2 flags
                    outgoing[10] = 0xFF; // Command 2 addr
                    outgoing[11] = 0xE1; // Command 2 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    MessageBox.Show("Interrupt mains supply for 5 seconds", "Required action");
                    Message("Mains supply interrupted for 5 seconds");

                    Info("Wait 5s");
                    Thread.Sleep(5000);

                    // flush input buffer
                    while (getData(incoming, 100, 0) != 0)
                        ;


                    // enable device type 6, broadcast query current protector enabled
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xFB; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if (incoming[6] == 0x00)
                    {
                        Error("Error 7264 \"QUERY CURRENT PROTECTOR ENABLED answered wrong when enabled\"");
                    }

                    Info("Wait 5s");
                    Thread.Sleep(5000);

                    // broadcast query lamp power on
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x93;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x00)
                        {
                            Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expected = 0, 0");
                            Error("Error 7265 \"CURRENT PROTECTOR not enabled\"");
                        }
                    }

                    MessageBox.Show("Disconnect additional load", "Required action");
                    Message("Additional load disconnected");

                }
                else
                {
                    Error("Error 7261 Reference measurement timeout");
                }

            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_7()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] value = new Byte[5];
            Byte[] answer = new Byte[5];
            Byte[] curve = new Byte[5];
            String[] text = new String[5];

            int r, i, iMax;

            double phiMax, phi191;

            Func<string, double> method = GetData;
            IAsyncResult cookie;

            value[0] = 1;
            value[1] = 255;
            value[2] = 0;
            value[3] = 255;
            value[4] = 1;

            answer[0] = 0x10;
            answer[1] = 0x10;
            answer[2] = 0x00;
            answer[3] = 0x00;
            answer[4] = 0x10;

            // change ref 22
            curve[0] = 0x01;
            curve[1] = 0x01;
            curve[2] = 0x00;
            curve[3] = 0x00;
            curve[4] = 0x01;

            text[0] = "not changed";
            text[1] = "changed";
            text[2] = "not changed";
            text[3] = "changed";
            text[4] = "not changed";

            Message("Starting test 12.7.2.7: SELECT DIMMING CURVE");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // dtr = 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 1;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, broadcast select dimming curve
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xE3; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // change ref 21
            Message("Wait 150ms");
            Thread.Sleep(150);

            // enable device type 6, broadcast query operating mode
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xFC; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if ((incoming[6] == 0x00) || ((incoming[8] & 0x10) != 0x00))
            {
                Error("Error 7271 \"Dimming curve changed if command received only once\"");
            }

            i = 0;
            iMax = 4;

            for (; i <= iMax; i++)
            {
                // dtr = value[i]
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA3;
                outgoing[3] = value[i];
                sendData(outgoing);
                r = getData(incoming, 100);

                // enable device type 6, 2x broadcast select dimming curve
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE3; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xE3; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // enable device type 6, broadcast query operating mode
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xFC; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if ((incoming[6] == 0x00) || ((incoming[8] & 0x10) != answer[i]))
                {
                    Error("Error 7272," + i.ToString() + " \"Dimming curve " + text[i] + " in test step " + i.ToString() + "\"");
                }

                // enable device type 6, broadcast query dimming curve
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xEE; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                // change ref 22 (part 2)
                if ((incoming[6] == 0x00) || (incoming[8] != curve[i]))
                {
                    Error("Error 7273," + i.ToString() + " \"QUERY DIMMING CURVE answered wrong in test step " + i.ToString() + "\"");
                }

            }

            cookie = method.BeginInvoke("Measure relative light output of lamps and enter value in the input field", null, null);
            phiMax = method.EndInvoke(cookie);

            Message("Storing " + phiMax.ToString() + " as maximum");

            // broadcast dapc 191
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFE;
            outgoing[3] = 191;
            sendData(outgoing);
            r = getData(incoming, 100);

            Info("Wait Response Time");
            Thread.Sleep((int)preaheatTimeControl.Value);

            cookie = method.BeginInvoke("Measure relative light output of lamps and enter value in the input field", null, null);
            phi191 = method.EndInvoke(cookie);

            Message("Phi Relative = " + (phi191 / phiMax).ToString());

            if ((0.65 >= phi191 / phiMax) || (phi191 / phiMax >= 0.85))
            {
                Error("Error 7274 \"Dimming Curve is not linear\"");
            }

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // enable device type 6, broadcast query operating mode
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xFC; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if ((incoming[6] == 0x00) || ((incoming[8] & 0x10) == 0x10))
            {
                Error("Error 7275 \"RESET does not switch to standard dimming curve\"");
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_8()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] value = new Byte[29];
            Byte[] answer = new Byte[29];

            int[] tMin = new int[29];
            int[] tMax = new int[29];

            double tMeasured;
            Func<string, double> method = GetData;
            IAsyncResult cookie;

            int r, i, iMax;


            value[0] = 1;
            value[1] = 2;
            value[2] = 3;
            value[3] = 4;
            value[4] = 5;
            value[5] = 6;
            value[6] = 7;
            value[7] = 8;
            value[8] = 9;
            value[9] = 10;
            value[10] = 11;
            value[11] = 12;
            value[12] = 13;
            value[13] = 14;
            value[14] = 15;
            value[15] = 16;
            value[16] = 17;
            value[17] = 18;
            value[18] = 19;
            value[19] = 20;
            value[20] = 21;
            value[21] = 22;
            value[22] = 23;
            value[23] = 24;
            value[24] = 25;
            value[25] = 26;
            value[26] = 27;
            value[27] = 0;
            value[28] = 254;

            answer[0] = 1;
            answer[1] = 2;
            answer[2] = 3;
            answer[3] = 4;
            answer[4] = 5;
            answer[5] = 6;
            answer[6] = 7;
            answer[7] = 8;
            answer[8] = 9;
            answer[9] = 10;
            answer[10] = 11;
            answer[11] = 12;
            answer[12] = 13;
            answer[13] = 14;
            answer[14] = 15;
            answer[15] = 16;
            answer[16] = 17;
            answer[17] = 18;
            answer[18] = 19;
            answer[19] = 20;
            answer[20] = 21;
            answer[21] = 22;
            answer[22] = 23;
            answer[23] = 24;
            answer[24] = 25;
            answer[25] = 26;
            answer[26] = 27;
            answer[27] = 0;
            answer[28] = 27;

            tMin[0] = 13;
            tMin[1] = 38;
            tMin[2] = 63;
            tMin[3] = 88;
            tMin[4] = 113;
            tMin[5] = 138;
            tMin[6] = 163;
            tMin[7] = 188;
            tMin[8] = 213;
            tMin[9] = 238;
            tMin[10] = 263;
            tMin[11] = 288;
            tMin[12] = 313;
            tMin[13] = 338;
            tMin[14] = 363;
            tMin[15] = 388;
            tMin[16] = 413;
            tMin[17] = 438;
            tMin[18] = 463;
            tMin[19] = 488;
            tMin[20] = 513;
            tMin[21] = 538;
            tMin[22] = 563;
            tMin[23] = 588;
            tMin[24] = 613;
            tMin[25] = 638;
            tMin[26] = 663;
            tMin[27] = 0;
            tMin[28] = 663;

            tMax[0] = 37;
            tMax[1] = 62;
            tMax[2] = 87;
            tMax[3] = 112;
            tMax[4] = 137;
            tMax[5] = 162;
            tMax[6] = 187;
            tMax[7] = 212;
            tMax[8] = 237;
            tMax[9] = 262;
            tMax[10] = 287;
            tMax[11] = 312;
            tMax[12] = 337;
            tMax[13] = 362;
            tMax[14] = 387;
            tMax[15] = 412;
            tMax[16] = 437;
            tMax[17] = 462;
            tMax[18] = 487;
            tMax[19] = 512;
            tMax[20] = 537;
            tMax[21] = 562;
            tMax[22] = 587;
            tMax[23] = 612;
            tMax[24] = 637;
            tMax[25] = 662;
            tMax[26] = 687;
            tMax[27] = 26;
            tMax[28] = 687;


            Message("Starting test 12.7.2.8: FAST FADE TIME");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);


            // enable device type 6, broadcast query min fast fade time
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xFE; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if ((incoming[6] == 0x00) || (incoming[8] < 1) || (incoming[8] > 27))
            {
                Error("Error 7281 \"Min Fast Fade Time out of valid range\"");
            }
            else
            {
                Message("\"Min Fast Fade Time is " + incoming[8].ToString() + "\"");

                i = incoming[8] - 1;
                iMax = 28;

                for (; i <= iMax; i++)
                {
                    // dtr = value[i]
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA3;
                    outgoing[3] = value[i];
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // enable device type 6, 2x broadcast store dtr as fast fade time
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x03; // 3 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xE4; // Command 1 data
                    outgoing[9] = 0x00; // Command 2 flags
                    outgoing[10] = 0xFF; // Command 2 addr
                    outgoing[11] = 0xE4; // Command 2 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    // enable device type 6, broadcast query fast fade time
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xFD; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }
                    if ((incoming[6] == 0x00) || (incoming[8] != answer[i]))
                    {
                        Error("Error 7282," + i.ToString() + " \"Wrong Fast Fade Time stored in test step " + i.ToString() + "\"");
                    }

                    // broadcast dapc 191
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFE;
                    outgoing[3] = 191;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    Message("Wait Response Time");
                    Thread.Sleep((int)preaheatTimeControl.Value);

                    tMeasured = 0;

                    cookie = method.BeginInvoke("Measure Fast Fade Time t[" + i.ToString() + "] with oscilloscope and enter value in the input field", null, null);
                    tMeasured = method.EndInvoke(cookie);

                    Message("Measured t[" + i.ToString() + "] = " + tMeasured.ToString());

                    // change ref 25
                    // When FAST FADE TIME is set to 0, tMeasured could be 0. However, if the implementation requires that tMeasured is within
                    // (tMin, tMax) without being either tMin or tMax, you would get an error if tMeasured is 0 in this case.
                    if ((tMin[i] > tMeasured) || (tMeasured >= tMax[i]))
                    {
                        Error("Error 7283 \"Fast Fade Time out of limit\"");
                    }

                    // recall max level
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x05;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                }
            }

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // enable device type 6, broadcast query fast fade time
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xFD; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if ((incoming[6] == 0x00) || (incoming[8] != 0))
            {
                Error("Error 7284 \"RESET does not reset the Fast Fade Time\"");
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_2_9()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r;

            Message("Starting test 12.7.2.9: Reset State / Persistent Memory");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // dtr = 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 1;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, 2x broadcast select dimming curve
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xE3; // Command 1 data
            outgoing[9] = 0x00; // Command 1 flags
            outgoing[10] = 0xFF; // Command 1 addr
            outgoing[11] = 0xE3; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            MessageBox.Show("Disconnect mains", "Required action");
            Message("Mains disconnected");

            Info("Wait 5s");
            Thread.Sleep(5000);

            MessageBox.Show("Reconnect mains", "Required action");
            Message("Mains reconnected");

            Info("Wait 500ms");
            Thread.Sleep(500);

            // flush input buffer
            while (getData(incoming, 100, 0) != 0)
                ;


            // enable device type 6, broadcast query operating mode
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xFC; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if ((incoming[6] == 0x00) || ((incoming[8] & 0x10) != 0x10))
            {
                Error("Error 7291 \"Linear mode not stored in persitent memory\"");
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x20) != 0x00) // check if status != xx0xxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxxx");
                        Error("Error 7292 \"Reset State indicated in STATUS during linear mode\"");
                    }
                }
            }

            // query reset state
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x95;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expexted = 0, 0");
                    Error("Error 7293 \"QUERY RESET STATE answered with 'Yes' in linear mode\"");
                }
            }

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // dtr = 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 1;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, 2x broadcast store dtr as fast fade time
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x03; // 3 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xE4; // Command 1 data
            outgoing[9] = 0x00; // Command 2 flags
            outgoing[10] = 0xFF; // Command 2 addr
            outgoing[11] = 0xE4; // Command 2 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            MessageBox.Show("Disconnect mains", "Required action");
            Message("Mains disconnected");

            Info("Wait 5s");
            Thread.Sleep(5000);

            MessageBox.Show("Reconnect mains", "Required action");
            Message("Mains reconnected");

            Info("Wait 500ms");
            Thread.Sleep(500);

            // flush input buffer
            while (getData(incoming, 100, 0) != 0)
                ;


            // enable device type 6, broadcast query fast fade time
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xFD; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }
            if (incoming[6] == 0x00)
            {
                Error("No answer on DALI bus");
            }
            else if (incoming[8] == 0)
            {
                Error("Error 7294 \"FAST FADE TIME not stored in persitent memory\"");
            }

            // broadcast query status
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x90;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    if ((incoming[3] & 0x20) != 0x00) // check if status != xx0xxxxx
                    {
                        Debug("\tReceived answer = " + incoming[3].ToString() + " expected = (bin)xx0xxxxxx");
                        Error("Error 7295 \"Reset State indicated in STATUS with programmed FAST FADE TIME \"");
                    }
                }
            }

            // query reset state
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x95;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if ((incoming[2] == 0x01) && (incoming[3] == 0xFF))
                {
                    Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expexted = 0, 0");
                    Error("Error 7296 \"QUERY RESET STATE answered with 'Yes' with programmed FAST FADE TIME\"");
                }
            }


            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_3_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] address = new Byte[5];

            bool[] answerExpected = new bool[5];

            Byte[] level = new Byte[5];

            String[] errorText1 = new String[5];
            String[] errorText2 = new String[5];
            String[] commandText = new String[5];

            int r, i, iMax;
            Byte PHM = 0;

            address[0] = 0xFF;
            address[1] = 0x03;
            address[2] = 0x05;
            address[3] = 0x83;
            address[4] = 0x85;

            answerExpected[0] = false;
            answerExpected[1] = false;
            answerExpected[2] = true;
            answerExpected[3] = false;
            answerExpected[4] = true;

            errorText1[0] = "executed";
            errorText1[1] = "executed";
            errorText1[2] = "not executed";
            errorText1[3] = "executed";
            errorText1[4] = "not executed";

            errorText2[0] = "not executed";
            errorText2[1] = "not executed";
            errorText2[2] = "executed";
            errorText2[3] = "not executed";
            errorText2[4] = "executed";

            Message("Starting test 12.7.3.1:ENABLE DEVICE TYPE: Application extended commands");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x broadcast add to group 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x61; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x61; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // dtr = 3
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);


            // broadcast query phm
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x9A;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != 0x01)
                {
                    Error("No answer on DALI bus");
                }
                else
                {
                    PHM = incoming[3];
                }
            }

            Debug("PHM = " + PHM.ToString());

            level[0] = PHM;
            level[1] = PHM;
            level[2] = 254;
            level[3] = PHM;
            level[4] = 254;

            i = 0;
            iMax = 4;

            for (; i <= iMax; i++)
            {
                // enable device type 6, send <address[i]> recall min level, broadcast query features
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x03; // 3 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = address[i]; // Command 1 addr
                outgoing[8] = 0x06; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xFF; // Command 2 addr
                outgoing[11] = 0xF0; // Command 2 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if ((incoming[9] != 0x00) != answerExpected[i])
                {
                    Error("Error 7311," + i.ToString() + " \"Command 'QUERY FEATURES' " + errorText1[i] + "\"");
                }

                Info("Wait Response Time");
                Thread.Sleep((int)preaheatTimeControl.Value);

                // broadcast query actual level
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0xA0;
                sendData(outgoing);
                r = getData(incoming, 100);
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != 0x01)
                    {
                        Error("No answer on DALI bus");
                    }
                    else
                    {
                        if (incoming[3] != level[i])
                        {
                            Debug("\tReceived answer = " + incoming[3].ToString() + " expected = " + level[i].ToString());
                            Error("Error 7312," + i.ToString() + " \"Other command " + errorText2[i] + "\"");
                        }
                    }
                }

                // dtr = 255
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xFF;
                outgoing[3] = 0x05;
                sendData(outgoing);
                r = getData(incoming, 100);

            }

            // 2x broadcast remove from group 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x71; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x71; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Message("Test done");

        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_3_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte[] address = new Byte[5];

            bool[] answer = new bool[5];

            bool[] level0 = new bool[5];

            String[] errorText1 = new String[5];
            String[] errorText2 = new String[5];
            String[] commandText = new String[5];

            int r, i, iMax;

            address[0] = 0xFF;
            address[1] = 0x03;
            address[2] = 0x05;
            address[3] = 0x83;
            address[4] = 0x85;

            answer[0] = false;
            answer[1] = false;
            answer[2] = true;
            answer[3] = false;
            answer[4] = true;

            level0[0] = true;
            level0[1] = true;
            level0[2] = false;
            level0[3] = true;
            level0[4] = false;

            errorText1[0] = "executed";
            errorText1[1] = "executed";
            errorText1[2] = "not executed";
            errorText1[3] = "executed";
            errorText1[4] = "not executed";

            errorText2[0] = "not executed";
            errorText2[1] = "not executed";
            errorText2[2] = "executed";
            errorText2[3] = "not executed";
            errorText2[4] = "executed";

            Message("Starting test 12.7.3.2: ENABLE DEVICE TYPE: Application extended configuration commands 1");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // 2x broadcast add to group 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x61; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x61; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // dtr = 3
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 3;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            // enable device type 6, broadcast query features
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xC1; // Command 0 addr
            outgoing[5] = 0x06; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0xF0; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            if ((incoming[6] == 0x00) || ((incoming[8] & 0x1C) == 0x00))
            {
                Message("\"REFERENCE SYSTEM POWER not supported by DUT\"");
            }
            else
            {
                i = 0;
                iMax = 4;

                for (; i <= iMax; i++)
                {
                    // enable device type 6, send <address[i]> off, 2x broadcast reference system power
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x04; // 4 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = address[i]; // Command 1 addr
                    outgoing[8] = 0x00; // Command 1 data
                    outgoing[9] = 0x00; // Command 2 flags
                    outgoing[10] = 0xFF; // Command 2 addr
                    outgoing[11] = 0xE0; // Command 2 data
                    outgoing[12] = 0x00; // Command 3 flags
                    outgoing[13] = 0xFF; // Command 3 addr
                    outgoing[14] = 0xE0; // Command 3 data
                    sendData(outgoing);
                    r = getData(incoming, 300); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    // enable device type 6, broadcast query reference running
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x02; // 2 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = 0xF9; // Command 1 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] != 0x00) != answer[i])
                    {
                        Error("Error 7321," + i.ToString() + " \"Application extended configuration command " + errorText1[i] + "\"");
                    }

                    Info("Wait Response Time");
                    Thread.Sleep((int)preaheatTimeControl.Value);

                    // broadcast query actual level
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0xA0;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != 0x01)
                        {
                            Error("No answer on DALI bus");
                        }
                        else
                        {
                            if ((incoming[3] == 0) != level0[i])
                            {
                                Debug("\tReceived answer = " + incoming[3].ToString());
                                Error("Error 7322," + i.ToString() + " \"Other command " + errorText2[i] + "\"");
                            }
                        }
                    }

                    // terminate
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xA1;
                    outgoing[3] = 0x00;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // recall max level
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x05;
                    sendData(outgoing);
                    r = getData(incoming, 100);

                    // change ref 13
                    Info("Wait Preheat Time + Response Time");
                    Thread.Sleep((int)preaheatTimeControl.Value + (int)preaheatTimeControl.Value);
                }
            }

            // 2x broadcast remove from group 1
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x71; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x71; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            // dtr = 255
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA3;
            outgoing[3] = 255;
            sendData(outgoing);
            r = getData(incoming, 100);

            // 2x broadcast store dtr as short addr
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x80; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x80; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_3_3()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r, i, iMax;

            Message("Starting test 12.7.3.3: ENABLE DEVICE TYPE: Application extended configuration commands 2");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // terminate
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xA1;
            outgoing[3] = 0x00;
            sendData(outgoing);
            r = getData(incoming, 100);

            i = 0;
            iMax = 254;

            for (; i <= iMax; i++)
            {
                // enable device type 6, broadcast reference system power, enable device type i, broadcast reference system power
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x04; // 4 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xE0; // Command 1 data
                outgoing[9] = 0x00; // Command 2 flags
                outgoing[10] = 0xC1; // Command 2 addr
                outgoing[11] = (Byte)i; // Command 2 data
                outgoing[12] = 0x00; // Command 3 flags
                outgoing[13] = 0xFF; // Command 3 addr
                outgoing[14] = 0xE0; // Command 3 data
                sendData(outgoing);
                r = getData(incoming, 300); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                // enable device type 6, broadcast query reference running
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xF9; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] != 0x00)
                {
                    Error("Error 7331," + i.ToString() + " \"Application extended configuration command executed\"");
                }

                // terminate
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                outgoing[2] = 0xA1;
                outgoing[3] = 0x00;
                sendData(outgoing);
                r = getData(incoming, 100);
            }

            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_4_1()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            int r, i, iMax;

            Message("Starting test 12.7.4.1: QUERY EXTENDED VERSION NUMBER");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            i = 0;
            iMax = 254;

            for (; i <= iMax; i++)
            {
                // enable device type i, broadcast query extended version number
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = (Byte)i; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xFF; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }

                if (incoming[6] != 0x00)
                {
                    Message("\"EXTENDED VERSION NUMBER is " + incoming[8].ToString() + " for device type " + i.ToString() + "\"");
                }
                else if (i == 6)
                {
                    Error("Error 7611 \"No answer to QUERY EXTENDED VERSION NUMBER\"");
                }
            }
            Message("Test done");
        }

        // Run test according to IEC 62386-207
        public void RunTest_12_7_4_2()
        {
            Byte[] outgoing = new Byte[65];
            Byte[] incoming = new Byte[65];

            Byte command;
            int r, k, kMax;
            bool pass;

            Message("Starting test 12.7.4.2: RESERVED APPLICATION EXTENDED COMMANDS");

            // 2x broadcast reset
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
            outgoing[2] = 0x02; // 2 commands
            outgoing[3] = 0x00; // Command 0 flags
            outgoing[4] = 0xFF; // Command 0 addr
            outgoing[5] = 0x20; // Command 0 data
            outgoing[6] = 0x00; // Command 1 flags
            outgoing[7] = 0xFF; // Command 1 addr
            outgoing[8] = 0x20; // Command 1 data
            sendData(outgoing);
            r = getData(incoming, 200); // more time allowed for the sequence to finish
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] != outgoing[2])
                {
                    Error("Command sequence error");
                }
            }


            Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
            Thread.Sleep((int)preaheatTimeControl.Value + 300);

            // broadcast query device type
            outgoing[0] = 0x00;
            outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
            outgoing[2] = 0xFF;
            outgoing[3] = 0x99;
            sendData(outgoing);
            r = getData(incoming, 100);
            if (r > 0)
            {
                if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                {
                    Error("Wrong USB packet type received");
                }
                else if (incoming[2] == 0x00)
                {
                    Error("No answer on DALI bus");
                }
            }

            pass = false;

            if ((incoming[2] != 0x00) && (incoming[3] == 6))
            {
                pass = true;
            }
            else if ((incoming[2] != 0x00) && (incoming[3] == 255))
            {
                // enable device type 6, query extended version number
                outgoing[0] = 0x00;
                outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                outgoing[2] = 0x02; // 2 commands
                outgoing[3] = 0x00; // Command 0 flags
                outgoing[4] = 0xC1; // Command 0 addr
                outgoing[5] = 0x06; // Command 0 data
                outgoing[6] = 0x00; // Command 1 flags
                outgoing[7] = 0xFF; // Command 1 addr
                outgoing[8] = 0xFF; // Command 1 data
                sendData(outgoing);
                r = getData(incoming, 200); // more time allowed for the sequence to finish
                if (r > 0)
                {
                    if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                    {
                        Error("Wrong USB packet type received");
                    }
                    else if (incoming[2] != outgoing[2])
                    {
                        Error("Command sequence error");
                    }
                }
                if (incoming[6] != 0x00)
                {
                    pass = true;
                }
            }

            if (!pass)
            {
                Message("\"DEVICE TYPE is not 6! Test of APPLICATION EXTENDED COMMANDS not possible with this test sequence!\"");
            }
            else
            {
                k = 0;
                kMax = 7;

                for (; k <= kMax; k++)
                {
                    command = (Byte)(229 + k);

                    // enable device type 6, 2x broadcast command
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                    outgoing[2] = 0x03; // 3 commands
                    outgoing[3] = 0x00; // Command 0 flags
                    outgoing[4] = 0xC1; // Command 0 addr
                    outgoing[5] = 0x06; // Command 0 data
                    outgoing[6] = 0x00; // Command 1 flags
                    outgoing[7] = 0xFF; // Command 1 addr
                    outgoing[8] = command; // Command 1 data
                    outgoing[9] = 0x00; // Command 2 flags
                    outgoing[10] = 0xFF; // Command 2 addr
                    outgoing[11] = command; // Command 2 data
                    sendData(outgoing);
                    r = getData(incoming, 200); // more time allowed for the sequence to finish
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                        {
                            Error("Wrong USB packet type received");
                        }
                        else if (incoming[2] != outgoing[2])
                        {
                            Error("Command sequence error");
                        }
                    }

                    if ((incoming[6] != 0x00) || (incoming[9] != 0x00))
                    {
                        Error("Error 7621," + k.ToString() + " \"Answer after receiving APPLICATION EXTENDED COMMAND " + command.ToString() + "\"");
                    }

                    // query reset state
                    outgoing[0] = 0x00;
                    outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND;
                    outgoing[2] = 0xFF;
                    outgoing[3] = 0x95;
                    sendData(outgoing);
                    r = getData(incoming, 100);
                    if (r > 0)
                    {
                        if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_ANSWER)
                        {
                            Error("Wrong USB packet type received");
                        }
                    }
                    else if ((incoming[2] == 0x00) || (incoming[3] != 0xFF))
                    {
                        Debug("\tReceived answers = " + incoming[2].ToString() + ", " + incoming[3].ToString() + " expexted = 1, 255");
                        Error("Error 7622," + k.ToString() + " \"DUT not in reset state after receiving APPLICATION EXTENDED COMMAND " + command.ToString() + "\"");

                        // 2x broadcast reset
                        outgoing[0] = 0x00;
                        outgoing[1] = (Byte)USB_PACKETS.TYPE_DALI_COMMAND_SEQUENCE; // Command sequence
                        outgoing[2] = 0x02; // 2 commands
                        outgoing[3] = 0x00; // Command 0 flags
                        outgoing[4] = 0xFF; // Command 0 addr
                        outgoing[5] = 0x20; // Command 0 data
                        outgoing[6] = 0x00; // Command 1 flags
                        outgoing[7] = 0xFF; // Command 1 addr
                        outgoing[8] = 0x20; // Command 1 data
                        sendData(outgoing);
                        r = getData(incoming, 200); // more time allowed for the sequence to finish
                        if (r > 0)
                        {
                            if (incoming[1] != (Byte)USB_REPLIES.REPLY_DALI_COMMAND_SEQUENCE)
                            {
                                Error("Wrong USB packet type received");
                            }
                            else if (incoming[2] != outgoing[2])
                            {
                                Error("Command sequence error");
                            }
                        }

                        Info("Wait Response Time + 300ms = " + (preaheatTimeControl.Value + 300).ToString() + "ms");
                        Thread.Sleep((int)preaheatTimeControl.Value + 300);
                    }
                }
            }
            Message("Test done");
        }
    }
}
